/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2008 by Sander Jansen. All Rights Reserved           *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#ifndef GMAUDIOSCROBBLER_H
#define GMAUDIOSCROBBLER_H

struct GMAudioScrobblerTrack {
  FXString artist;
  FXString album;
  FXString title;
  FXuint   duration;
  FXint    no;
  FXlong   timestamp;
  GMAudioScrobblerTrack(){}
  GMAudioScrobblerTrack(FXlong time,GMTrack & t) : artist(t.artist),album(t.album),title(t.title),duration(t.time),no(t.no),timestamp(time){}

  void load(FXStream & store);
  void save(FXStream & store) const;
  };

typedef FXArray<GMAudioScrobblerTrack> TrackQueue;

class GMAudioScrobbler : public FXObject {
FXDECLARE(GMAudioScrobbler)
protected:
  FXApp * app;
  FXuint  state;
  FXuint  disconnect;
  FXint   server;
  FXint   handshake_failed_timeout;
  FXint   failed;
  FXbool  try_handshake;
  FXDict  dnscache;
protected: // Set by Server
  FXint    submit_npayload;
  FXString submit_payload;
  FXString session;
  FXString submit_url;
  FXString nowplaying_url;  
protected: // Set by User
  GMAudioScrobblerTrack npt;
  TrackQueue queue;
  FXString   username;
  FXString   password;
protected:
  GMAudioScrobbler();
private:
  GMAudioScrobbler(const GMAudioScrobbler&);
  GMAudioScrobbler& operator=(const GMAudioScrobbler&);
protected:
  void process_handshake_response(const FXchar * buffer,FXint len);
  void process_submit_response(const FXchar * buffer,FXint len);
  void process_nowplaying_response(const FXchar * buffer,FXint len);
  void start_handshake();
  void start_submit();
  void start_nowplaying();
  void clear_submit_payload();
  void clear_nowplaying_payload();  
  void create_submit_payload();
  void create_submit_request(FXString & msg);
  void create_handshake_request(FXString & msg);
  void create_nowplaying_request(FXString & msg);
  void open_connection(const FXString & hostname,FXint port,FXuint message);
  void close_connection();
  void set_permanent_failure(FXuint s);
  void set_failed();
  void reschedule_submit();
  void load_queue();
  void save_queue();
protected:

  enum {
    DISCONNECT_NONE=0,
    DISCONNECT_SHUTDOWN,
    DISCONNECT_LOGIN
    };
public:
  enum {
    STATE_DISCONNECTED=0,     /// Doing nothing
    STATE_CONNECTING, /// Waiting for open connection
    STATE_CONNECTED,  /// Waiting for server response
    STATE_BANNED,     /// Permanent Failure
    STATE_BADAUTH,    /// Login didn't work
    };
  enum {
    ID_HANDSHAKE = 1,
    ID_SUBMIT,
    ID_NOWPLAYING,
    ID_NEXT_TASK
    };
public:
  long onNextTask(FXObject*,FXSelector,void*);
  long onHandshake(FXObject*,FXSelector,void*); 
  long onSubmit(FXObject*,FXSelector,void*);
  long onNowPlaying(FXObject*,FXSelector,void*);
public:
  GMAudioScrobbler(FXApp *);

  /// Set Username and Password
  void setLogin(const FXString & username,const FXString & password);

  /// Get Username
  FXString getUsername() const { return username; }

  /// Get Password
  FXString getPassword() const { return password; }

  /// Submit track
  void submit(FXlong time,GMTrack & track);

  /// Now Playing Track
  void nowplaying(GMTrack & track);

  /// Clears the queue
  void clear();

  /// Returns Queue Length
  FXint getQueueLength() const;

  /// Shutdown the scrobbler
  void shutdown();

  const FXchar * getStatus(FXColor &c) const;

  FXbool isBanned() const;

  FXbool isEnabled() const;

  FXbool isLoginFailed() const;

  ~GMAudioScrobbler();
  };

#endif



