/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#ifndef GMSEARCHDIALOG_H
#define GMSEARCHDIALOG_H

class GMTrackDatabase;
class GMThread;

struct GMSearchProgress {
	FXString filename;
	FXString directory;
  FXint count;
  GMSearchProgress();

  void load(FXStream & store);
  void save(FXStream & store) const;
  };

class GMSearchThread : public GMThread {
protected:
	GMPreferences options;
	FXStringList files;
  FXString path;
	GMSearchProgress progress;
	GMTrackDatabase  database;
  FXint playlist;
  int lastid;
protected:
  FXint run();
	void listDirectory(const FXString & path);
  void insertFile(const FXString & searchpath,const FXString & file,FXint num);
public:
	GMSearchThread(const FXStringList & list,FXint playlist,const GMPreferences & pref,FXObject*);

  void sendProgress();

  int getLastInserted() const;

	virtual ~GMSearchThread();
	};


class GMAnimImage;

class GMSearchDialog : public FXDialogBox {
  FXDECLARE(GMSearchDialog)
private:
	FXTextField * text_file;
	FXTextField * text_dir;
	FXTextField * text_count;
  GMAnimImage * animation;
  FXImage     * animation_image;
private:
	GMSearchThread * searchthread;
  void setup(FXFont * font);
private:
  GMSearchDialog(){}
  GMSearchDialog(const GMSearchDialog&);
  GMSearchDialog& operator=(const GMSearchDialog&);
public:
	enum {
		ID_SELECT_DIR = FXDialogBox::ID_LAST,
    ID_SEARCH_DONE,
    ID_SEARCH_ERROR,
    ID_SEARCH_PROGRESS,
    ID_SEARCH_DIRECTORY,
		};
public:
	long onCmdCancel(FXObject*,FXSelector,void*);
  long onSearchDone(FXObject*,FXSelector,void*);
  long onSearchError(FXObject*,FXSelector,void*);
  long onSearchProgress(FXObject*,FXSelector,void*);
  long onSearchDirectory(FXObject*,FXSelector,void*);
public:
	/// Construct free-floating About dialog
  GMSearchDialog(FXApp* a,const FXStringList & list,FXint playlist,const GMPreferences & pref,FXFont * icon=NULL);

  /// Construct dialog which will always float over the owner window
  GMSearchDialog(FXWindow* owner,const FXStringList & list,FXint playlist,const GMPreferences & pref,FXFont * icon=NULL);

  /// Run modal invocation of the dialog
  virtual FXuint execute(FXuint placement=PLACEMENT_CURSOR);

  /// Get Last Track Inserted
  int getLastInserted() const;

  /// Destructor
  virtual ~GMSearchDialog();
  };

#endif
