/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#ifndef GMTRACKLIST_H
#define GMTRACKLIST_H

enum {
  COLUMN_JUSTIFY_NORMAL=0,
  COLUMN_JUSTIFY_CENTER_RIGHT_ALIGNED,
  COLUMN_JUSTIFY_LEFT_RIGHT_ALIGNED,
  };

class GMTrackList;

class GMTrackItem : public FXIconItem {
  FXDECLARE(GMTrackItem)
friend class GMTrackList;
public:
  static FXint max_time;
protected:
  void draw(const GMTrackList* list,FXDC& dc,FXint index,FXint x,FXint y,FXint w,FXint h,FXint dw) const;
  virtual FXint hitItem(const FXIconList* list,FXint rx,FXint ry,FXint rw=1,FXint rh=1) const;
protected:
  virtual const FXString * getColumnData(FXint,FXString &,FXuint &,FXint &) const { return NULL; }
public:
  GMTrackItem();
  FXint getTrackId() const { return (FXint)(FXival)data; }
  virtual FXint getWidth(const FXIconList*) const { return 0; }
  virtual FXint getHeight(const FXIconList*) const { return 0; }
  virtual ~GMTrackItem();
  };

class GMDBTrackItem;

class GMTrackList : public FXIconList {
FXDECLARE(GMTrackList)
friend class GMTrackItem;
private:
  FXColor     rowColor;
  FXColor     activeColor;
  FXColor     activeTextColor;
  FXIcon    * selectedIcon;
  FXIcon    * activeIcon;
  FXFont    * activeFont;
  FXint       activeItem;
  FXint       sortMethod;
protected:
  GMTrackList();
  virtual FXIconItem * createItem(const FXString& text,FXIcon *big,FXIcon* mini,void* ptr);
  void measure();
private:
  GMTrackList(const GMTrackList&);
  GMTrackList &operator=(const GMTrackList&);
public:
  long onPaint(FXObject*,FXSelector,void*);
  long onCmdHeaderClicked(FXObject*,FXSelector,void*);
  long onCmdHeader(FXObject*,FXSelector,void*);
  long onUpdHeader(FXObject*,FXSelector,void*);
  long onCmdHeaderRightBtnRelease(FXObject*,FXSelector,void*);
public:
  GMTrackList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  void appendHeader(const FXString & label,FXint size,GMColumn * data);

  void clearHeaders();

  void saveHeaders();

  GMColumn * getHeaderData(FXint i) const { return reinterpret_cast<GMColumn*>(header->getItemData(i)); }

  virtual void create();

  virtual FXint getContentWidth();

  virtual FXint getContentHeight();

  void showActiveItem() { if (activeItem>=0) makeItemVisible(activeItem); }

  FXint getActiveItem() const { return activeItem; }

  void setActiveItem(FXint i) { activeItem=i; if (getNumItems()) { if (i>=0) { makeItemVisible(i); update(); } else { update(); } }}

  FXColor getRowColor() const { return rowColor; }

  void setRowColor(FXColor c) { rowColor=c; update(); }

  FXColor getActiveColor() const { return activeColor; }

  void setActiveColor(FXColor c);

  FXColor getActiveTextColor() const { return activeTextColor; }

  void setActiveTextColor(FXColor c) { activeTextColor=c; update(); }

  void setActiveFont(FXFont *f) { activeFont=f; update(); }

  void setSortMethod(FXint m) { sortMethod=m; }

  FXint getSortMethod() const { return sortMethod; }

  void setUnsorted();

  virtual ~GMTrackList();

  /// Get the track id
  FXint getTrackId(FXint index) const { return (FXint)(FXival)(items[index]->getData()); }

  };
















#if 0

class GMTrackList : public FXIconList{
FXDECLARE(GMTrackList)
friend class GMTrackItemBase;
private:
  FXColor rowColor;
  FXColor activeColor;
  FXIcon* activeIcon;
  FXIcon* activeRowIcon;


  FXColor  rowcolor;
  FXColor  playingcolor;
  FXColor  playtextcolor;
  FXIcon* icon_playing;
  FXIcon* icon_playing_white;
  FXIconItem* active;
  FXbool   sort_browse;
  FXbool   sort_reverse;
  FXint    sort_column;
  FXuint   sort_seed;


  GMColumnInfo columns[7];

//  FXbool   columns_show[7];
//  FXuint   columns_width[7];


  FXint    max_no;
  FXint    digits_no;
  FXint    digits_count;
  FXint    max_trackwidth;
  FXint    max_trackcount;
  FXint    max_tracktime;
  FXint    lw,rx,rw;

  static FXuint random_seed;
  static FXbool reverse_artist;
  static FXbool reverse_album;
protected:
  FXint index;
protected:
  void loadColumns(const FXString & key,FXbool queuemode);
  void saveColumns(const FXString & key);
  void saveColumnSizes();
  void showColumns();
public:
  static FXint defaultSort(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingTrack(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingTrack(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingQueue(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingQueue(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingTitle(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingTitle(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingAlbum(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingAlbum(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingArtist(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingArtist(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingTime(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingTime(const FXIconItem* pa,const FXIconItem* pb);
  static FXint ascendingGenre(const FXIconItem* pa,const FXIconItem* pb);
  static FXint descendingGenre(const FXIconItem* pa,const FXIconItem* pb);
public:
  enum {
    ID_REFRESHTIMER=FXIconList::ID_LAST,
    ID_OPENTIMER,
    ID_DETAILED,
    ID_SORT_NORMAL,
    ID_SORT_BY_TRACK,
    ID_SORT_BY_TITLE,
    ID_SORT_BY_TIME,
    ID_SORT_BY_ALBUM,
    ID_SORT_BY_ARTIST,
    ID_SORT_BY_GENRE,
    ID_SORT_BY_QUEUE,
    ID_SORT_RANDOM,
    ID_SORT_REVERSE,
    ID_SORT_CUSTOM,
    ID_COLUMN_TRACK,
    ID_COLUMN_NO,
    ID_COLUMN_TITLE,
    ID_COLUMN_TIME,
    ID_COLUMN_ALBUM,
    ID_COLUMN_ARTIST,
    ID_COLUMN_GENRE,
    ID_LAST
    };
public:
  long onPaint(FXObject*,FXSelector,void*);
  long onCmdSortByTrack(FXObject*,FXSelector,void*);
  long onCmdSortByTitle(FXObject*,FXSelector,void*);
  long onCmdSortByAlbum(FXObject*,FXSelector,void*);
  long onCmdSortByArtist(FXObject*,FXSelector,void*);
  long onCmdSortByTime(FXObject*,FXSelector,void*);
  long onCmdSortByGenre(FXObject*,FXSelector,void*);
  long onUpdSortByTrack(FXObject*,FXSelector,void*);
  long onUpdSortByTitle(FXObject*,FXSelector,void*);
  long onUpdSortByAlbum(FXObject*,FXSelector,void*);
  long onUpdSortByArtist(FXObject*,FXSelector,void*);
  long onUpdSortByTime(FXObject*,FXSelector,void*);
  long onUpdSortByGenre(FXObject*,FXSelector,void*);
  long onCmdSortByQueue(FXObject*,FXSelector,void*);
  long onUpdSortByQueue(FXObject*,FXSelector,void*);
  long onCmdSortReverse(FXObject*,FXSelector,void*);
  long onUpdSortReverse(FXObject*,FXSelector,void*);
  long onCmdSortRandom(FXObject*,FXSelector,void*);
  long onUpdSortRandom(FXObject*,FXSelector,void*);
  long onCmdSortNormal(FXObject*,FXSelector,void*);
  long onUpdSortNormal(FXObject*,FXSelector,void*);
  long onCmdHeader(FXObject*,FXSelector,void*);
  long onUpdHeader(FXObject*,FXSelector,void*);
  long onCmdColumn(FXObject*,FXSelector,void*);
  long onUpdColumn(FXObject*,FXSelector,void*);
  long onUpdSortCustom(FXObject*,FXSelector,void*);
  long onUpdDetailed(FXObject*,FXSelector,void*);

protected:
  GMTrackList();
private:
  GMTrackList(const GMTrackList&);
  GMTrackList &operator=(const GMTrackList&);
public:
  GMTrackList(FXComposite *p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

  void appendItem(FXint id,const FXchar * title,const FXchar * artist,const FXchar * album,const FXchar * genre,FXint time,FXint no,FXint queue);

  /// Active Track
  void clearActive() { active=NULL; recalc(); }

  /// Set the active track
  void setActive(FXint i) { if (i>=0) { active=getItem(i); makeItemVisible(i); } else {active=NULL;}  recalc(); }

  /// Get Active Index
  FXint getActiveIndex() const;

  /// Show Active
  void showActive();

  const GMTrackItem * getTrackItem(FXint index) const { return dynamic_cast<GMTrackItem*>(items[index]); }

  /// Get the track id
  FXint getTrackId(FXint index) const { return (FXint)(FXival)(items[index]->getData()); }


/*
  void updateQueue();

  /// Return Item Track Id
  FXint getItemTrackQueue(FXint i) const;

  /// Return Item Track Id
  FXint getItemTrackId(FXint i) const;

  /// Return Track Number
  FXint getItemTrackNumber(FXint i) const;

  /// Return Track Title
  FXString getItemTrackTitle(FXint i) const;

  /// Load Column Settings for given key
  void loadSettings(const FXString & key,FXint defaultheader,FXbool browsemode,FXbool queumode);

  /// Save Column Settings for given key
  void saveSettings(const FXString & key);


  void setUnsorted();

  FXint getSortColumn() const { return sort_column; }


  void showActiveItem();

  void setReverseArtist(FXbool yesno);
  void setReverseAlbum(FXbool yesno);

  FXbool isDefaultSort() const;

  void doRandom();
  void doSortItems();

*/



  virtual ~GMTrackList();
  };

#endif
#endif
