/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include "common.h"
#include "GMList.h"
#include "GMDatabase.h"
#include "GMHeaderButton.h"
#include "GMTrackDatabase.h"
#include "GMTrackList.h"
#include "GMTrackItem.h"
#include "GMTrackView.h"
#include "GMWindow.h"
#include "GMSource.h"
#include "GMSourceView.h"
#include "GMClipboard.h"
#include "GMDatabaseSource.h"
#include "GMPlayListSource.h"
#include "GMPlayerManager.h"
#include "GMTag.h"
#include "GMIconTheme.h"
#include "GMFilename.h"
#include "GMThread.h"
#include "GMSearch.h"
#include "GMCodecNames.h"
#include "GMAudioScrobbler.h"

void tracklist2urilist(GMTrackDatabase * db,FXString & url,FXIntList & list){
  url=FXURL::fileToURL(db->getTrackFilename(list[0]));
  for (FXint i=1;i<list.no();i++){
    url+="\r\n";
    url+=FXURL::fileToURL(db->getTrackFilename(list[i]));
    }
  }


void tracklist2gnomeclipboard(GMTrackDatabase * db,FXString & url,FXIntList & list){
  url="copy\n" + FXURL::fileToURL(db->getTrackFilename(list[0]));
  for (FXint i=1;i<list.no();i++){
    url+="\r\n";
    url+=FXURL::fileToURL(db->getTrackFilename(list[i]));
    }
  }

void gnomeclipboard2filelist(FXString & files,FXStringList & filelist){
  FXint begin,end;
  FXString file;
  for(begin=0;begin<files.length();begin=end+1){
    end=files.find_first_of("\r\n",begin);
    if (end<0) end = files.length();
    if (begin) {
      file = FXURL::decode(FXURL::fileFromURL(files.mid(begin,end-begin)));
      if (!file.empty()) filelist.append(file);
      }
    }
  }

void uri2filelist(FXString & files,FXStringList & filelist){
  FXint begin,end;
  FXString file;
  for(begin=0;begin<files.length();begin=end+2){
    end=files.find_first_of("\r\n",begin);
    if (end<0) end = files.length();
    file = FXURL::decode(FXURL::fileFromURL(files.mid(begin,end-begin)));
    if (!file.empty()) filelist.append(file);
    }
  }


FXbool GMDatabaseClipboardData::request(FXDragType target,GMClipboard * clipboard){
  if (target==GMClipboard::urilistType){
    FXString url;
    tracklist2urilist(db,url,tracks);
    clipboard->setDNDData(FROM_CLIPBOARD,target,url);
    return true;
    }
  else if (target==GMClipboard::kdeclipboard){
    clipboard->setDNDData(FROM_CLIPBOARD,target,"0");
    return true;
    }
  else if (target==GMClipboard::gnomeclipboard){
    FXString url;
    tracklist2gnomeclipboard(db,url,tracks);
    clipboard->setDNDData(FROM_CLIPBOARD,target,url);
    return true;
    }
  return false;
  }



FXDEFMAP(GMDatabaseSource) GMDatabaseSourceMap[]={
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EDIT_GENRE,GMDatabaseSource::onCmdEditGenre),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EDIT_ARTIST,GMDatabaseSource::onCmdEditArtist),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EDIT_ALBUM,GMDatabaseSource::onCmdEditAlbum),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EDIT_TRACK,GMDatabaseSource::onCmdEditTrack),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_DELETE_GENRE,GMDatabaseSource::onCmdDelete),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_DELETE_ARTIST,GMDatabaseSource::onCmdDelete),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_DELETE_ALBUM,GMDatabaseSource::onCmdDelete),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_DELETE_TRACK,GMDatabaseSource::onCmdDelete),

  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EXPORT_GENRE,GMDatabaseSource::onCmdExportTracks),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EXPORT_ARTIST,GMDatabaseSource::onCmdExportTracks),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EXPORT_ALBUM,GMDatabaseSource::onCmdExportTracks),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EXPORT_TRACK,GMDatabaseSource::onCmdExportTracks),

  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_COPY_ARTIST,GMDatabaseSource::onCmdCopyArtistAlbum),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_COPY_ALBUM,GMDatabaseSource::onCmdCopyArtistAlbum),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_COPY_TRACK,GMDatabaseSource::onCmdCopyTrack),
  FXMAPFUNC(SEL_DND_REQUEST,GMDatabaseSource::ID_COPY_ARTIST,GMDatabaseSource::onCmdRequestArtistAlbum),
  FXMAPFUNC(SEL_DND_REQUEST,GMDatabaseSource::ID_COPY_ALBUM,GMDatabaseSource::onCmdRequestArtistAlbum),
  FXMAPFUNC(SEL_DND_REQUEST,GMDatabaseSource::ID_COPY_TRACK,GMDatabaseSource::onCmdRequestTrack),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_EXPORT,GMDatabaseSource::onCmdExport),
  FXMAPFUNC(SEL_UPDATE,GMDatabaseSource::ID_EXPORT,GMDatabaseSource::onUpdExport),

  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_NEW_PLAYLIST,GMDatabaseSource::onCmdNewPlayList),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_CLEAR,GMDatabaseSource::onCmdClear),
  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_INFO,GMDatabaseSource::onCmdInfo),
  FXMAPFUNC(SEL_DND_DROP,GMDatabaseSource::ID_DROP,GMDatabaseSource::onCmdDrop),

  FXMAPFUNC(SEL_COMMAND,GMDatabaseSource::ID_PASTE,GMDatabaseSource::onCmdPaste),
  FXMAPFUNC(SEL_UPDATE,GMDatabaseSource::ID_PASTE,GMDatabaseSource::onUpdPaste),
  FXMAPFUNC(SEL_TIMEOUT,GMDatabaseSource::ID_TRACK_PLAYED,GMDatabaseSource::onCmdTrackPlayed),
  };

FXIMPLEMENT(GMDatabaseSource,GMSource,GMDatabaseSourceMap,ARRAYNUMBER(GMDatabaseSourceMap));

GMDatabaseSource* GMDatabaseSource::filterowner=NULL;

GMDatabaseSource::GMDatabaseSource() : dbowned(true), db(NULL),filtermode(0) {
  sort_browse=GMDBTrackItem::browseSort;
  }

GMDatabaseSource::GMDatabaseSource(GMTrackDatabase * database) : dbowned(true), db(database),filtermode(0)  {
  FXASSERT(db);
  sort_browse=GMDBTrackItem::browseSort;
  }

GMDatabaseSource::~GMDatabaseSource() {
  clearAlbumIconCache();
  if (dbowned) delete db;
  }


#include <stdlib.h> // need this for rand()

//generates a psuedo-random integer between min and max
int randint(int min, int max,unsigned int * random_seed)
{
  return min+int( ((double)(max-min+1))*rand_r(random_seed)/(RAND_MAX+1.0));
}


void GMDatabaseSource::shuffle(GMTrackList*list,FXuint sort_seed) const {
  list->setSortFunc(GMDBTrackItem::ascendingAlbum);
  list->sortItems();
  list->setSortFunc(NULL);

  /// Initial Value comes from sort_seed (read from registry and such...)
  FXuint random_seed = sort_seed;
  rand_r(&random_seed);

  FXint n;
  FXint nitems=list->getNumItems()-1;
  for (FXint i=0;i<nitems;i++){
    n=randint(i,nitems,&random_seed);
    FXASSERT(n<list->getNumItems());
    list->moveItem(i,n);
    }
  }



void GMDatabaseSource::configure(GMColumnList& list) const {
  list.no(8);
  list[0]=GMColumn("No",HEADER_TRACK,GMDBTrackItem::ascendingTrack,GMDBTrackItem::descendingTrack,43,(getPlayList()==-1) ? true : false ,true,0);
  list[1]=GMColumn("Queue",HEADER_QUEUE,GMDBTrackItem::ascendingQueue,GMDBTrackItem::descendingQueue,60,(getPlayList()==-1) ? false : true,false,1);
  list[2]=GMColumn("Title",HEADER_TITLE,GMDBTrackItem::ascendingTitle,GMDBTrackItem::descendingTitle,360,true,true,2);
  list[3]=GMColumn("Artist",HEADER_ARTIST,GMDBTrackItem::ascendingArtist,GMDBTrackItem::descendingArtist,400,true,false,3);
  list[4]=GMColumn("Album",HEADER_ALBUM,GMDBTrackItem::ascendingAlbum,GMDBTrackItem::descendingAlbum,200,true,false,4);
  list[5]=GMColumn("Genre",HEADER_GENRE,GMDBTrackItem::ascendingGenre,GMDBTrackItem::descendingGenre,200,true,false,5);
  list[6]=GMColumn("Year",HEADER_YEAR,GMDBTrackItem::ascendingYear,GMDBTrackItem::descendingYear,60,true,true,6);
  list[7]=GMColumn("Time",HEADER_TIME,GMDBTrackItem::ascendingTime,GMDBTrackItem::descendingTime,60,true,true,7);
  }


FXbool GMDatabaseSource::hasCurrentTrack(GMSource * src) const {
  if (src==this) return true;
  return false;
  }

FXbool GMDatabaseSource::findCurrent(GMTrackList * list,GMSource * src) {
  GMDatabaseSource * db = dynamic_cast<GMDatabaseSource*>(src);
  if (db && db->getCurrentTrack()!=-1 )
    return GMSource::findCurrent(list,db);
  return false;
  }


FXbool GMDatabaseSource::findCurrentArtist(GMList * list,GMSource * src) {
  GMDatabaseSource * dbs = dynamic_cast<GMDatabaseSource*>(src);
  if (dbs && dbs->getCurrentTrack()!=-1 ){
    FXint artist,album;
    db->getTrackAssociation(dbs->getCurrentTrack(),artist,album);
    if (artist==-1) return false;
    for (FXint i=0;i<list->getNumItems();i++){
      if (artist==(FXint)(FXival)list->getItemData(i)){
        list->selectItem(i);
        list->makeItemVisible(i);
        return true;
        }
      }
    }
  return false;
  }


FXbool GMDatabaseSource::findCurrentAlbum(GMList * list,GMSource * src) {
  GMDatabaseSource * dbs = dynamic_cast<GMDatabaseSource*>(src);
  if (dbs && dbs->getCurrentTrack()!=-1 ){
    FXint artist=-1,album=-1,i,j;
    GMAlbumListItem * item=NULL;
    db->getTrackAssociation(dbs->getCurrentTrack(),artist,album);
    if (album==-1) return false;
    for (i=0;i<list->getNumItems();i++){
      item = dynamic_cast<GMAlbumListItem*>(list->getItem(i));
      if (!item) continue;
      if (album==(FXint)(FXival)list->getItemData(i)){
        list->selectItem(i);
        list->makeItemVisible(i);
        return true;
        }
      for (j=0;j<item->albums.no();j++){
        if (album==item->albums[j]){
          list->selectItem(i);
          list->makeItemVisible(i);
          return true;
          }
        }
      }
    }
  return false;
  }


extern const char *albumartfiles[]={"cover","album","albumart",".folder","folder",NULL};

FXIcon * GMDatabaseSource::getAlbumIcon(FXint id,FXbool cacheonly){
  FXIconSource src(FXApp::instance());
  FXIcon * albumicon=NULL;
  FXint pathid;
  FXString name;
  pathid = db->getAlbumPath(id);
  if (pathid>=0){
    albumicon = (FXIcon*)albumicons.find((void*)(FXival)pathid);
    if (!albumicon && !cacheonly) {
      name = db->getPathName(pathid);
      FXString * files=NULL;
      FXString * names=NULL;
      FXint i,c;
      FXint nfound = FXDir::listFiles(files,name,"*.(png,jpg,jpeg,bmp,gif)",FXDir::NoDirs|FXDir::NoParent|FXDir::CaseFold|FXDir::HiddenFiles);
      if (nfound>1) { // find the best one, based on albumartfiles preference...
        
        names = new FXString[nfound];
        for (i=0;i<nfound;i++) 
          names[i]=FXPath::title(files[i]);

        for (c=0;albumartfiles[c]!=NULL;c++) {
          for (FXint i=0;i<nfound;i++){
            if (comparecase(names[i],albumartfiles[c])==0) {
              albumicon = src.loadScaledIconFile(name+PATHSEPSTRING+files[i],48,1);
              if (albumicon) {
                albumicon->create();
                albumicons.insert((void*)(FXival)pathid,(void*)albumicon);
                delete [] files;
                delete [] names;
                return albumicon;
                }
              }
            }
          }
        delete [] names;
        }
      if (nfound>=1) {
        albumicon = src.loadScaledIconFile(name+PATHSEPSTRING+files[0],48,1);
        if (albumicon) {
          albumicon->create();
          albumicons.insert((void*)(FXival)pathid,(void*)albumicon);
          }
        }        
      delete [] files;
      }
    }
  return albumicon;
  }

FXIcon * GMDatabaseSource::getAlbumIcon(){
  return getAlbumIcon(db->getTrackAlbum(current_track),false);
  }


void GMDatabaseSource::clearAlbumIconCache() {
  FXIcon * albumicon;
#if FOXVERSION < FXVERSION(1,7,0)
  for (FXint i=0;i<albumicons.size();i++){
#else
  for (FXuint i=0;i<albumicons.size();i++){
#endif
    if (!albumicons.empty(i)) {
      albumicon = (FXIcon*)albumicons.value(i);
      delete albumicon;
      }
    }
  albumicons.clear();
  }



void GMDatabaseSource::initPlaylists(GMSourceList & sources) {
  FXIntList playlists;
  if (db->listPlaylists(playlists)) {
    for (FXint i=0;i<playlists.no();i++) {
      sources.append(new GMPlayListSource(db,playlists[i]));
      }
    }
  }

FXint GMDatabaseSource::getNumTracks() const{
  return db->getNumTracks();
  }

FXString GMDatabaseSource::getTrackFilename(FXint id) const{
  return db->getTrackFilename(id);
  }

FXbool GMDatabaseSource::getTrack(GMTrack & info) const{
  return db->getTrack(current_track,info);
  }

FXbool GMDatabaseSource::genre_context_menu(FXMenuPane * pane) {
  new FXMenuCommand(pane,"Edit" UTF8_ELLIPSIS "\tF2\tEdit Genre.",GMIconTheme::instance()->icon_edit,this,GMDatabaseSource::ID_EDIT_GENRE);
//  new FXMenuCommand(pane,"Export" UTF8_ELLIPSIS "\t\tCopy associated tracks to destination.",GMIconTheme::instance()->icon_export,this,ID_EXPORT_GENRE);
  new FXMenuSeparator(pane);
  new FXMenuCommand(pane,"Remove" UTF8_ELLIPSIS "\tDel\tRemove Genre from Library.",GMIconTheme::instance()->icon_delete,this,GMSource::ID_DELETE_GENRE);
  return true;
  }

FXbool GMDatabaseSource::artist_context_menu(FXMenuPane * pane){
  new FXMenuCommand(pane,"Edit" UTF8_ELLIPSIS "\tF2\tEdit Artist.",GMIconTheme::instance()->icon_edit,this,GMDatabaseSource::ID_EDIT_ARTIST);
  new FXMenuCommand(pane,"Copy\tCtrl-C\tCopy associated tracks to the clipboard.",GMIconTheme::instance()->icon_copy,this,ID_COPY_ARTIST);
//  new FXMenuCommand(pane,"Export" UTF8_ELLIPSIS "\t\tCopy associated tracks to destination.",GMIconTheme::instance()->icon_export,this,ID_EXPORT_ARTIST);
  new FXMenuSeparator(pane);
  new FXMenuCommand(pane,"Remove" UTF8_ELLIPSIS "\tDel\tRemove associated tracks from library.",GMIconTheme::instance()->icon_delete,this,GMSource::ID_DELETE_ARTIST);
  return true;
  }

FXbool GMDatabaseSource::album_context_menu(FXMenuPane * pane){
  new FXMenuCommand(pane,"Edit" UTF8_ELLIPSIS "\tF2\tEdit Album.",GMIconTheme::instance()->icon_edit,this,GMDatabaseSource::ID_EDIT_ALBUM);
  new FXMenuCommand(pane,"Copy\tCtrl-C\tCopy associated tracks to the clipboard.",GMIconTheme::instance()->icon_copy,this,ID_COPY_ALBUM);
//  new FXMenuCommand(pane,"Export" UTF8_ELLIPSIS "\t\tCopy associated tracks to destination.",GMIconTheme::instance()->icon_export,this,ID_EXPORT_ALBUM);
  new FXMenuSeparator(pane);
  new FXMenuCommand(pane,"Remove" UTF8_ELLIPSIS "\tDel\tRemove associated tracks from library.",GMIconTheme::instance()->icon_delete,this,GMSource::ID_DELETE_ALBUM);
  return true;
  }

FXbool GMDatabaseSource::track_context_menu(FXMenuPane * pane){
  new FXMenuCommand(pane,"Edit" UTF8_ELLIPSIS "\tF2\tEdit Track Information.",GMIconTheme::instance()->icon_edit,this,GMDatabaseSource::ID_EDIT_TRACK);
  new FXMenuCommand(pane,"Copy\tCtrl-C\tCopy track(s) to clipboard.",GMIconTheme::instance()->icon_copy,this,ID_COPY_TRACK);
//  new FXMenuCommand(pane,"Export" UTF8_ELLIPSIS "\t\tCopy tracks to destination.",GMIconTheme::instance()->icon_export,this,ID_EXPORT_TRACK);
  new FXMenuSeparator(pane);

//  if (GMPlayerManager::instance()->getTrackView()->numTrackSelected()==1)
//    new FXMenuCommand(pane,"Copy Folder Location\t\tCopy Folder Location to clipboard.",GMIconTheme::instance()->icon_copy,this,ID_COPY_LOCATION);
  new FXMenuCommand(pane,"Remove" UTF8_ELLIPSIS "\tDel\tRemove track(s) from library.",GMIconTheme::instance()->icon_delete,this,GMSource::ID_DELETE_TRACK);

/*
    if (getCurrentSourceType()==SOURCE_PLAYLIST && !browserframe->shown()) {
      new FXMenuCommand(&pane,"Remove from Playlist\t\tRemove track(s) from playlist.",icon_delete,this,ID_PLAYLIST_DEL_TRACK);
      new FXMenuSeparator(&pane);
      new FXMenuCommand(&pane,tr("Reorder Playlist"),NULL,this,ID_REORDER_PLAYLIST);
      }
*/
  return true;
  }

FXbool GMDatabaseSource::source_context_menu(FXMenuPane * pane){
  new FXMenuCommand(pane,"Import File(s)"UTF8_ELLIPSIS"\t\tImport Music into Library",GMIconTheme::instance()->icon_importfile,GMPlayerManager::instance()->getMainWindow(),GMWindow::ID_IMPORT_FILES);
  new FXMenuCommand(pane,"Import Folder"UTF8_ELLIPSIS"\tCtrl-O\tImport Music from folder into Library",GMIconTheme::instance()->icon_import,GMPlayerManager::instance()->getMainWindow(),GMWindow::ID_IMPORT_DIRS);
  new FXMenuCommand(pane,"New Play List"UTF8_ELLIPSIS"\t\tCreate a new play list.",GMIconTheme::instance()->icon_playlist,this,GMDatabaseSource::ID_NEW_PLAYLIST);
  new FXMenuSeparator(pane);
  new FXMenuCommand(pane,"Export" UTF8_ELLIPSIS,GMIconTheme::instance()->icon_export,this,GMDatabaseSource::ID_EXPORT);
  new FXMenuCommand(pane,"Information" UTF8_ELLIPSIS "\t\tLibrary Statistics",GMIconTheme::instance()->icon_info,this,GMDatabaseSource::ID_INFO);
  new FXMenuSeparator(pane);
  new FXMenuCommand(pane,"Remove All Tracks\t\tRemove all tracks from the library",GMIconTheme::instance()->icon_delete,this,GMDatabaseSource::ID_CLEAR);
  return true;
  }



FXbool GMDatabaseSource::dnd_source_accepts(FXDragType*types,FXuint ntypes){
  if (FXApp::instance()->getDragWindow()) return false;
  for (FXuint i=0;i<ntypes;i++){
    if (types[i]==GMClipboard::kdeclipboard) return true;
    else if (types[i]==FXWindow::urilistType) return true;
    }
  return false;
  }




void GMDatabaseSource::setFilter(const FXString & text,FXuint mode){
 // fxmessage("setfilter %s %d\n",text.text(),mode);

 /// fxmessage("setting filter\n");

  if (filtermode==mode && filter==text && filterowner==this) return;


  filter=text;
  filtermode=mode;

	FXStringList keywords;
	FXString word;

	for (FXint i=0;i<text.length();i++){
		if (Ascii::isSpace(text[i])) {
			if (!word.empty()) {
		    FXchar * sf = sqlite3_mprintf("LIKE '%%%q%%'",word.text());
				keywords.append(FXString(sf));
				word=FXString::null;
				sqlite3_free(sf);
				}
			}
		else {
			word+=text[i];
			}
		}

	 if (!word.empty()) {
		 FXchar * sf = sqlite3_mprintf("LIKE '%%%q%%'",word.text());
		 keywords.append(FXString(sf));
		 word=FXString::null;
		 sqlite3_free(sf);
		 }



  db->database()->execute("DROP VIEW IF EXISTS filtered;");
  if (keywords.no()) {
    FXString query = "CREATE TEMP VIEW filtered AS SELECT tracks.id as track ,artists.id as artist ,albums.id as album ,genre FROM tracks,artists,albums,album_artist WHERE tracks.album == albums.id AND artists.id == album_artist.artist AND albums.id == album_artist.album ";
    if (getPlayList()!=-1) {
      query+=" AND tracks.id IN (SELECT DISTINCT(track) FROM playlist_tracks WHERE playlist == " + GMStringVal(getPlayList()) + ") ";
      }
		for (FXint i=0;i<keywords.no();i++){
			query+=" AND (";
    	switch(filtermode) {
      	case 0: query+=" title " + keywords[i] + " OR albums.name " + keywords[i] + " OR artists.name " + keywords[i]; break;
      	case 1: query+=" artists.name " + keywords[i]; break;
      	case 2: query+=" albums.name " + keywords[i] ; break;
      	case 3: query+=" title " + keywords[i];				break;
				}
      query+=" )";
			}
#ifdef DEBUG
		FXlong start = fxgetticks();
#endif
    db->database()->execute(query);
#ifdef DEBUG
		FXlong end = fxgetticks();
		fxmessage("setFilter(): %lld\n",end-start);
#endif
    }
  filterowner=this;
  }




FXbool GMDatabaseSource::listGenres(GMList * list,FXIcon * icon) {
  FXint id;
  const FXchar * name;
  GMQuery q;
  FXString query;

#ifdef DEBUG
	FXlong start = fxgetticks();
#endif

  try {
    if (getPlayList()==-1) {
      if (filter.empty()) {
        while(db->list_genres.execute()) {
          db->list_genres.getResult(0,id);
          name = db->list_genres.getResult(1);
          list->appendItem(name,icon,(void*)(FXival)id);
          }
        db->list_genres.reset();
        return true;
        }
      else {
        query = "SELECT genres.id,genres.name FROM genres WHERE id IN (SELECT genre FROM filtered); ";
        }
      }
    else {
      if (filter.empty()) {
        query = "SELECT id,name "
                "FROM genres "
                "WHERE id IN ( "
                  "SELECT DISTINCT(genre) "
                  "FROM tracks,playlist_tracks "
                  "WHERE id == playlist_tracks.track AND playlist_tracks.playlist == " + GMStringVal(getPlayList()) +
                  ");";
        }
      else {
        query = "SELECT genres.id,genres.name FROM genres WHERE id IN (SELECT genre FROM filtered); ";
        }
      }

    q.compile(db->database(),query);
    while( q.execute()){
      q.getResult(0,id);
      name=q.getResult(1);
      list->appendItem(name,icon,(void*)(FXival)id);
      }
    q.reset();

    }
  catch(FXCompileException & e){
    list->clearItems();
    return false;
    }
  catch(FXExecuteException &){
    list->clearItems();
    return false;
    }
#ifdef DEBUG
	FXlong end = fxgetticks();
	fxmessage("listGenres(): %lld\n",end-start);
#endif
  return true;
  }


FXbool GMDatabaseSource::listArtists(GMList * list,FXIcon * icon,FXint genre){
  GMListItem * item;
  const FXchar * name;
  FXint id;
  GMQuery q;
  FXString query;
  FXString filterquery;
#ifdef  DEBUG
	FXlong start = fxgetticks();
#endif

  try {
    if (filter.empty()){
      if (genre==-1) {
        if (getPlayList()==-1) {
          query = "SELECT id,name FROM artists;";
          }
        else{
          query = "SELECT DISTINCT(artists.id), artists.name "
                  "FROM artists,album_artist "
                  "WHERE album_artist.artist == artists.id AND album_artist.album IN ( "
                    "SELECT DISTINCT(album) FROM tracks WHERE id IN ( "
                      "SELECT DISTINCT(track) FROM playlist_tracks WHERE playlist == " + GMStringVal(getPlayList()) + "));";
          }
        }
      else {
        if (getPlayList()==-1) {
          query = "SELECT id, name FROM artists "
                  "WHERE id IN ( "
                    "SELECT DISTINCT(artist) "
                    "FROM album_artist WHERE album IN ( "
                      "SELECT DISTINCT(album) FROM tracks "
                      "WHERE genre == " + GMStringVal(genre) +
                      ")"
                    ");";
          }
        else{
          query = "SELECT DISTINCT(artists.id), artists.name "
                  "FROM artists,album_artist "
                  "WHERE album_artist.artist == artists.id AND album_artist.album IN ( "
                    "SELECT DISTINCT(album) "
                    "FROM tracks "
                    "WHERE genre==" + GMStringVal(genre) + " AND id IN (" +
                      "SELECT DISTINCT(track) FROM playlist_tracks WHERE playlist == " + GMStringVal(getPlayList()) +
                      ")"
                    ");";
          }
        }
      }
    else {
//			query = "SELECT DISTINCT(artists.id), artists.name FROM artists,filtered WHERE filtered.artist == artists.id;";

      query = "SELECT artists.id, artists.name FROM artists WHERE id IN (SELECT artist FROM filtered ";
      if (genre>=0){
        query+=" WHERE genre == " + GMStringVal(genre);
        }
      query+=");";


/*
      if (genre>=0 || getPlayList()>=0) {
        query+=",tracks,albums,album_artist";
        query+=" WHERE tracks.album == albums.id AND albums.id == album_artist.album AND artists.id == album_artist.artist ";
        }
      else{
        switch(filtermode) {
          case 1: query="SELECT artists.id, artists.name FROM artists WHERE "; break;
          case 3:
          case 0: query+=",tracks,albums,album_artist";
                  query+=" WHERE tracks.album == albums.id AND albums.id == album_artist.album AND artists.id == album_artist.artist AND ";
                  break;
          case 2: query+=",albums,album_artist";
                  query+=" WHERE albums.id == album_artist.album AND artists.id == album_artist.artist AND ";
                  break;
          }
        }

      if (genre!=-1)
        query+=" tracks.genre == " + GMStringVal(genre) + " AND ";

      if (getPlayList()!=-1)
        query+=" tracks.id IN (SELECT DISTINCT(track) FROM playlist_tracks WHERE playlist == " + FXStringVal(getPlayList()) + ") AND ";

      switch(filtermode) {
         case 0: query+=" ( title LIKE '%" + filter + "%' OR albums.name LIKE '%" + filter + "%' OR artists.name LIKE '%" + filter + "%') "; break;
         case 1: query+=" ( artists.name LIKE '%" + filter + "%') "; break;
         case 2: query+=" ( albums.name LIKE '%" + filter + "%') "; break;
         case 3: query+=" ( title LIKE '%" +filter + "%') "; break;
         }
      query+=";"; ORDER BY artists.name; ";
*/
//      fxmessage("query=%s\n",query.text());
      }
//		fxmessage("query: %s\n",query.text());
    q.compile(db->database(),query);
    while( q.execute()){
      q.getResult(0,id);
      name=q.getResult(1);
      item = new GMListItem(name,icon,(void*)(FXival)id);
      item->setDraggable(true);
      list->appendItem(item);
      }
    q.reset();
    }
  catch(FXCompileException & e){
    list->clearItems();
    return false;
    }
  catch(FXExecuteException & e){
    list->clearItems();
    return false;
    }
#ifdef DEBUG
	FXlong end = fxgetticks();
	fxmessage("listArtists(): %lld\n",end-start);
#endif
  return true;
  }


FXbool GMDatabaseSource::listAlbums(GMList * list,FXIcon * icon,const FXIntList & artistlist,FXint genre){
  const FXchar * c_name=NULL;
  FXint id;
  FXString name;
  FXString query;
  FXIcon * albumicon=NULL;

#ifdef DEBUG
	FXlong start = fxgetticks();
#endif

  GMAlbumListItem * item=NULL;
  GMQuery q;
  try {
    if (!filter.empty()){
      query = "SELECT albums.id,albums.name FROM albums WHERE id IN (SELECT album FROM filtered ";
      if (artistlist.no()>1) {
        query+=" WHERE artist IN ( " + GMStringVal(artistlist[0]);
        for (FXint i=1;i<artistlist.no();i++){
          query+=","+GMStringVal(artistlist[i]);
          }
        if (genre>=0)
          query+=" AND genre == " + GMStringVal(genre);
        query+=" )";
        }
      else if (artistlist.no()==1) {
        query+=" WHERE artist == " + GMStringVal(artistlist[0]);
        if (genre>=0)
          query+=" AND genre == " + GMStringVal(genre);
        }
      else if (genre>=0){
        query+=" WHERE genre == " + GMStringVal(genre);
        }
      query+=");";
      }
    else {

      if (artistlist.no()==0 && genre==-1 && getPlayList()==-1){
        query = "SELECT id, name FROM albums ORDER BY name;";
        }
      else if (genre==-1 && getPlayList()==-1) {
        query = "SELECT albums.id,albums.name FROM albums,album_artist WHERE albums.id == album_artist.album";

        if (artistlist.no()>1) {
          query+=" AND album_artist.artist IN ( " + GMStringVal(artistlist[0]);
          for (FXint i=1;i<artistlist.no();i++){
            query+=","+GMStringVal(artistlist[i]);
            }
          query+=" )";
          }
        else {
          query+=" AND album_artist.artist == " + GMStringVal(artistlist[0]);
          }
        query+=" ORDER BY albums.name;";
        }
      else {
        query = "SELECT DISTINCT(albums.id), albums.name  FROM albums,tracks";


        if (artistlist.no() || !filter.empty())
          query+=",album_artist";
        if (getPlayList()>=0)
          query+=",playlist_tracks";

        if (artistlist.no() || !filter.empty())
          query+=" WHERE album_artist.album == tracks.album AND album_artist.album == albums.id";
        else
          query+=" WHERE albums.id == tracks.album";

        if (getPlayList()>=0) {
          query+=" AND playlist_tracks.track == tracks.id";
          query+=" AND playlist_tracks.playlist == "+GMStringVal(getPlayList());
          }

        if (genre >= 0) query += " AND genre == " + GMStringVal(genre);
        if (artistlist.no()>1) {
          query+=" AND album_artist.artist IN ( " + GMStringVal(artistlist[0]);

          for (FXint i=1;i<artistlist.no();i++){
            query+=","+GMStringVal(artistlist[i]);
            }

          query+=" )";

          if (!filter.empty())
            query += " AND ( albums.name LIKE '%" + filter + "%' OR tracks.title LIKE '%" + filter + "%' OR artists.name LIKE '%" + filter + "%')";

          query+=" ORDER BY albums.name;";
          }
        else if (artistlist.no()==1){
          query+=" AND album_artist.artist == " + GMStringVal(artistlist[0]);
          query+=" ORDER BY albums.name;";
          }
        else {
          query+=" ORDER BY albums.name;";
          }
        }
      }
    q.compile(db->database(),query);
    while(q.execute()){
      q.getResult(0,id);
      c_name = q.getResult(1);
      if (c_name!=NULL && item && c_name==name) {
        FXASSERT(item);
        item->albums.append(id);
        }
      else {
        name=c_name;
        if (GMPlayerManager::instance()->getPreferences().gui_show_albumcovers){
          albumicon = getAlbumIcon(id,true);
          if (albumicon)
            item = new GMAlbumListItem(c_name,albumicon,id);
          else
            item = new GMAlbumListItem(c_name,icon,id);
          }
        else {
          item = new GMAlbumListItem(c_name,icon,id);
          }
        list->appendItem(item);
        }
      }
    }
  catch(FXCompileException & e){
    list->clearItems();
    return false;
    }
  catch(FXExecuteException & e){
    list->clearItems();
    return false;
    }
#ifdef DEBUG
	FXlong end = fxgetticks();
	fxmessage("listAlbums(): %lld\n",end-start);
#endif
  return true;
  }


















FXbool GMDatabaseSource::listTracks(GMTrackList * tracklist,const FXIntList & albumlist,FXint genre){
  GMQuery q;
  FXString query;
  const FXchar * c_artist;
  const FXchar * c_albumname;
  const FXchar * c_title;
  const FXchar * c_genre;
  FXint time;
  FXuint no;
  FXint id;
  FXint queue=1;
	FXint year;
  GMDBTrackItem * item;

  GMDBTrackItem::max_queue=0;
  GMDBTrackItem::max_trackno=0;

#ifdef DEBUG
	FXlong start = fxgetticks();
#endif

  try {

//    if (!filter.empty()){


 //     SELECT filtered.track,tracks.title,tracks.time,tracks.no,genres.name,artists.name,albums.name FROM filtered,tracks WHERE tracks.id == filtered.track,tracks.genre == filtered.genre,albums.id==filtered.album




    query = "SELECT tracks.id,title,time,no,year,genres.name,artists.name,albums.name "
            "FROM tracks, genres, albums, artists, album_artist";

    if (getPlayList()>=0)
      query+=", playlist_tracks";

    query += " WHERE genres.id == tracks.genre AND "
                   "album_artist.artist == artists.id AND "
                   "album_artist.album == albums.id AND "
                   "tracks.album == albums.id";

    if (genre>=0) {
      query+=" AND genre == " + GMStringVal(genre);
      }

    if (getPlayList()>=0) {
      query+=" AND playlist_tracks.track == tracks.id";
      if (filter.empty()) query+=" AND playlist_tracks.playlist == "+GMStringVal(getPlayList());
      }

    if (albumlist.no()) {
      if (albumlist.no()>1) {
        query+=" AND tracks.album IN ( ";
        query+=GMStringVal(albumlist[0]);

        for (FXint i=1;i<albumlist.no();i++){
          query+=","+GMStringVal(albumlist[i]);
          }
        query+=" )";
        }
      else {
        query+=" AND tracks.album == " + GMStringVal(albumlist[0]);
        }
      }

    if (!filter.empty()) {
      query+=" AND tracks.id IN (SELECT track FROM filtered) ";
//      switch(filtermode) {
//        case 0: query+=" AND ( title LIKE '%" + filter + "%' OR albums.name LIKE '%" + filter + "%' OR artists.name LIKE '%" + filter + "%') "; break;
//        case 1: query+=" AND ( artists.name LIKE '%" + filter + "%') ";break;
//        case 2: query+=" AND ( albums.name LIKE '%" + filter + "%') ";break;
//        case 3: query+=" AND ( title LIKE '%" +filter + "%') "; break;
//        }
      }

    if (getPlayList()>=0)
      query+=" ORDER BY playlist_tracks.queue;";
    else
      query+=";";

    q.compile(db->database(),query);

    while(q.execute()){
      q.getResult(0,id);
      c_title = q.getResult(1);
      q.getResult(2,time);
      q.getResult(3,no);
      q.getResult(4,year);
      c_genre = q.getResult(5);
      c_artist = q.getResult(6);
      c_albumname = q.getResult(7);

      GMDBTrackItem::max_trackno=FXMAX(GMDBTrackItem::max_digits(GMTRACKNO(no)),GMDBTrackItem::max_trackno);
      item = new GMDBTrackItem(id,c_title,c_artist,c_albumname,c_genre,time,no,queue++,(FXushort)year);
      tracklist->appendItem((FXIconItem*)item);
      }
    GMDBTrackItem::max_trackno = tracklist->getFont()->getTextWidth(FXString('8',GMDBTrackItem::max_trackno));
    GMDBTrackItem::max_queue   = tracklist->getFont()->getTextWidth(FXString('8',GMDBTrackItem::max_digits(queue)));
    }
  catch(FXCompileException & e){
    tracklist->clearItems();
    return false;
    }
  catch(FXExecuteException & e){
    tracklist->clearItems();
    return false;
    }
#ifdef DEBUG
	FXlong end = fxgetticks();
	fxmessage("listTracks(): %lld\n",end-start);
#endif
  return true;
  }


long GMDatabaseSource::onCmdEditGenre(FXObject*,FXSelector,void*) {
  FXIntList tracks;
  FXIntList genres;
  FXint i;
  FXString name;
  FXbool changed=false;

  GMPlayerManager::instance()->getTrackView()->getSelectedGenres(genres);
  GMPlayerManager::instance()->getTrackView()->getTracks(tracks);

  if (tracks.no()==0) {
    FXApp::instance()->beep();
    return 1;
    }

  name = db->getGenreName(genres[0]);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Edit Genre Information",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Edit Genre Information","Update genre on tracks");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Save",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main         = new FXVerticalFrame(&dialog,LAYOUT_FILL_X,0,0,0,0,60,40,10,10);
  FXComboBox      * genrebox     = new FXComboBox(main,30,NULL,0,LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN);
  FXCheckButton   * sync         = new FXCheckButton(main,"Update Tag in File",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
  FXCheckButton   * syncfilename = new FXCheckButton(main,"Update Filename",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);

  sync->setCheck(FXApp::instance()->reg().readBoolEntry("update files","edit-genre",false));


  db->listGenres(genrebox);

  genrebox->setSortFunc(genre_list_sort);
  genrebox->setNumVisible(FXMIN(10,genrebox->getNumItems()));
  genrebox->sortItems();
  genrebox->setCurrentItem(-1);
  genrebox->setCurrentItem(genrebox->findItem(name));

  if (dialog.execute()) {

    db->beginEdit();

    /// Update Genre
    if (name!=genrebox->getText() && !genrebox->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackGenre(tracks[i],genrebox->getText());
        }
      changed=true;
      }

    /// Update GUI / write to files
    if (changed) {
      if (sync->getCheck()) {
        GMTrack info;
        for (i=0;i<tracks.no();i++){
          if (!db->getTrack(tracks[i],info)) break;
          GMTag::sync(info);
          }
        }

      FXApp::instance()->reg().writeBoolEntry("update files","edit-genre",sync->getCheck());
      }


    if (syncfilename->getCheck()){
      updateTrackFilenames(tracks);
      }

    db->endEdit();

    if (changed) GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }

long GMDatabaseSource::onCmdEditArtist(FXObject*,FXSelector,void*){
  FXIntList artists;
  FXIntList tracks;
  FXString name;
  FXint i;
  FXbool changed=false;

  GMPlayerManager::instance()->getTrackView()->getSelectedArtists(artists);
  GMPlayerManager::instance()->getTrackView()->getTracks(tracks);

  if (tracks.no()==0 || artists.no()!=1 ) {
    FXApp::instance()->beep();
    return 1;
    }

  name = db->getArtistName(artists[0]);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Edit Artist Information",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Edit Artist Information","Update artist on tracks");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Save",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main         = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXMatrix        * matrix       = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS);
                                   new FXLabel(matrix,"Artist",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox      * artistbox    = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
                                   new FXLabel(matrix,"Genre",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXHorizontalFrame * hframe     = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

  FXTextField * yearfield        = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED);
                                   new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
  FXComboBox      * genrebox     = new FXComboBox(hframe,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
                                   new FXFrame(matrix,FRAME_NONE);
  FXCheckButton   * sync         = new FXCheckButton(matrix,"Update Tag in File",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
                                   new FXFrame(matrix,FRAME_NONE);
  FXCheckButton   * syncfilename = new FXCheckButton(matrix,"Update Filename",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);

  db->listArtists(artistbox);
  artistbox->setSortFunc(artist_list_sort);
  artistbox->setNumVisible(FXMIN(10,artistbox->getNumItems()));
  artistbox->sortItems();
  artistbox->setCurrentItem(-1);
  artistbox->setCurrentItem(artistbox->findItem(name));

  db->listGenres(genrebox);
  genrebox->setSortFunc(genre_list_sort);
  genrebox->setNumVisible(FXMIN(10,genrebox->getNumItems()));
  genrebox->sortItems();
  genrebox->setCurrentItem(-1);

  sync->setCheck(FXApp::instance()->reg().readBoolEntry("update files","edit-artist",false));
  if (dialog.execute()) {
    db->beginEdit();

    /// Update Genre
    if (!genrebox->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackGenre(tracks[i],genrebox->getText());
        }
      changed=true;
      }

    if (!yearfield->getText().empty()){
#if FOXVERSION >= FXVERSION(1,7,12)
      FXint year=yearfield->getText().toInt();
#else
      FXint year=FXIntVal(yearfield->getText());
#endif
      for (i=0;i<tracks.no();i++){
        db->setTrackYear(tracks[i],year);
        }
      changed=true;
      }


    // Update Artist
    if (name!=artistbox->getText() && !artistbox->getText().empty()){
      for (i=0;i<tracks.no();i++){
        db->setTrackArtist(tracks[i],artistbox->getText());
        }
      changed=true;
      }

    /// Update GUI / write to files
    if (changed) {
      if (sync->getCheck()) {
        GMTrack info;
        for (i=0;i<tracks.no();i++){
          if (!db->getTrack(tracks[i],info)) break;
          GMTag::sync(info);
          }
        }
      FXApp::instance()->reg().writeBoolEntry("update files","edit-artist",sync->getCheck());
      }

    if (syncfilename->getCheck()){
     updateTrackFilenames(tracks);
     }

    db->endEdit();

    if (changed) GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }

long GMDatabaseSource::onCmdEditAlbum(FXObject*,FXSelector,void*){
  FXIntList albums;
  FXIntList tracks;
  FXString name;
  FXString artist;
  FXint i;
  FXbool changed=false;

  GMPlayerManager::instance()->getTrackView()->getSelectedAlbums(albums);
  GMPlayerManager::instance()->getTrackView()->getTracks(tracks);

  if (tracks.no()==0 || albums.no()!=1 ) {
    FXApp::instance()->beep();
    return 1;
    }

  name = db->getAlbumName(albums[0]);
  artist = db->getAlbumArtist(albums[0]);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Edit Album Information",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Edit Album Information","Update album on associated tracks");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Save",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main          = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXMatrix        * matrix        = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS);
                                    new FXLabel(matrix,"Album",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox      * albumbox      = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
                                    new FXLabel(matrix,"Artist",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox      * artistbox     = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
                                    new FXLabel(matrix,"Genre",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);

  FXHorizontalFrame * hframe      = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

  FXTextField * yearfield         = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED);
                                    new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
  FXComboBox      * genrebox      = new FXComboBox(hframe,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);

                                    new FXFrame(matrix,FRAME_NONE);
  FXCheckButton   * sync          = new FXCheckButton(matrix,"Update Tag in File",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
                                    new FXFrame(matrix,FRAME_NONE);
  FXCheckButton   * syncfilename  = new FXCheckButton(matrix,"Update Filename",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);


  db->listAlbums(albumbox,albums[0]);
  albumbox->setSortFunc(album_list_sort);
  albumbox->setNumVisible(FXMIN(10,albumbox->getNumItems()));
  albumbox->sortItems();
  albumbox->setCurrentItem(-1);
  albumbox->setCurrentItem(albumbox->findItem(name));

  db->listArtists(artistbox);
  artistbox->setSortFunc(artist_list_sort);
  artistbox->setNumVisible(FXMIN(10,artistbox->getNumItems()));
  artistbox->sortItems();
  artistbox->setCurrentItem(-1);
  artistbox->setCurrentItem(artistbox->findItem(artist));

  db->listGenres(genrebox);
  genrebox->setSortFunc(genre_list_sort);
  genrebox->setNumVisible(FXMIN(10,genrebox->getNumItems()));
  genrebox->sortItems();
  genrebox->setCurrentItem(-1);

  sync->setCheck(FXApp::instance()->reg().readBoolEntry("update files","edit-album",false));

  if (dialog.execute()) {

    db->beginEdit();

    /// Update Genre
    if (!genrebox->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackGenre(tracks[i],genrebox->getText());
        }
      changed=true;
      }

    if (!yearfield->getText().empty()){
#if FOXVERSION >= FXVERSION(1,7,12)
      FXint year=yearfield->getText().toInt();
#else
      FXint year=FXIntVal(yearfield->getText());
#endif
      for (i=0;i<tracks.no();i++){
        db->setTrackYear(tracks[i],year);
        }
      changed=true;
      }

    /// Update Artist and Album
    if (artist!=artistbox->getText() && !artistbox->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackArtist(tracks[i],artistbox->getText(),albumbox->getText());
        }
      changed=true;
      }

    /// Update Album Only
    else if (name!=albumbox->getText() && !albumbox->getText().empty()){
      for (i=0;i<tracks.no();i++){
        db->setTrackAlbum(tracks[i],albumbox->getText());
        }
      changed=true;
      }

    /// Update GUI / write to files
    if (changed) {
      if (sync->getCheck()) {
        GMTrack info;
        for (i=0;i<tracks.no();i++){
          if (!db->getTrack(tracks[i],info)) break;
          GMTag::sync(info);
          }
        }
      FXApp::instance()->reg().writeBoolEntry("update files","edit-album",sync->getCheck());
      }

    if (syncfilename->getCheck()){
      updateTrackFilenames(tracks);
      }

    db->endEdit();

    if (changed) GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }


long GMDatabaseSource::onCmdEditTrack(FXObject*,FXSelector,void*){
  FXIntList tracks;
  FXString name;
  FXString field;
  FXString album;
  FXint i;
  FXbool changed=false;
  GMTrack info;
  GMTag::Properties prop;

  GMPlayerManager::instance()->getTrackView()->getSelectedTracks(tracks);

  db->getTrack(tracks[0],info);
  if (tracks.no()==1)
    GMTag::properties(info.mrl,prop);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Edit Track Information",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Edit Track Information","Update track info and filename");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Save",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);

	FXVerticalFrame * main=NULL;
  if (tracks.no()==1) { /* only show spinner when one track is selected */
		main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);
		}
	else {
		new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
		main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,5,5);
		}
//  FXMatrix * matrix = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS,0,0,0,0,0,0,0,0);

  FXSpinner * spinner = NULL;
  FXSpinner * discspinner = NULL;
  FXTextField * textfield = NULL;
  FXTextField * yearfield = NULL;
  FXTextField * discfield = NULL;
  FXCheckButton * autonumber = NULL;
  FXHorizontalFrame * hframe = NULL;
  FXTabBook * tabbook = NULL;
  FXMatrix * matrix = NULL;
  if (tracks.no()==1) { /* only show spinner when one track is selected */

    tabbook = new FXTabBook(main,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);

    new FXTabItem(tabbook," Tag ");
    FXMatrix * tagmatrix = new FXMatrix(tabbook,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS|FRAME_RAISED|FRAME_THICK,0,0,0,0,10,10,10,10);

    new FXTabItem(tabbook," Properties ");
    matrix = new FXMatrix(tabbook,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS|FRAME_RAISED|FRAME_THICK,0,0,0,0,10,10,10,10);

    new FXLabel(matrix,"Filename",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
    textfield->setText(info.mrl);

    new FXLabel(matrix,"Type",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
    textfield->setText(GMStringFormat("%s",FXPath::extension(info.mrl).upper().text()));

    new FXLabel(matrix,"Size",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
#if defined(__LP64__) || defined(_LP64) || (_MIPS_SZLONG == 64) || (__WORDSIZE == 64)
      textfield->setText(GMStringFormat("%'ld",FXStat::size(info.mrl)));
#else
      textfield->setText(GMStringFormat("%'lld",FXStat::size(info.mrl)));
#endif

    new FXLabel(matrix,"Bitrate",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
    textfield->setText(GMStringFormat("%dkbs",prop.bitrate));

    new FXLabel(matrix,"Samplerate",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
    textfield->setText(GMStringFormat("%dHz",prop.samplerate));

    new FXLabel(matrix,"Channels",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK|TEXTFIELD_READONLY);
    textfield->setText(GMStringFormat("%d",prop.channels));

/*
    new FXLabel(matrix,"Type",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
    new FXLabel(hframe,GMStringFormat("%s",FXPath::extension(info.mrl).upper().text()),NULL,FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
    new FXLabel(hframe,"Size",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%'ld",FXStat::size(info.mrl)),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(matrix,"Bitrate",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
    new FXLabel(hframe,GMStringFormat("%dkbs",prop.bitrate),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(hframe,"Samplerate",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%dHz",prop.samplerate),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(hframe,"Channels",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%d",prop.channels),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
*/
//    FXLabel * label = new FXLabel(matrix,GMStringFormat("[%s] %dch, %dHz, %dkbs, %d:%.2d",FXPath::extension(info.mrl).upper().text(),prop.channels,prop.samplerate,prop.bitrate,info.time/60,info.time%60),NULL,/*FRAME_SUNKEN|FRAME_THICK*/LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
//label->disable();


    matrix = tagmatrix;

    new FXFrame(matrix,FRAME_NONE,0,0,0,0,0,0,3);
    new FXFrame(matrix,FRAME_NONE|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,3);

    new FXLabel(matrix,"Track",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

    spinner = new FXSpinner(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT);
    spinner->setRange(0,1000);

    new FXLabel(hframe,"Disc",NULL,LABEL_NORMAL|LAYOUT_LEFT|LAYOUT_CENTER_Y);
    discspinner = new FXSpinner(hframe,3,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT);
    discspinner->setRange(0,100);

    yearfield = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED);
    new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);


   // new FXLabel(hframe,"Audio",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    }
  else {

    matrix = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS,0,0,0,0,0,0,0,0);

    new FXLabel(matrix,"Disc",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
    discfield = new FXTextField(hframe,3,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED|JUSTIFY_RIGHT);

    yearfield = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED|JUSTIFY_RIGHT);
    new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);


    }


  new FXLabel(matrix,"Title",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXTextField * title_field = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK);

  new FXLabel(matrix,"Artist",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox * artistbox = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
  new FXLabel(matrix,"Album",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox * albumbox = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);

  new FXLabel(matrix,"Genre",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

//  yearfield = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED);
//  new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
  FXComboBox * genrebox = new FXComboBox(hframe,20,NULL,0,LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN);

  if (tracks.no()>1 && tracks.no()<=0xFFFF) {
    new FXFrame(matrix,FRAME_NONE);
    autonumber = new FXCheckButton(matrix,"Assign track number based on selection order.",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
    }

  new FXFrame(matrix,FRAME_NONE);
  FXCheckButton * sync = new FXCheckButton(matrix,"Update Tag in File",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
  new FXFrame(matrix,FRAME_NONE);
  FXCheckButton * syncfilename = new FXCheckButton(matrix,"Update Filename",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);

  sync->setCheck(FXApp::instance()->reg().readBoolEntry("update files","edit-track",false));

  if (spinner) spinner->setValue(GMTRACKNO(info.no));

  db->listAlbums(albumbox,db->getTrackAlbum(tracks[0]));
  albumbox->setSortFunc(album_list_sort);
  albumbox->setNumVisible(FXMIN(10,albumbox->getNumItems()));
  albumbox->sortItems();
  albumbox->setCurrentItem(-1);

  db->listArtists(artistbox);
  artistbox->setSortFunc(artist_list_sort);
  artistbox->setNumVisible(FXMIN(10,artistbox->getNumItems()));
  artistbox->sortItems();
  artistbox->setCurrentItem(-1);

  db->listGenres(genrebox);
  genrebox->setSortFunc(genre_list_sort);
  genrebox->setNumVisible(FXMIN(10,genrebox->getNumItems()));
  genrebox->sortItems();
  genrebox->setCurrentItem(-1);


  if (tracks.no()==1) {
    albumbox->setCurrentItem(albumbox->findItem(info.album));
    artistbox->setCurrentItem(artistbox->findItem(info.artist));
    genrebox->setCurrentItem(genrebox->findItem(info.genre));
    yearfield->setText(GMStringVal(info.year));
    title_field->setText(info.title);
    discspinner->setValue(GMDISCNO(info.no));
    }

  if (dialog.execute()) {

    db->beginEdit();

    if (tracks.no()==1) {
      if (GMTRACKNO(info.no)!=spinner->getValue() || GMDISCNO(info.no)!=discspinner->getValue() ) {
        db->setTrackDiscNumber(tracks[0],GMALBUMNO(discspinner->getValue(),spinner->getValue()));
        changed=true;
        }
      }
    else {
      if (autonumber->getCheck()) {
        if (discfield->getText().empty()) {
          for (i=0;i<tracks.no();i++){
            db->setTrackNumber(tracks[i],i+1);
            }
          }
        else {
          for (i=0;i<tracks.no();i++){
#if FOXVERSION >= FXVERSION(1,7,12)
            db->setTrackDiscNumber(tracks[i],GMALBUMNO(discfield->getText().toUInt(),i+1));
#else
            db->setTrackDiscNumber(tracks[i],GMALBUMNO(FXUIntVal(discfield->getText()),i+1));
#endif
            }
          }
        changed=true;
        }
      else if (!discfield->getText().empty()) {
        for (i=0;i<tracks.no();i++){
#if FOXVERSION >= FXVERSION(1,7,12)
          db->setTrackDisc(tracks[i],discfield->getText().toUInt());
#else
          db->setTrackDisc(tracks[i],FXUIntVal(discfield->getText()));
#endif
          }
        changed=true;
        }
      }


/*




    if (title_field && info.title!=title_field->getText() && !title_field->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackTitle(tracks[i],title_field->getText());
        }
      changed=true;
      db->setTrackTitle(tracks[0],title_field->getText());
      info.title = title_field->getText();
      changed=true;
*/

    if (!yearfield->getText().empty()) {
#if FOXVERSION >= FXVERSION(1,7,12)
      FXint year=yearfield->getText().toInt();
#else
      FXint year=FXIntVal(yearfield->getText());
#endif
      if ((tracks.no()>1) || (tracks.no()==1 && info.year!=year)){
        for (i=0;i<tracks.no();i++){
          db->setTrackYear(tracks[i],year);
          }
        changed=true;
        }
      }


    field=title_field->getText().trim().simplify();
    if ((tracks.no()==1 && info.title!=field && !field.empty()) || (tracks.no()>1 && !field.empty()) ) {
      for (i=0;i<tracks.no();i++){
        db->setTrackTitle(tracks[i],field);
        }
      changed=true;
      }

    field=genrebox->getText().trim().simplify();
    if ((tracks.no()==1 && info.genre!=field && !field.empty()) || (tracks.no()>1 && !field.empty())){
      for (i=0;i<tracks.no();i++){
        db->setTrackGenre(tracks[i],field);
        }
      changed=true;
      }


    field=artistbox->getText().trim().simplify();
    album=albumbox->getText().trim().simplify();
    // If we need to change the artist name, we already update/create a new album entry
    if ((tracks.no()==1 && info.artist!=field && !field.empty()) || (tracks.no()>1 && !field.empty())){
      for (i=0;i<tracks.no();i++){
        // albumbox may be empty, in which case we use the existing album name.
        db->setTrackArtist(tracks[i],field,album);
        }
      changed=true;
      }
    else if ((tracks.no()==1 && info.album!=album && !album.empty()) || (tracks.no()>1 && !album.empty())){
      for (i=0;i<tracks.no();i++){
        db->setTrackAlbum(tracks[i],album);
        }
      changed=true;
      }

    /// Update GUI / write to files
    if (changed) {
      if (sync->getCheck()) {
        GMTrack info;
        for (i=0;i<tracks.no();i++){
          if (!db->getTrack(tracks[i],info)) break;
          GMTag::sync(info);
          }
        }
      FXApp::instance()->reg().writeBoolEntry("update files","edit-track",sync->getCheck());
      }


    if (syncfilename->getCheck()){
      updateTrackFilenames(tracks);
      }

    db->endEdit();

    if (changed) GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }
#if 0
long GMDatabaseSource::onCmdEditTrack(FXObject*,FXSelector,void*){
  FXIntList tracks;
  FXString name;
  FXString field;
  FXString album;
  FXint i;
  FXbool changed=false;
  GMTrack info;
  GMTag::Properties prop;

  GMPlayerManager::instance()->getTrackView()->getSelectedTracks(tracks);

  db->getTrack(tracks[0],info);
  if (tracks.no()==1)
    GMTag::properties(info.mrl,prop);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Edit Track Information",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Edit Track Information","Update track info and filename");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Save",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);

  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,5);
  FXMatrix * matrix = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS,0,0,0,0,0,0,0,0);

  FXSpinner * spinner = NULL;
  FXTextField * yearfield = NULL;
  FXCheckButton * autonumber = NULL;
  if (tracks.no()==1) { /* only show spinner when one track is selected */

    new FXLabel(matrix,"Type",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    FXHorizontalFrame * hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
    new FXLabel(hframe,GMStringFormat("%s",FXPath::extension(info.mrl).upper().text()),NULL,FRAME_SUNKEN|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
    new FXLabel(hframe,"Size",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%'ld",FXStat::size(info.mrl)),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(matrix,"Bitrate",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
    hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
    new FXLabel(hframe,GMStringFormat("%dkbs",prop.bitrate),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(hframe,"Samplerate",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%dHz",prop.samplerate),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

    new FXLabel(hframe,"Channels",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    new FXLabel(hframe,GMStringFormat("%d",prop.channels),NULL,FRAME_SUNKEN|LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);

//    FXLabel * label = new FXLabel(matrix,GMStringFormat("[%s] %dch, %dHz, %dkbs, %d:%.2d",FXPath::extension(info.mrl).upper().text(),prop.channels,prop.samplerate,prop.bitrate,info.time/60,info.time%60),NULL,/*FRAME_SUNKEN|FRAME_THICK*/LAYOUT_FILL_X|JUSTIFY_LEFT|LAYOUT_CENTER_Y);
//label->disable();

    new FXFrame(matrix,FRAME_NONE,0,0,0,0,0,0,3);
    new FXFrame(matrix,FRAME_NONE|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,3);

    new FXLabel(matrix,"No",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);

//    FXHorizontalFrame * hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

    spinner = new FXSpinner(matrix,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_LEFT|LAYOUT_FILL_COLUMN);
    spinner->setRange(0,1000);

   // new FXLabel(hframe,"Audio",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y);
    }

  new FXLabel(matrix,"Title",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXTextField * title_field = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK);

  new FXLabel(matrix,"Artist",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox * artistbox = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
  new FXLabel(matrix,"Album",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXComboBox * albumbox = new FXComboBox(matrix,30,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_THICK|FRAME_SUNKEN);
  new FXLabel(matrix,"Genre",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXHorizontalFrame * hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);

  yearfield = new FXTextField(hframe,4,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_RIGHT|TEXTFIELD_INTEGER|TEXTFIELD_LIMITED);
  new FXLabel(hframe,"Year",NULL,LABEL_NORMAL|LAYOUT_CENTER_Y|LAYOUT_RIGHT);
  FXComboBox * genrebox = new FXComboBox(hframe,20,NULL,0,LAYOUT_FILL_X|FRAME_THICK|FRAME_SUNKEN);

  if (tracks.no()>1) {
    new FXFrame(matrix,FRAME_NONE);
    autonumber = new FXCheckButton(matrix,"Assign track number based on selection order.",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
    }

  new FXFrame(matrix,FRAME_NONE);
  FXCheckButton * sync = new FXCheckButton(matrix,"Update Tag in File",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
  new FXFrame(matrix,FRAME_NONE);
  FXCheckButton * syncfilename = new FXCheckButton(matrix,"Update Filename",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);

  sync->setCheck(FXApp::instance()->reg().readBoolEntry("update files","edit-track",false));

  if (spinner) spinner->setValue(info.no);

  db->listAlbums(albumbox,db->getTrackAlbum(tracks[0]));
  albumbox->setSortFunc(album_list_sort);
  albumbox->setNumVisible(FXMIN(10,albumbox->getNumItems()));
  albumbox->sortItems();
  albumbox->setCurrentItem(-1);

  db->listArtists(artistbox);
  artistbox->setSortFunc(artist_list_sort);
  artistbox->setNumVisible(FXMIN(10,artistbox->getNumItems()));
  artistbox->sortItems();
  artistbox->setCurrentItem(-1);

  db->listGenres(genrebox);
  genrebox->setSortFunc(genre_list_sort);
  genrebox->setNumVisible(FXMIN(10,genrebox->getNumItems()));
  genrebox->sortItems();
  genrebox->setCurrentItem(-1);


  if (tracks.no()==1) {
    albumbox->setCurrentItem(albumbox->findItem(info.album));
    artistbox->setCurrentItem(artistbox->findItem(info.artist));
    genrebox->setCurrentItem(genrebox->findItem(info.genre));
    yearfield->setText(GMStringVal(info.year));
    title_field->setText(info.title);
    }

  if (dialog.execute()) {

    db->beginEdit();

    if (spinner && info.no!=spinner->getValue()) {
      db->setTrackNo(tracks[0],spinner->getValue());
      info.no = spinner->getValue();
      changed=true;
      }
    else  if (autonumber && autonumber->getCheck()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackNo(tracks[i],i+1);
        }
      changed=true;
      }
/*




    if (title_field && info.title!=title_field->getText() && !title_field->getText().empty()) {
      for (i=0;i<tracks.no();i++){
        db->setTrackTitle(tracks[i],title_field->getText());
        }
      changed=true;
      db->setTrackTitle(tracks[0],title_field->getText());
      info.title = title_field->getText();
      changed=true;
*/

    if (!yearfield->getText().empty()) {
#if FOXVERSION >= FXVERSION(1,7,12)
      FXint year=yearfield->getText().toInt();
#else
      FXint year=FXIntVal(yearfield->getText());
#endif
      if ((tracks.no()>1) || (tracks.no()==1 && info.year!=year)){
        for (i=0;i<tracks.no();i++){
          db->setTrackYear(tracks[i],year);
          }
        changed=true;
        }
      }


    field=title_field->getText().trim().simplify();
    if ((tracks.no()==1 && info.title!=field && !field.empty()) || (tracks.no()>1 && !field.empty()) ) {
      for (i=0;i<tracks.no();i++){
        db->setTrackTitle(tracks[i],field);
        }
      changed=true;
      }

    field=genrebox->getText().trim().simplify();
    if ((tracks.no()==1 && info.genre!=field && !field.empty()) || (tracks.no()>1 && !field.empty())){
      for (i=0;i<tracks.no();i++){
        db->setTrackGenre(tracks[i],field);
        }
      changed=true;
      }


    field=artistbox->getText().trim().simplify();
    album=albumbox->getText().trim().simplify();
    // If we need to change the artist name, we already update/create a new album entry
    if ((tracks.no()==1 && info.artist!=field && !field.empty()) || (tracks.no()>1 && !field.empty())){
      for (i=0;i<tracks.no();i++){
        // albumbox may be empty, in which case we use the existing album name.
        db->setTrackArtist(tracks[i],field,album);
        }
      changed=true;
      }
    else if ((tracks.no()==1 && info.album!=album && !album.empty()) || (tracks.no()>1 && !album.empty())){
      for (i=0;i<tracks.no();i++){
        db->setTrackAlbum(tracks[i],album);
        }
      changed=true;
      }

    /// Update GUI / write to files
    if (changed) {
      if (sync->getCheck()) {
        GMTrack info;
        for (i=0;i<tracks.no();i++){
          if (!db->getTrack(tracks[i],info)) break;
          GMTag::sync(info);
          }
        }
      FXApp::instance()->reg().writeBoolEntry("update files","edit-track",sync->getCheck());
      }


    if (syncfilename->getCheck()){
      updateTrackFilenames(tracks);
      }

    db->endEdit();

    if (changed) GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }
#endif

void GMDatabaseSource::getTrackFilenames(const FXIntList & tracks,FXStringList & files){
  files.no(tracks.no());
  for (int i=0;i<tracks.no();i++){
    files[i]=db->getTrackFilename(tracks[i]);
    }
  }


void GMDatabaseSource::removeFiles(const FXStringList & files) {
  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Remove Audio Files?",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,600,400,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Remove Audio Files...","The following audio files are going to be removed");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Remove ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXVerticalFrame * sunken = new FXVerticalFrame(main,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0);
  FXList * list = new FXList(sunken,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);

  for (int i=0;i<files.no();i++) {
    list->appendItem(files[i]);
    }
  if (dialog.execute()) {
    for (int i=0;i<files.no();i++){
      FXFile::remove(files[i]);
      }
    }
  }

void GMDatabaseSource::updateTrackFilenames(FXIntList & tracks) {
  register FXint i=0;
	FXint numchanges=0;
  FXString mrl;
  GMTrack trackinfo;
  FXStringList newmrls;
  FXStringList oldmrls;

  if (!GMPlayerManager::instance()->getPreferences().export_format_template.contains("%T")) {
    FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Invalid Template","The provided template is invalid. The track title %%T needs to be specified.\nPlease fix the filename template in the preference panel.");
    return;
    }

  FXbool utf8=false;
  FXTextCodec * codec = GMFilename::findcodec(GMPlayerManager::instance()->getPreferences().export_encoding,utf8);
  FXuint options=0;

  if (GMPlayerManager::instance()->getPreferences().export_lowercase)
    options|=GMFilename::LOWERCASE;

  if (GMPlayerManager::instance()->getPreferences().export_lowercase_extension)
    options|=GMFilename::LOWERCASE_EXTENSION;

  if (GMPlayerManager::instance()->getPreferences().export_underscore)
    options|=GMFilename::NOSPACES;

  /// Create New Mrls.
  for (i=0;i<tracks.no();i++) {
    if (!db->getTrack(tracks[i],trackinfo)) {
      FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Database Error","Oops. Database Error");
      return;
      }
    if (GMFilename::create(mrl,trackinfo,GMPlayerManager::instance()->getPreferences().export_format_template,options,codec,utf8) && mrl!=trackinfo.mrl) {
      newmrls.append(mrl);
      oldmrls.append(trackinfo.mrl);
			numchanges++;
      }
    else {
      newmrls.append(FXString::null);
      oldmrls.append(FXString::null);
      }
    }

  if (numchanges==0){
		FXMessageBox::information(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"No changes","Filenames did not require any changes");
    delete codec;
    return;
    }


  /// Ask For Permission
  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Rename Audio Files?",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,600,400,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Renaming Audio Files...","The following audio files are going to be renamed");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Rename ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  FXVerticalFrame * sunken = new FXVerticalFrame(main,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_SUNKEN|FRAME_THICK,0,0,0,0,0,0,0,0);
  FXIconList * list = new FXIconList(sunken,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y);
  list->appendHeader("Old mrl",NULL,100);
  list->appendHeader("New mrl",NULL,500);

  for (i=0;i<tracks.no();i++) {
    if (!newmrls[i].empty()) {
      list->appendItem(oldmrls[i]+"\t"+newmrls[i]);
      }
    }

  if (dialog.execute()) {
    for (i=0;i<tracks.no();i++){
      if (!newmrls[i].empty()) {
        if (GMFilename::createDirectory(FXPath::directory(newmrls[i]))) {
          if (!FXStat::exists(newmrls[i])) {
            if (FXFile::rename(oldmrls[i],newmrls[i])){
              db->setTrackFilename(tracks[i],newmrls[i]);
              }
            else {
              if (i+1<tracks.no()) {
                if (FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_YES_NO,"Unable to rename file","Unable to rename:\n%s\n\nto:%s\nContinue renaming files?",oldmrls[i].text(),newmrls[i].text())==MBOX_CLICKED_NO)
                  break;
                }
              else {
                FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Unable to rename file","Unable to rename:\n%s\n\nto:%s",oldmrls[i].text(),newmrls[i].text());
                }
              }
            }
          }
        }
      }
    }
  delete codec;
  }




long GMDatabaseSource::onCmdExport(FXObject*,FXSelector,void*){
  const FXchar patterns[]="XML Shareable Playlist Format (*.xspf)\nPLS (*.pls)\nExtended M3U (*.m3u)\nM3U (*.m3u)\nText Comma-Separated (*.csv)";
  FXString searchdir = FXApp::instance()->reg().readStringEntry("Settings","last-export-directory",FXSystem::getHomeDirectory().text());
  FXString title;
  if (getPlayList()==-1)
    title="Export Main Library";
  else
    title="Export Play List";
  FXFileDialog dialog(GMPlayerManager::instance()->getMainWindow(),title);
  dialog.setDirectory(searchdir);
  dialog.setSelectMode(SELECTFILE_ANY);
  dialog.setPatternList(patterns);
  dialog.setCurrentPattern(0);
  dialog.setMatchMode(FILEMATCH_CASEFOLD);
  if (dialog.execute()){
    if (FXStat::exists(dialog.getFilename())){
      if (FXMessageBox::question(GMPlayerManager::instance()->getMainWindow(),MBOX_YES_NO,"Overwrite File?","File already exists. Would you like to overwrite it?")!=MBOX_CLICKED_YES)
        return 1;
      }
   FXApp::instance()->reg().writeStringEntry("Settings","last-export-directory",dialog.getDirectory().text());
    FXApp::instance()->beginWaitCursor();
    switch(dialog.getCurrentPattern()){
      case 0: db->exportList(dialog.getFilename(),getPlayList(),PLAYLIST_XSPF); break;
      case 1: db->exportList(dialog.getFilename(),getPlayList(),PLAYLIST_PLS); break;
      case 2: db->exportList(dialog.getFilename(),getPlayList(),PLAYLIST_M3U_EXTENDED); break;
      case 3: db->exportList(dialog.getFilename(),getPlayList(),PLAYLIST_M3U); break;
      case 4: db->exportList(dialog.getFilename(),getPlayList(),PLAYLIST_CSV); break;
      }
    FXApp::instance()->endWaitCursor();
    }
  return 1;
  }

long GMDatabaseSource::onUpdExport(FXObject*sender,FXSelector,void*){
  sender->handle(this,FXSEL(SEL_COMMAND,FXWindow::ID_ENABLE),NULL);
  return 1;
  }



long GMDatabaseSource::onCmdExportTracks(FXObject*,FXSelector sel,void*){
  const FXuint labelstyle=LAYOUT_CENTER_Y|LABEL_NORMAL|LAYOUT_RIGHT;
  //const FXuint textfieldstyle=TEXTFIELD_ENTER_ONLY|LAYOUT_FILL_X|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN;


  FXIntList tracks;
  if (FXSELID(sel)==ID_EXPORT_TRACK) {
    GMPlayerManager::instance()->getTrackView()->getSelectedTracks(tracks);
    }
  else {
    GMPlayerManager::instance()->getTrackView()->getTracks(tracks);
    }
  if (tracks.no()==0) return 1;

  FXString title;
  FXString subtitle;

  switch(FXSELID(sel)){
    case ID_EXPORT_GENRE: title="Export Genre";
                          subtitle="Export tracks with genre to destination directory.";
                          break;
    case ID_EXPORT_ARTIST:title="Export Artists";
                          subtitle="Export tracks from artist to destination directory.";
                          break;
    case ID_EXPORT_ALBUM: title="Export Albums";
                          subtitle="Export tracks from album to destination directory.";
                          break;
    case ID_EXPORT_TRACK: title="Export Tracks";
                          subtitle="Export tracks to destination directory.";
                          break;
    default: FXASSERT(0); break;
    }

  FXTextField * textfield;
  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),title,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,title,subtitle,NULL);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Export ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXMatrix * matrix = new FXMatrix(main,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X,0,0,0,0,0,0,4,0);
/*
  new FXLabel(matrix,"Directory:",NULL,labelstyle);
  FXHorizontalFrame * hframe = new FXHorizontalFrame(matrix,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN,0,0,0,0,0,0,0,0);
  FXTextField * textfield = new FXTextField(hframe,20,NULL,0,LAYOUT_FILL_X|TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK);
  new FXButton(hframe,UTF8_ELLIPSIS "\tSelect Directory",NULL,NULL,0);

*/
  new FXLabel(matrix,"Template:",NULL,labelstyle);
  textfield = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|TEXTFIELD_ENTER_ONLY|FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN);
 // textfield->setFont(font_fixed);

  new FXLabel(matrix,"Encoding:",NULL,labelstyle);
  FXListBox * list_codecs = new FXListBox(matrix,NULL,0,FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_COLUMN);
  for (int i=0;gmcodecnames[i]!=NULL;i++)
    list_codecs->appendItem(gmcodecnames[i]);
  list_codecs->setNumVisible(9);

  new FXLabel(matrix,"Options:",NULL,labelstyle);
  new FXCheckButton(matrix,"Replace spaces with underscores",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
  new FXFrame(matrix,FRAME_NONE);
  new FXCheckButton(matrix,"Lower case",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);
  new FXFrame(matrix,FRAME_NONE);
  new FXCheckButton(matrix,"Lower case extension",NULL,0,LAYOUT_FILL_COLUMN|CHECKBUTTON_NORMAL);

  if (dialog.execute()){
    }
  return 1;
  }























long GMDatabaseSource::onCmdDelete(FXObject*,FXSelector sel,void*){
  FXIntList tracks;
  FXIntList selected;
  FXStringList files;
  if (FXSELID(sel)==ID_DELETE_TRACK) {
    GMPlayerManager::instance()->getTrackView()->getSelectedTracks(tracks);
    }
  else {
    GMPlayerManager::instance()->getTrackView()->getTracks(tracks);
    }
  if (tracks.no()==0) return 1;

  FXString title;
  FXString subtitle;

  switch(FXSELID(sel)){
    case ID_DELETE_GENRE: title="Remove Genre?";
                          subtitle="Remove tracks with genre from library?";
                          GMPlayerManager::instance()->getTrackView()->getSelectedGenres(selected);
                          break;
    case ID_DELETE_ARTIST:title="Remove Artist?";
                          subtitle="Remove tracks from artist from library?";
                          GMPlayerManager::instance()->getTrackView()->getSelectedArtists(selected);
                          break;
    case ID_DELETE_ALBUM: title="Remove Album?";
                          subtitle="Remove tracks from album from library?";
                          GMPlayerManager::instance()->getTrackView()->getSelectedAlbums(selected);
                          break;
    case ID_DELETE_TRACK: title="Remove Track(s)?";
                          subtitle="Remove track(s) from library?";
                          break;
    default: FXASSERT(0); break;
    }

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),title,DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,title,subtitle,NULL);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Remove ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXCheckButton * from_disk = new FXCheckButton(main,"Remove tracks from disk");

  if (dialog.execute()){

    db->beginDelete();

    if (from_disk->getCheck())
      getTrackFilenames(tracks,files);


    switch(FXSELID(sel)){
      case ID_DELETE_GENRE:
        if (!db->removeGenre(selected[0]))
          FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Library Error","Unable to remove genre from the library");
        break;
      case ID_DELETE_ARTIST:
        if (!db->removeArtist(selected[0]))
          FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Library Error","Unable to remove artist from the library");
        break;
      case ID_DELETE_ALBUM:
        if (!db->removeAlbum(selected[0]))
          FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Library Error","Unable to remove album from the library");
        break;
      case ID_DELETE_TRACK:
        for (int i=0;i<tracks.no();i++){
          if (!db->removeTrack(tracks[i])){
            FXMessageBox::error(GMPlayerManager::instance()->getMainWindow(),MBOX_OK,"Library Error","Unable to remove track from the library.");
            }
          }
        break;
      default: FXASSERT(0); break;
      }

    if (from_disk->getCheck())
      removeFiles(files);

    db->endDelete();
    GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }


long GMDatabaseSource::onCmdCopyArtistAlbum(FXObject*,FXSelector,void*){
  FXDragType types[4]={GMClipboard::trackdatabase,GMClipboard::kdeclipboard,GMClipboard::gnomeclipboard,FXWindow::urilistType};
  GMDatabaseClipboardData * data = new GMDatabaseClipboardData;
  if (GMClipboard::instance()->acquire(this,types,4,data)){
    FXApp::instance()->beginWaitCursor();
    data->db=db;
    GMPlayerManager::instance()->getTrackView()->getTracks(data->tracks);
    FXApp::instance()->endWaitCursor();
    }
  else {
    delete data;
    FXApp::instance()->beep();
    }
  return 1;
  }

long GMDatabaseSource::onCmdCopyTrack(FXObject*,FXSelector,void*){
  FXDragType types[4]={GMClipboard::trackdatabase,GMClipboard::kdeclipboard,GMClipboard::gnomeclipboard,FXWindow::urilistType};
  GMDatabaseClipboardData * data = new GMDatabaseClipboardData;
  if (GMClipboard::instance()->acquire(this,types,4,data)){
    FXApp::instance()->beginWaitCursor();
    data->db=db;
    GMPlayerManager::instance()->getTrackView()->getSelectedTracks(data->tracks);
    FXApp::instance()->endWaitCursor();
    }
  else {
    delete data;
    FXApp::instance()->beep();
    }
  return 1;
  }



long GMDatabaseSource::onCmdRequestArtistAlbum(FXObject*sender,FXSelector,void*ptr){
  FXEvent* event=(FXEvent*)ptr;
  FXWindow*window=(FXWindow*)sender;
  if(event->target==GMClipboard::urilistType){
    FXIntList tracks;
    FXString url;
    GMPlayerManager::instance()->getTrackView()->getTracks(tracks);
    tracklist2urilist(db,url,tracks);
    window->setDNDData(FROM_DRAGNDROP,event->target,url);
    return 1;
    }
  else if (event->target==GMClipboard::kdeclipboard){
    window->setDNDData(FROM_DRAGNDROP,event->target,"0"); // copy
    return 1;
    }
  return 0;
  }

long GMDatabaseSource::onCmdRequestTrack(FXObject*sender,FXSelector,void*ptr){
  FXEvent* event=(FXEvent*)ptr;
  FXWindow*window=(FXWindow*)sender;
  if(event->target==GMClipboard::urilistType){
    FXIntList tracks;
    FXString url;
    GMPlayerManager::instance()->getTrackView()->getSelectedTracks(tracks);
    tracklist2urilist(db,url,tracks);
    window->setDNDData(FROM_DRAGNDROP,event->target,url);
    return 1;
    }
  else if (event->target==GMClipboard::kdeclipboard){
    window->setDNDData(FROM_DRAGNDROP,event->target,"0"); // copy
    return 1;
    }
  return 0;
  }






long GMDatabaseSource::onCmdDrop(FXObject*sender,FXSelector,void*){
  FXWindow * window=(FXWindow*)sender;
  FXString     files;
  FXStringList filelist;
  FXbool from_kde=false;
  FXbool from_uri=false;
  FXDragType * types;
  FXuint       ntypes;

  if (getPlayList()==-1 && FXApp::instance()->getDragWindow()) return 0;

  if (window->inquireDNDTypes(FROM_DRAGNDROP,types,ntypes)){
    for (FXuint i=0;i<ntypes;i++){
    fxmessage("[%d] %s\n",i,FXApp::instance()->getDragTypeName(types[i]).text());

      if (types[i]==GMClipboard::kdeclipboard) from_kde=true;
      else if (types[i]==FXWindow::urilistType) from_uri=true;
     }

//    else if (from_kde && from_uri) {
//      }
    if (from_uri) {
      if (window->getDNDData(FROM_DRAGNDROP,FXWindow::urilistType,files)){
        uri2filelist(files,filelist);
        }
      }

    window->dropFinished(DRAG_LINK);

    if (filelist.no() && GMPlayerManager::instance()->getMainWindow()->question("Import Files?","Would you like import the pasted files and/or directories into the Music Library?","Yes","No")) {
      GMPlayerManager::instance()->stop();
      GMSearchDialog searchdialog(GMPlayerManager::instance()->getMainWindow(),filelist,getPlayList(),GMPlayerManager::instance()->getPreferences(),GMPlayerManager::instance()->getMainWindow()->getThickFont());
      searchdialog.execute();
      GMPlayerManager::instance()->getTrackView()->refresh();
      }
    else {
      FXApp::instance()->beep();
      }
    }
  return 1;
  }



long GMDatabaseSource::onCmdPaste(FXObject*,FXSelector,void*){
  FXString     files;
  FXStringList filelist;
  FXDragType * types;
  FXuint       num;
//  FXbool       from_db=false;
  FXbool       from_kde=false;
  FXbool       from_gnome=false;
  FXbool       from_uri=false;

  GMClipboard * clipboard = GMClipboard::instance();

  if (clipboard->inquireDNDTypes(FROM_CLIPBOARD,types,num)){

    for (FXuint i=0;i<num;i++) {
      if (types[i]==GMClipboard::kdeclipboard)
        from_kde=true;
      else if (types[i]==GMClipboard::gnomeclipboard)
        from_gnome=true;
      else if (types[i]==FXWindow::urilistType)
        from_uri=true;
      }

    freeElms(types);

    if ((from_gnome || from_uri || from_kde) && clipboard->hasClipboard() ) {
      return 1;
      }

    if (from_gnome) {
      if (clipboard->getDNDData(FROM_CLIPBOARD,GMClipboard::gnomeclipboard,files)){
        gnomeclipboard2filelist(files,filelist);
        }
      }
    else if (from_kde && from_uri) {
      if (clipboard->getDNDData(FROM_CLIPBOARD,GMClipboard::kdeclipboard,files)){
#ifdef DEBUG
        if (files=="1") {
          fxmessage("We do not cut files...\n");
          }
#endif
        }
      if (clipboard->getDNDData(FROM_CLIPBOARD,FXWindow::urilistType,files)){
        uri2filelist(files,filelist);
        }
      }

    if (filelist.no() && GMPlayerManager::instance()->getMainWindow()->question("Import Files?","Would you like import the pasted files and/or directories into the Music Library?","Yes","No")) {
      GMPlayerManager::instance()->stop();
      GMSearchDialog searchdialog(GMPlayerManager::instance()->getMainWindow(),filelist,getPlayList(),GMPlayerManager::instance()->getPreferences(),GMPlayerManager::instance()->getMainWindow()->getThickFont());
      searchdialog.execute();
      GMPlayerManager::instance()->getTrackView()->refresh();
      }
    else {
      FXApp::instance()->beep();
      }

    return 1;
    }
  return 0;
  }

long GMDatabaseSource::onUpdPaste(FXObject*,FXSelector,void*){
  return 1;
  }


long GMDatabaseSource::onCmdNewPlayList(FXObject*,FXSelector,void*){
  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Create Playlist",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Create Playlist","Specify name of the new playlist",NULL);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Create ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXMatrix * matrix = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS);
  new FXLabel(matrix,"Name",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXTextField * name_field = new FXTextField(matrix,20,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_COLUMN|FRAME_SUNKEN|FRAME_THICK);
  name_field->setText("New Playlist");
  if (dialog.execute()) {
    FXString label= name_field->getText().trim();
    if (!label.empty()) {
      FXint pl;
      db->insertPlaylist(label,pl);
      GMPlayerManager::instance()->insertSource(new GMPlayListSource(db,pl));
      GMPlayerManager::instance()->getSourceView()->refresh();
      }
    }
  return 1;
  }

long GMDatabaseSource::onCmdClear(FXObject*,FXSelector,void*){
  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Clear Music Library?",DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE|DECOR_CLOSE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Clear Music Library?","Remove all tracks from the music library?",NULL);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox," &Remove All ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXButton(closebox," &Cancel ",NULL,&dialog,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);
  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,30,20,10,10);
  FXCheckButton * playlist_check = new FXCheckButton(main,"Keep play lists");
  if (dialog.execute()){
    GMPlayerManager::instance()->stop();
    db->clearTracks(!playlist_check->getCheck());
    GMPlayerManager::instance()->removePlayListSources();
    GMPlayerManager::instance()->getSourceView()->refresh();
    GMPlayerManager::instance()->getTrackView()->refresh();
    }
  return 1;
  }


long GMDatabaseSource::onCmdInfo(FXObject*,FXSelector,void*){
  FXint num_tracks = db->getNumTracks();
  FXint num_albums= db->getNumAlbums();
  FXint num_artists= db->getNumArtists();
  FXint time = db->getTotalTime();
  FXint	days = (FXint) floor((double)time/86400.0);
  time -= (FXint) (86400.0*days);
  FXint	hours = (FXint) floor((double)time/3600.0);
  time -= (FXint) (3600.0*hours);
  FXint	minutes = (FXint) floor((double)time/60.0);
  time -= (FXint) (60.0*minutes);
  FXint	seconds = (FXint) floor((double)time);

  FXDialogBox dialog(GMPlayerManager::instance()->getMainWindow(),"Music Library Information",DECOR_TITLE|DECOR_BORDER|DECOR_CLOSE,0,0,0,0,0,0,0,0,0,0);
  GMPlayerManager::instance()->getMainWindow()->create_dialog_header(&dialog,"Music Library Information","You music collection consists of...");
  FXHorizontalFrame *closebox=new FXHorizontalFrame(&dialog,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox,"   &Close   ",NULL,&dialog,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_CENTER_X|FRAME_RAISED|FRAME_THICK,0,0,0,0, 15,15);
  new FXSeparator(&dialog,SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);


  FXVerticalFrame * main = new FXVerticalFrame(&dialog,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,20,10,5,5);
  FXMatrix * matrix = new FXMatrix(main,2,LAYOUT_FILL_X|MATRIX_BY_COLUMNS);
  FXLabel * label = new FXLabel(matrix,"Tracks:",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);

  label = new FXLabel(matrix,GMStringVal(num_tracks),NULL,LABEL_NORMAL|JUSTIFY_LEFT);
  label->setTextColor(FXRGB(0,0,255));

  new FXLabel(matrix,"Artists:",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  label = new FXLabel(matrix,GMStringVal(num_artists),NULL,LABEL_NORMAL|JUSTIFY_LEFT);
  label->setTextColor(FXRGB(0,0,255));

  new FXLabel(matrix,"Albums:",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  label = new FXLabel(matrix,GMStringVal(num_albums),NULL,LABEL_NORMAL|JUSTIFY_LEFT);
  label->setTextColor(FXRGB(0,0,255));

  new FXLabel(matrix,"Total Time:",NULL,LABEL_NORMAL|LAYOUT_RIGHT|LAYOUT_CENTER_Y);
  FXString duration;
  if (days) {
    if (days>=2)
      duration+=GMStringFormat("%d days",days);
    else
      duration+=GMStringFormat("%d day",days);

    if (seconds==0 && minutes==0 && hours) duration+="."; else duration+=", ";
    }
  if (hours) {
    if (hours>=2)
      duration+=GMStringFormat("%d hours",hours);
    else
      duration+=GMStringFormat("%d hour",hours);

    if (seconds==0 && minutes==0) duration+="."; else duration+=", ";
    }
  if (minutes) {
    if (minutes>=2)
      duration+=GMStringFormat("%d minutes",minutes);
    else
      duration+=GMStringFormat("%d minute",minutes);

    if (seconds==0) duration+="."; else duration+=", ";
    }
  if (seconds) {
    if (seconds>=2)
      duration+=GMStringFormat("%d seconds. ",seconds);
    else
      duration+=GMStringFormat("%d second.",seconds);
    }
  new FXLabel(matrix,duration,NULL,LABEL_NORMAL|JUSTIFY_LEFT);


  dialog.execute();
  return 1;
  }


long GMDatabaseSource::onCmdTrackPlayed(FXObject*,FXSelector,void*) {
  FXTRACE((60,"%s::onCmdTrackPlayed\n",getClassName()));
  FXASSERT(current_track>=0);
  FXlong timestamp = (FXlong)FXThread::time();
  db->playedTrack(current_track,timestamp);
  GMTrack info;
  if (getTrack(info) && GMPlayerManager::instance()->getAudioScrobbler())
    GMPlayerManager::instance()->getAudioScrobbler()->submit(timestamp,info);
  return 1;
  }

