/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include "common.h"
#include "GMIconTheme.h"
#include "icons.h"

#ifdef HAVE_EXTERNAL_ICONS
#define QMAKESTR(x) #x
#define MAKESTR(x) QMAKESTR(x)
#define DEFAULT_ICON_DIRECTORY MAKESTR(DEFAULT_ICON_PATH)
#endif

GMIconTheme * GMIconTheme::me=NULL;

GMIconTheme::GMIconTheme() {
  FXASSERT(me==NULL);
  me=this;

  icon_about=NULL;
  }

GMIconTheme::~GMIconTheme() {
  }

GMIconTheme * GMIconTheme::instance(){
  return me;
  }

#ifdef HAVE_EXTERNAL_ICONS
FXIcon * GMIconTheme::loadIcon(FXIconSource & src,const FXString & basedir,const FXString & size,const char * key,const char * value) {
  FXIcon * icon = src.loadIconFile(basedir+PATHSEPSTRING+size+PATHSEPSTRING+FXApp::instance()->reg().readStringEntry("user-icons",key,value));
  if (!icon) {
#ifdef DEBUG
		fxmessage("Failed to load: %s\n",FXString(basedir+PATHSEPSTRING+size+PATHSEPSTRING+FXApp::instance()->reg().readStringEntry("user-icons",key,value)).text());
#endif
    icon = src.loadIconData(x16_status_iconloaderror_gif,"gif");
    FXASSERT(icon);
    FXApp::instance()->reg().writeStringEntry("user-icons",key,value);
    }
  return icon;
  }
#endif


void GMIconTheme::loadVolumeIcons(FXIconPtr & muted,FXIconPtr & low,FXIconPtr & medium,FXIconPtr &high){
  FXIconSource source(FXApp::instance());
#ifdef HAVE_EXTERNAL_ICONS
  const FXString base = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-dir",DEFAULT_ICON_DIRECTORY);
  const FXString big     = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-size-big","22x22");
  const FXString small   = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-size-small","16x16"); 
	high   = loadIcon(source,base,small,"status-audio-volume-high","status/audio-volume-high.png");
  medium = loadIcon(source,base,small,"status-audio-volume-medium","status/audio-volume-medium.png");
  low    = loadIcon(source,base,small,"status-audio-volume-low","status/audio-volume-low.png");
  muted  = loadIcon(source,base,small,"status-audio-volume-muted","status/audio-volume-muted.png");
#else /* HAVE_BUILDIN_ICONS */
 	high   = source.loadIconData(x16_status_audiovolumehigh_png,"png");
  medium = source.loadIconData(x16_status_audiovolumemedium_png,"png");
  low    = source.loadIconData(x16_status_audiovolumelow_png,"png");
  muted  = source.loadIconData(x16_status_audiovolumemuted_png,"png");
#endif
  high->blend(FXApp::instance()->getBaseColor());
  medium->blend(FXApp::instance()->getBaseColor());
  low->blend(FXApp::instance()->getBaseColor());
  muted->blend(FXApp::instance()->getBaseColor());
  high->create();
  medium->create();
  low->create();
  muted->create();
	}


void GMIconTheme::load() {
  FXIconSource source(FXApp::instance());
#ifdef HAVE_EXTERNAL_ICONS
  const FXString base    = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY);
  const FXString big     = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-big","22x22");
  const FXString small   = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-small","16x16");

  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-dir",base.text());
  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-size-big",big.text());
  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-size-small",small.text());

  icon_copy           = loadIcon(source,base,small,"actions-edit-copy","actions/edit-copy.png");
  icon_cut            = loadIcon(source,base,small,"actions-edit-cut","actions/edit-cut.png");
  icon_paste          = loadIcon(source,base,small,"actions-edit-paste","actions/edit-paste.png");
  icon_delete         = loadIcon(source,base,small,"actions-edit-delete","actions/edit-delete.png");
  icon_play           = loadIcon(source,base,small,"actions-media-playback-start","actions/media-playback-start.png");
  icon_pause          = loadIcon(source,base,small,"actions-media-playback-pause","actions/media-playback-pause.png");
  icon_next           = loadIcon(source,base,small,"actions-media-skip-forward","actions/media-skip-forward.png");
  icon_prev           = loadIcon(source,base,small,"actions-media-skip-backward","actions/media-skip-backward.png");
  icon_stop           = loadIcon(source,base,small,"actions-media-playback-stop","actions/media-playback-stop.png");
  icon_import         = loadIcon(source,base,small,"places-folder","places/folder.png");
  icon_importfile     = loadIcon(source,base,small,"actions-document-open","actions/document-open.png");
  icon_exit           = loadIcon(source,base,small,"actions-exit","actions/exit.png");
  icon_close          = loadIcon(source,base,small,"actions-window-close","actions/window-close.png");
  icon_find           = loadIcon(source,base,small,"actions-edit-find","actions/edit-find.png");
  icon_album          = loadIcon(source,base,small,"devices-media-optical","devices/media-optical.png");
  icon_artist         = loadIcon(source,base,small,"apps-system-users","apps/system-users.png");
  icon_genre          = loadIcon(source,base,small,"actions-bookmark-new","actions/bookmark-new.png");
  icon_export         = loadIcon(source,base,small,"actions-document-save","actions/document-save.png");

  icon_homepage       = loadIcon(source,base,small,"categories-applications-internet","categories/applications-internet.png");
  icon_info           = loadIcon(source,base,small,"apps-help-browser","apps/help-browser.png");
  icon_source_library = loadIcon(source,base,big,  "places-user-home","places/user-home.png");
  icon_source_internetradio = loadIcon(source,base,big,"categories-applications-internet","categories/applications-internet.png");
  icon_settings       = loadIcon(source,base,small,"categories-preferences-apps","categories/preferences-desktop.png");
  icon_edit           = loadIcon(source,base,small,"apps-accessorries-text-editor","apps/accessories-text-editor.png");
#else
  icon_copy           = source.loadIconData(x16_actions_editcopy_png,"png");
  icon_cut            = source.loadIconData(x16_actions_editcut_png,"png");
  icon_paste          = source.loadIconData(x16_actions_editpaste_png,"png");
  icon_delete         = source.loadIconData(x16_actions_editdelete_png,"png");
  icon_find           = source.loadIconData(x16_actions_editfind_png,"png");
  icon_play           = source.loadIconData(x16_actions_mediaplaybackstart_gif,"gif");
  icon_pause          = source.loadIconData(x16_actions_mediaplaybackpause_gif,"gif");
  icon_next           = source.loadIconData(x16_actions_mediaskipforward_gif,"gif");
  icon_prev           = source.loadIconData(x16_actions_mediaskipbackward_gif,"gif");
  icon_stop           = source.loadIconData(x16_actions_mediaplaybackstop_gif,"gif");
  icon_import         = source.loadIconData(x16_places_folder_png,"png");
  icon_importfile     = source.loadIconData(x16_actions_documentopen_png,"png");
  icon_exit           = source.loadIconData(x16_actions_exit_png,"png");
  icon_close          = source.loadIconData(x16_actions_windowclose_png,"png");
  icon_album          = source.loadIconData(x16_devices_mediaoptical_png,"png");
  icon_artist         = source.loadIconData(x16_apps_systemusers_png,"png");
  icon_genre          = source.loadIconData(x16_actions_bookmarknew_png,"png");
  icon_export         = source.loadIconData(x16_actions_documentsave_png,"png");
  icon_homepage       = source.loadIconData(x16_apps_internetwebbrowser_png,"png");
  icon_info           = source.loadIconData(x16_apps_helpbrowser_png,"png");
  icon_source_library = source.loadIconData(x22_places_userhome_png,"png");
  icon_source_internetradio =  source.loadIconData(x22_apps_internetwebbrowser_png,"png");
  icon_settings       = source.loadIconData(x16_categories_preferencesdesktop_png,"png");
  icon_edit           = source.loadIconData(x16_apps_accessoriestexteditor_png,"png");
#endif

  icon_sort           = source.loadIconData(x16_actions_viewsortdescending_png,"png"); // found in gnome, not tango
  icon_playlist       = source.loadIconData(x16_mimetypes_sourceplaylist_png,"png");
  icon_source_playlist= source.loadIconData(x22_mimetypes_sourceplaylist_png,"png");
  icon_applogo        = source.loadIconData(x32_apps_musicmanager_png,"png");
  icon_applogo_small  = source.loadIconData(x16_apps_musicmanager_png,"png");
  icon_cover          = source.loadIconData(cover_png,"png");

  FXASSERT(icon_copy);
  FXASSERT(icon_cut);
  FXASSERT(icon_paste);
  FXASSERT(icon_delete);
  FXASSERT(icon_sort);
  FXASSERT(icon_play);
  FXASSERT(icon_pause);
  FXASSERT(icon_next);
  FXASSERT(icon_prev);
  FXASSERT(icon_stop);
  FXASSERT(icon_import);
  FXASSERT(icon_importfile);
  FXASSERT(icon_exit);
  FXASSERT(icon_close);
  FXASSERT(icon_album);
  FXASSERT(icon_artist);
  FXASSERT(icon_genre);
  FXASSERT(icon_export);
  FXASSERT(icon_source_library);
  FXASSERT(icon_playlist);
  FXASSERT(icon_source_playlist);
  FXASSERT(icon_source_internetradio);
  FXASSERT(icon_homepage);
  FXASSERT(icon_info);
  FXASSERT(icon_settings);
  FXASSERT(icon_edit);
  FXASSERT(icon_find);
  FXASSERT(icon_cover);

  icon_copy->blend(FXApp::instance()->getBaseColor());
  icon_cut->blend(FXApp::instance()->getBaseColor());
  icon_paste->blend(FXApp::instance()->getBaseColor());

#if FOXVERSION >= FXVERSION(1,7,13)
  icon_copy->setThresholdValue(700);
  icon_cut->setThresholdValue(500);
  icon_paste->setThresholdValue(500);
  icon_find->setThresholdValue(500);
#endif

  icon_delete->blend(FXApp::instance()->getBaseColor());
  icon_sort->blend(FXApp::instance()->getBaseColor());
  icon_play->blend(FXApp::instance()->getBaseColor());
  icon_pause->blend(FXApp::instance()->getBaseColor());
  icon_next->blend(FXApp::instance()->getBaseColor());
  icon_prev->blend(FXApp::instance()->getBaseColor());
  icon_stop->blend(FXApp::instance()->getBaseColor());
  icon_import->blend(FXApp::instance()->getBaseColor());
  icon_importfile->blend(FXApp::instance()->getBaseColor());
  icon_exit->blend(FXApp::instance()->getBaseColor());
  icon_close->blend(FXApp::instance()->getBaseColor());
  icon_album->blend(FXApp::instance()->getBackColor());
  icon_artist->blend(FXApp::instance()->getBackColor());
  icon_genre->blend(FXApp::instance()->getBackColor());
  icon_export->blend(FXApp::instance()->getBackColor());
  icon_homepage->blend(FXApp::instance()->getBaseColor());
  icon_info->blend(FXApp::instance()->getBaseColor());
  icon_settings->blend(FXApp::instance()->getBaseColor());
  icon_edit->blend(FXApp::instance()->getBaseColor());
  icon_find->blend(FXApp::instance()->getBaseColor());

  icon_source_library->blend(FXApp::instance()->getBackColor());
  icon_source_internetradio->blend(FXApp::instance()->getBackColor());
  icon_playlist->blend(FXApp::instance()->getBaseColor());
  icon_source_playlist->blend(FXApp::instance()->getBackColor());
  icon_applogo->blend(FXRGB(255,255,255));
  icon_applogo_small->blend(FXRGB(255,255,255));
  icon_cover->blend(FXRGB(255,255,255));

  icon_album->create();
  icon_artist->create();
  icon_genre->create();
  icon_source_library->create();
  icon_source_playlist->create();
  icon_source_internetradio->create();
  icon_cover->create();
  }


void GMIconTheme::loadToolbar(FXbool large) {
  FXIconSource source(FXApp::instance());

  if (icon_audio_volume_high) {
    delete ((FXIcon*)icon_audio_volume_high);
    delete ((FXIcon*)icon_audio_volume_medium);
    delete ((FXIcon*)icon_audio_volume_low);
    delete ((FXIcon*)icon_audio_volume_muted);
    }

  if (!large && icon_play_big) {
    delete ((FXIcon*)icon_play_big);
    delete ((FXIcon*)icon_pause_big);
    delete ((FXIcon*)icon_next_big);
    delete ((FXIcon*)icon_prev_big);
    delete ((FXIcon*)icon_stop_big);
    icon_play_big=NULL;
    icon_pause_big=NULL;
    icon_next_big=NULL;
    icon_prev_big=NULL;
    icon_stop_big=NULL;
    }

#ifdef HAVE_EXTERNAL_ICONS
  const FXString base = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-dir",DEFAULT_ICON_DIRECTORY);
  const FXString big     = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-size-big","22x22");
  const FXString small   = FXApp::instance()->reg().readStringEntry("user-interface","icon-base-size-small","16x16");

  if (large) {
    icon_audio_volume_high   = loadIcon(source,base,big,"status-audio-volume-high","status/audio-volume-high.png");
    icon_audio_volume_medium = loadIcon(source,base,big,"status-audio-volume-medium","status/audio-volume-medium.png");
    icon_audio_volume_low    = loadIcon(source,base,big,"status-audio-volume-low","status/audio-volume-low.png");
    icon_audio_volume_muted  = loadIcon(source,base,big,"status-audio-volume-muted","status/audio-volume-muted.png");
    icon_play_big            = loadIcon(source,base,big,"actions-media-playback-start","actions/media-playback-start.png");
    icon_pause_big           = loadIcon(source,base,big,"actions-media-playback-pause","actions/media-playback-pause.png");
    icon_next_big            = loadIcon(source,base,big,"actions-media-skip-forward","actions/media-skip-forward.png");
    icon_prev_big            = loadIcon(source,base,big,"actions-media-skip-backward","actions/media-skip-backward.png");
    icon_stop_big            = loadIcon(source,base,big,"actions-media-playback-stop","actions/media-playback-stop.png");
    }
  else {
    icon_audio_volume_high   = loadIcon(source,base,small,"status-audio-volume-high","status/audio-volume-high.png");
    icon_audio_volume_medium = loadIcon(source,base,small,"status-audio-volume-medium","status/audio-volume-medium.png");
    icon_audio_volume_low    = loadIcon(source,base,small,"status-audio-volume-low","status/audio-volume-low.png");
    icon_audio_volume_muted  = loadIcon(source,base,small,"status-audio-volume-muted","status/audio-volume-muted.png");
    }
#else /* HAVE_BUILDIN_ICONS */

  if (large) {
    icon_audio_volume_high   = source.loadIconData(x22_status_audiovolumehigh_png,"png");
    icon_audio_volume_medium = source.loadIconData(x22_status_audiovolumemedium_png,"png");
    icon_audio_volume_low    = source.loadIconData(x22_status_audiovolumelow_png,"png");
    icon_audio_volume_muted  = source.loadIconData(x22_status_audiovolumemuted_png,"png");
    icon_play_big            = source.loadIconData(x22_actions_mediaplaybackstart_png,"png");
    icon_pause_big           = source.loadIconData(x22_actions_mediaplaybackpause_png,"png");
    icon_next_big            = source.loadIconData(x22_actions_mediaskipforward_png,"png");
    icon_prev_big            = source.loadIconData(x22_actions_mediaskipbackward_png,"png");
    icon_stop_big            = source.loadIconData(x22_actions_mediaplaybackstop_png,"png");
    }
  else {
    icon_audio_volume_high   = source.loadIconData(x16_status_audiovolumehigh_png,"png");
    icon_audio_volume_medium = source.loadIconData(x16_status_audiovolumemedium_png,"png");
    icon_audio_volume_low    = source.loadIconData(x16_status_audiovolumelow_png,"png");
    icon_audio_volume_muted  = source.loadIconData(x16_status_audiovolumemuted_png,"png");
    }

#endif

  FXASSERT(icon_audio_volume_high);
  FXASSERT(icon_audio_volume_medium);
  FXASSERT(icon_audio_volume_low);
  FXASSERT(icon_audio_volume_muted);

  if (large) {
    FXASSERT(icon_play_big);
    FXASSERT(icon_pause_big);
    FXASSERT(icon_next_big);
    FXASSERT(icon_prev_big);
    FXASSERT(icon_stop_big);

    icon_play_big->blend(FXApp::instance()->getBaseColor());
    icon_pause_big->blend(FXApp::instance()->getBaseColor());
    icon_next_big->blend(FXApp::instance()->getBaseColor());
    icon_prev_big->blend(FXApp::instance()->getBaseColor());
    icon_stop_big->blend(FXApp::instance()->getBaseColor());

    icon_play_big->create();
    icon_pause_big->create();
    icon_next_big->create();
    icon_prev_big->create();
    icon_stop_big->create();
    }

  icon_audio_volume_high->blend(FXApp::instance()->getBaseColor());
  icon_audio_volume_medium->blend(FXApp::instance()->getBaseColor());
  icon_audio_volume_low->blend(FXApp::instance()->getBaseColor());
  icon_audio_volume_muted->blend(FXApp::instance()->getBaseColor());
  icon_audio_volume_high->create();
  icon_audio_volume_medium->create();
  icon_audio_volume_low->create();
  icon_audio_volume_muted->create();
  }

void GMIconTheme::loadAbout() {
  FXIconSource source(FXApp::instance());
  if (icon_about==NULL) {
    icon_about = source.loadIconData(about_png,"png");
    icon_about->blend(FXRGB(255,255,255));
    }
  }


#if FOXVERSION >= FXVERSION(1,7,11)
void GMIconTheme::loadFileAssociations(FXFileDict & filedict) {
  FXFileAssoc * assoc;
  FXIconSource  source(FXApp::instance());

#ifdef HAVE_EXTERNAL_ICONS
  const FXString base    = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY);
  const FXString big     = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-big","22x22");
  const FXString small   = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-small","16x16");

  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-dir",base.text());
  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-size-big",big.text());
  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-size-small",small.text());

  icon_file_small   = loadIcon(source,base,small,"mimetypes-text-x-generic","mimetypes/text-x-generic.png");
  icon_file_big     = loadIcon(source,base,big,"mimetypes-text-x-generic","mimetypes/text-x-generic.png");
  icon_audio_small  = loadIcon(source,base,small,"mimetypes-audio-x-generic","mimetypes/audio-x-generic.png");
  icon_audio_big    = loadIcon(source,base,big,"mimetypes-audio-x-generic","mimetypes/audio-x-generic.png");
  icon_folder_small = loadIcon(source,base,small,"places-folder","places/folder.png");
  icon_folder_big   = loadIcon(source,base,big,"places-folder","places/folder.png");
#else
  icon_file_small   = source.loadIconData(x16_mimetypes_textxgeneric_png,"png");
  icon_file_big     = source.loadIconData(x22_mimetypes_textxgeneric_png,"png");
  icon_audio_small  = source.loadIconData(x16_mimetypes_audioxgeneric_png,"png");
  icon_audio_big    = source.loadIconData(x22_mimetypes_audioxgeneric_png,"png");
  icon_folder_small = source.loadIconData(x16_places_folder_png,"png");
  icon_folder_big   = source.loadIconData(x22_places_folder_png,"png");
#endif
  FXASSERT(icon_file_small);
  FXASSERT(icon_file_big);
  FXASSERT(icon_audio_small);
  FXASSERT(icon_audio_big);
  FXASSERT(icon_folder_small);
  FXASSERT(icon_folder_big);

  icon_file_small->blend(FXApp::instance()->getBackColor());
  icon_file_big->blend(FXApp::instance()->getBackColor());
  icon_audio_small->blend(FXApp::instance()->getBackColor());
  icon_audio_big->blend(FXApp::instance()->getBackColor());
  icon_folder_small->blend(FXApp::instance()->getBackColor());
  icon_folder_big->blend(FXApp::instance()->getBackColor());

  icon_file_small->create();
  icon_file_big->create();
  icon_audio_small->create();
  icon_audio_big->create();
  icon_folder_small->create();
  icon_folder_big->create();

  assoc = filedict.replace("mp3",";MPEG-1 Audio Layer 3");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("MP3",";MPEG-1 Audio Layer 3");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("ogg",";Ogg Vorbis");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("OGG",";Ogg Vorbis");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("flac",";Free Lossless Audio Codec");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("FLAC",";Free Lossless Audio Codec");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("flac",";Free Lossless Audio Codec");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("mpc",";Musepack");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

#ifndef DISABLE_ASF_SUPPORT
  assoc = filedict.replace("wma",";Windows Media");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("asf",";Windows Media");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;
#endif

#ifndef DISABLE_MP4_SUPPORT
  assoc = filedict.replace("m4a",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("mp4",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("m4p",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("aac",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("M4A",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("MP4",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("M4P",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;

  assoc = filedict.replace("AAC",";MPEG-4 Part 14");
  assoc->bigicon  = icon_audio_big;
  assoc->miniicon = icon_audio_small;
#endif

  assoc = filedict.replace(FXFileDict::defaultFileBinding,";");
  assoc->bigicon  = icon_file_big;
  assoc->miniicon = icon_file_small;

  assoc = filedict.replace(FXFileDict::defaultDirBinding,";");
  assoc->bigicon      = icon_folder_big;
  assoc->miniicon     = icon_folder_small;
  }


void GMIconTheme::freeFileAssociations() {
  delete icon_file_small;
  delete icon_file_big;
  delete icon_audio_small;
  delete icon_audio_big;
  delete icon_folder_small;
  delete icon_folder_big;
  }


void GMIconTheme::loadDirAssociations(FXFileDict & filedict) {
  FXFileAssoc * assoc;
  FXIconSource source(FXApp::instance());
#ifdef HAVE_EXTERNAL_ICONS
  const FXString base    = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-dir",DEFAULT_ICON_DIRECTORY);
  const FXString big     = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-big","22x22");
  const FXString small   = FXApp::instance()->reg().readStringEntry("user-icons","icon-base-size-small","16x16");

  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-dir",base.text());
  FXApp::instance()->reg().writeStringEntry("user-icons","icon-base-size-small",small.text());

  icon_folder     = loadIcon(source,base,small,"places-folder","places/folder.png");
  icon_folderopen = loadIcon(source,base,small,"status-folder-open","status/folder-open.png");
#else
  icon_folder     = source.loadIconData(x16_places_folder_png,"png");
  icon_folderopen = source.loadIconData(x16_status_folderopen_png,"png");
#endif
  FXASSERT(icon_folder);
  FXASSERT(icon_folderopen);

  icon_folder->blend(FXApp::instance()->getBackColor());
  icon_folderopen->blend(FXApp::instance()->getBackColor());

  assoc = filedict.replace(FXFileDict::defaultDirBinding,";");
  assoc->miniiconopen = icon_folderopen;
  assoc->miniicon     = icon_folder;
  }

void GMIconTheme::freeDirAssociations() {
  delete icon_folder;
  delete icon_folderopen;
  }


#endif
