/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <unistd.h>
#include <signal.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>

#include <xincs.h>


#include <xine.h>
#include "icons.h"
#include "common.h"
#include "version.h"

#ifdef HAVE_DBUS
#include "fxdbus.h"
#endif

#include <FXPNGIcon.h>
#include "GMApp.h"
#include "GMPlayer.h"
#include "GMAbout.h"
#include "GMWizard.h"
#include "GMWindow.h"
#include "GMTrackList.h"
#include "GMRemote.h"
#include "GMTag.h"
#include "GMThread.h"
#include "GMSearch.h"
#include "GMList.h"
#include "GMTrackView.h"
#include "GMSource.h"
#include "GMDatabaseSource.h"
#include "GMStreamSource.h"
#include "GMIconTheme.h"
#include "GMPlayerManager.h"
#include "GMFetch.h"
#ifdef HAVE_DBUS
#include "GMNotifyDaemon.h"
#endif

#include "GMAudioScrobbler.h"

enum {
  FIFO_STATUS_ERROR  = 0,
  FIFO_STATUS_OWNER  = 1,
  FIFO_STATUS_EXISTS = 2
  };

#if APPLICATION_BETA > 0
#define DATABASE_FILENAME "goggles_beta.db"
#else
#define DATABASE_FILENAME "goggles.db"
#endif


FXbool isLocalFile(const FXString & filename) {
  if (filename[0]=='/' || (comparecase("file:",filename,5)==0) ) 
    return true;
  return false;
  }	


FXDEFMAP(GMPlayerManager) GMPlayerManagerMap[]={
  FXMAPFUNC(SEL_TIMEOUT,GMPlayerManager::ID_UPDATE_TRACK_DISPLAY,GMPlayerManager::onUpdTrackDisplay),
  FXMAPFUNC(SEL_TIMEOUT,GMPlayerManager::ID_HANDLE_EVENTS,GMPlayerManager::onUpdEvents),
  FXMAPFUNC(SEL_TIMEOUT,GMPlayerManager::ID_SLEEP_TIMER,GMPlayerManager::onCmdSleepTimer),
  FXMAPFUNC(SEL_TIMEOUT,GMPlayerManager::ID_PLAY_NOTIFY,GMPlayerManager::onPlayNotify),

  FXMAPFUNC(SEL_IO_READ,GMPlayerManager::ID_DDE_MESSAGE,GMPlayerManager::onDDEMessage),
  FXMAPFUNC(SEL_CHORE,GMPlayerManager::ID_PLAY_NEXT,GMPlayerManager::onCmdPlayNext),
  FXMAPFUNC(SEL_CLOSE,GMPlayerManager::ID_REMOTE,GMPlayerManager::onCmdCloseRemote),
  FXMAPFUNC(SEL_CLOSE,GMPlayerManager::ID_WINDOW,GMPlayerManager::onCmdCloseWindow),

  FXMAPFUNC(SEL_COMMAND,GMPlayerManager::ID_DOWNLOAD_COMPLETE,GMPlayerManager::onCmdDownloadComplete),
  };

FXIMPLEMENT(GMPlayerManager,FXObject,GMPlayerManagerMap,ARRAYNUMBER(GMPlayerManagerMap))

#ifdef HAVE_DBUS

#define DBUS_SIGNAL_PREVIOUS "previous"
#define DBUS_SIGNAL_PLAY "play"
#define DBUS_SIGNAL_PLAYPAUSE "play_pause"
#define DBUS_SIGNAL_PAUSE "pause"
#define DBUS_SIGNAL_NEXT "next"
#define DBUS_SIGNAL_STOP "stop"
#define DBUS_SIGNAL_RAISE "raise"

DBusHandlerResult dbus_playermanager_filter(DBusConnection *,DBusMessage * msg,void * data){
  GMPlayerManager * p = (GMPlayerManager*)data;
  if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_PREVIOUS)){
    if (p->can_prev()) p->prev();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_PLAY)){
    if (p->can_play()) p->play();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_PLAYPAUSE)){
    if (p->can_pause())
      p->pause();
    else if (p->can_unpause())
      p->unpause();
    else if (p->can_play())
      p->play();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_PAUSE)){
    if (p->can_pause()) p->pause();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_NEXT)){
    if (p->can_next()) p->next();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_STOP)){
    if (p->can_stop()) p->stop();
    }
  else if (dbus_message_is_signal(msg,GOGGLESMM_DBUS_INTERFACE,DBUS_SIGNAL_RAISE)){
    if (p->getMainWindow()->shown())
      p->getMainWindow()->raise();
    }
  else {
    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
    }
  return DBUS_HANDLER_RESULT_HANDLED;
  }

void dbus_send_signal_to_self(DBusConnection * connection,const FXchar * signal) {
  DBusMessage * msg;
  msg = dbus_message_new_signal(GOGGLESMM_DBUS_PATH,GOGGLESMM_DBUS_INTERFACE,signal);
  if (msg){
    if (dbus_connection_send(connection,msg,NULL))
       dbus_connection_flush(connection);
  
    dbus_message_unref(msg);
    }
  }


FXint dbus_send_commands(DBusConnection * connection,int& argc,char** argv){
  FXString cmd=DBUS_SIGNAL_RAISE;
  if (argc>1) {
    if (compare(argv[1],"--previous")==0)
      cmd=DBUS_SIGNAL_PREVIOUS;
    else if (compare(argv[1],"--play")==0)
      cmd=DBUS_SIGNAL_PLAY;
    else if (compare(argv[1],"--play-pause")==0)
      cmd=DBUS_SIGNAL_PLAYPAUSE;
    else if (compare(argv[1],"--pause")==0)
      cmd=DBUS_SIGNAL_PAUSE;
    else if (compare(argv[1],"--next")==0)
      cmd=DBUS_SIGNAL_NEXT;
    else if (compare(argv[1],"--stop")==0)
      cmd=DBUS_SIGNAL_STOP;
    }
  dbus_send_signal_to_self(connection,cmd.text());
  return 1; 
  }

#endif

long GMPlayerManager::onPlayNotify(FXObject*,FXSelector,void*){

  if (player->playing() ) {

    GMTrack information;

    if (source) {
      if (!source->getTrack(information)) {
        player->getTrackInformation(information);
        }
      }
    else {
      player->getTrackInformation(information);
      }

    if (preferences.gui_show_playing_albumcover)
      mainwindow->loadCover(information.mrl);



#ifdef HAVE_DBUS
  if (dbus) {
    FXString body = "<b>"+information.title+"</b>\n<i>by:</i> " + information.artist + "\n<i>from:</i> " + information.album;
{	

		FXString cover = "gmm";
		if (mainwindow->getSmallCover())	
			cover = FXSystem::getHomeDirectory() + PATHSEPSTRING + ".goggles" + PATHSEPSTRING + "cover.jpg";

//    GMNotifyDaemon daemon(dbus);
    if (daemon && preferences.dbus_notify_daemon) daemon->notify("gogglesmm",cover.text(),"Now Playing",body.text(),-1);
}

  if (getPreferences().mugshot_notification){

    DBusMessage * msg = dbus_message_new_signal("/org/fiftplanet/gogglesmm","org.gnome.Banshee.Mugshot","StateChangedEvent");
    if (msg) {
      dbus_uint32_t serial;
      dbus_int32_t state=1;
      const FXchar * c_artist=information.artist.text();
      const FXchar * c_album=information.album.text();
      const FXchar * c_title=information.title.text();

      dbus_message_append_args(msg,DBUS_TYPE_INT32,&state,
                                   DBUS_TYPE_STRING,&c_artist,
                                   DBUS_TYPE_STRING,&c_title,
                                   DBUS_TYPE_STRING,&c_album,
                                   DBUS_TYPE_INT32,&information.time,
                                   DBUS_TYPE_INVALID);

      dbus_connection_send(dbus->connection(),msg,&serial);
      dbus_message_unref(msg);
      dbus_connection_flush(dbus->connection());
      }
    }

    }
#endif
    if (lastfm) lastfm->nowplaying(information);
    }
  return 0;
  }

long GMPlayerManager::onUpdTrackDisplay(FXObject*,FXSelector,void*){
  update_track_display();
  //update_track_display_nodb();
  return 0;
  }

long GMPlayerManager::onUpdEvents(FXObject*,FXSelector,void*){
  handle_async_events();
  application->addTimeout(this,GMPlayerManager::ID_HANDLE_EVENTS,TIME_MSEC(500));
  return 0;
  }

/* Stop Playback! */
long GMPlayerManager::onCmdSleepTimer(FXObject*,FXSelector,void*){
  if (playing()) {
    stop();
    }
  return 1;
  }


long GMPlayerManager::onDDEMessage(FXObject*,FXSelector,void*){
  FXString cmd;
  FXchar buffer[1024];
  int nread=fifo.readBlock(buffer,1024);
  if (nread>0) {
    cmd=FXString(buffer,nread);
    cmd.trim();
    if (cmd=="--previous") { // Skip backwards in playlist
      if (can_prev()) prev();
      }
    else if (cmd=="--play") { // Start playing current playlist
      if (can_play()) play();
      }
    else if (cmd=="--play-pause"){ // Play if stopped, pause if playing
      if (can_pause())
        pause();
      else if (can_unpause())
        unpause();
      else if (can_play())
        play();
      }
    else if (cmd=="--pause") { // Pause playback
      if (can_pause()) pause();
      }
    else if (cmd=="--next"){ //Skip forwards in playlist
      if (can_next()) next();
      }
    else if (cmd=="--stop"){ //Stop Playback
      if (can_stop()) stop();
      }
    else if (cmd=="--raise"){
      if (mainwindow->shown())
        mainwindow->raise();
      }
    }


#if 0
  FXString mrl;
  FXchar buffer[255];
  int nread=dde->readBlock(buffer,255);
  if (nread>0) {
    mrl=FXString(buffer,nread);
    fxmessage("gmm.dde: %s\n",mrl.text());

    if (player.open(mrl)) {
      player.play();
      GMTrack t;
      player.getTrackInformation(t);
      remotecontrol->setToolTip(FXStringVal(t.no)+". "+t.title+"\n"+t.artist+"\n"+t.album);
      tickertext = FXString(" *** " + t.artist + " - " + t.album + " - " + FXStringVal(t.no) + " " + t.title);
      ticker->setText(tickertext);
      ticker->setSpeed(50);
      }
#endif



/*
    if (FXStat::exists(mrl) && FXStat::isFile(mrl)){
      FXString ext = FXPath::extension(mrl);
      if (comparecase(ext,"mp3")==0 || comparecase(ext,"ogg")==0 || comparecase(ext,"flac")==0 || comparecase(ext,"mpc")==0) {
        int id = insertTempTrack(mrl);
        if (id>0) {
          track_id=id;
          if (updateTrackId(track_id)) {
            fxmessage(" - Playing Song %d: %s\n",id,track.mrl.text());
            playSong();
            updateView(FALSE);
            }
          }
        else {
          fxmessage(" - File not in Library and unable to insert.\n");
          }
        }
      else {
        fxmessage(" - Unsupported file.\n");
        }
      }

    else {
      fxmessage(" - File does not exists or not a file.\n");
      }
    }
*/
  return 1;
  }



GMPlayerManager * GMPlayerManager::myself = NULL;

GMPlayerManager* GMPlayerManager::instance() {
  return myself;
  }



/// Constructor
GMPlayerManager::GMPlayerManager() :
  count_track_remaining(0),
#ifdef HAVE_DBUS
  dbus(NULL),
#endif
  application(NULL),
  mainwindow(NULL),
  remote(NULL),
  player(NULL),
  fetch(NULL),
  lastfm(NULL),
  source(NULL) {
  FXASSERT(myself==NULL);
  myself=this;
  }




/// Destructor
GMPlayerManager::~GMPlayerManager() {

  /// Cleanup fifo crap
  if (fifo.isOpen())
    fifo.close();
  if (!fifofilename.empty())
    FXFile::remove(fifofilename);

  delete lastfm;

  /// Delete Sources
  for (FXint i=0;i<sources.no();i++)
    delete sources[i];

#ifdef HAVE_DBUS
  delete dbus;
#endif
  delete application;
  delete player;
  delete fetch;
  }




FXint GMPlayerManager::init_fifo(int& argc,char** argv){
  FXString fifodir = FXSystem::getHomeDirectory() + PATHSEPSTRING + ".goggles";
  FXStat info;

  if ( (!FXStat::exists(fifodir) && !FXDir::create(fifodir) ) || !FXStat::isDirectory(fifodir) ) {
    FXMessageBox::error(application,MBOX_OK,"Goggles Music Manager","Unable to create directory %s\n",fifodir.text());
    return FIFO_STATUS_ERROR;
    }

  fifofilename = fifodir + PATHSEPSTRING + "gmm.dde";

  /// Find existing fifo
  if (FXStat::statFile(fifofilename,info)) {

    /// File exists, but it's not a fifo... try removing it
    if (!info.isFifo() && !FXFile::remove(fifofilename)) {
      fifofilename=FXString::null;
      return FIFO_STATUS_ERROR;
      }

    if (fifo.open(fifofilename,FXIO::WriteOnly|FXIO::NonBlocking)){
      FXString commandline;

      for (FXint i=1;i<argc;i++){
        commandline+=argv[i];
        }

      /// Try raising the window
      if (commandline.empty())
        commandline="--raise";

      /// if write failed or command line was empty, user probably tries to start up normally
      /// and fifo may be stale
      if (fifo.writeBlock(commandline.text(),commandline.length())) {
        fifofilename=FXString::null;
        return FIFO_STATUS_EXISTS;
        }

      /// Most likely left behind by crashed music manager
      fifo.close();
      }

    if (!FXFile::remove(fifofilename))
      return FIFO_STATUS_OWNER;
    }

  /// Create the fifo
  if (mkfifo(fifofilename.text(),S_IWUSR|S_IRUSR)!=0){
      fifofilename=FXString::null;
      return FIFO_STATUS_OWNER;
      }

  /// Try open the fifo
  fifo.open(fifofilename,FXIO::Reading|FXIO::WriteOnly,FXIO::OwnerWrite);

  /// Close fifo on execute or remove fifo if we couldn't open it...
  if (fifo.isOpen()) {
    fcntl(fifo.handle(),F_SETFD,FD_CLOEXEC);
    }
  else {
    FXFile::remove(fifofilename);
    fifofilename=FXString::null;
    }

  return FIFO_STATUS_OWNER;
  }




FXbool GMPlayerManager::init_sources() {
  GMDatabaseSource * src;

  // Main Database
  GMTrackDatabase * database = new GMTrackDatabase;

  // Make sure we can open it.
  if (!init_database(database,DATABASE_FILENAME)) {
    delete database;
    return false;
    }

  // Create the main database source
  src = new GMDatabaseSource(database);
  sources.append(src);

  /// Add the playlists
  src->initPlaylists(sources);

  sources.append(new GMStreamSource(database));

  return true;
  }




void GMPlayerManager::init_window() {

  /// Create Main Window
  mainwindow = new GMWindow(application,this,ID_WINDOW);
  mainwindow->create();

  /// Handle interrupt to save stuff nicely
  application->addSignal(SIGINT,mainwindow,GMWindow::ID_QUIT);
  application->addSignal(SIGCHLD,mainwindow,GMWindow::ID_CHILD);

  /// Create Tooltip Window
  FXToolTip * tooltip = new FXToolTip(application);
  tooltip->create();


/*
  FXString mrl;

  if (!initSources())
    return false;


  /// Any Filename on the command line ?
  for (FXint i=1;i<application->getArgc();i++){
    if (FXStat::exists(application->getArgv()[i])) {
      mrl = application->getArgv()[i];
      break;
      }
    }

  /// Play File and such from command line
  if (!mrl.empty()) {
    play(mrl);
    mainwindow->init();
    mainwindow->show();
    }
  else {
*/
    if (sources[0]->getNumTracks()==0) {

      cleanSourceSettings();

      GMIconTheme::instance()->loadAbout();

      FXWizardDialog dialog(mainwindow,GMIconTheme::instance()->icon_about);
      if (dialog.execute(PLACEMENT_SCREEN)) {
        FXString searchdir = dialog.getDirectory();
        FXStringList list;
        list.append(searchdir);
        GMSearchDialog dialog(mainwindow,list,-1,getPreferences(),mainwindow->font_thick);
        dialog.execute();
        application->reg().writeStringEntry("SETTINGS","LastScanDir",searchdir.text());
        }

      delete GMIconTheme::instance()->icon_about;
      GMIconTheme::instance()->icon_about=NULL;

      mainwindow->init();
      mainwindow->show();
      }
    else {
      if (preferences.gui_startup==STARTUP_MAINWINDOW || (preferences.gui_startup==STARTUP_LAST && preferences.gui_last_startup==STARTUP_MAINWINDOW) ) {
        mainwindow->show();
        if (preferences.gui_always_show_remote) {
          remote = new GMRemote(application,this,ID_REMOTE);
          remote->create();
          }
        }
      else {
        remote = new GMRemote(application,this,ID_REMOTE);
        remote->create();
        }
      mainwindow->init();
      }
//    }
  application->addTimeout(this,GMPlayerManager::ID_HANDLE_EVENTS,TIME_MSEC(500));
//  return true;
  }




FXint GMPlayerManager::run(int& argc,char** argv) {
  FXint result;


  /// Create Application so we can do things like popup dialogs and such
  application = new GMApp("musicmanager","");
  application->init(argc,argv);
  application->create();

  /// Set Locale
  setlocale(LC_ALL,"");

  /// Make sure we use SQLITE 3.3.8 (DROP VIEW IF EXISTS support).
  if (sqlite3_libversion_number()<SQVERSION(3,3,8)) {
    FXMessageBox::error(application,MBOX_OK,"Goggles Music Manager",
      "An incompatible version of SQLite (%s) is being used.\n"
      "Goggles Music Manager requires at least SQLite 3.3.8.\n"
      "Please upgrade your SQLite installation.",sqlite3_libversion());
    return 1;
    }


  /// Give warning when PNG is not compiled in...
  if (FXPNGIcon::supported==false) {
    FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager",
      "For some reason the FOX library was compiled without PNG support.\n"
      "In order to show all icons, Goggles Music Manager requires PNG\n"
      "support in the FOX library. If you've compiled FOX yourself, most\n"
      "likely the libpng header files were not installed on your system.");
    }

#ifdef HAVE_DBUS

  dbus = new FXDBusConnection();
  if (!dbus->open(DBUS_BUS_SESSION) || !dbus->connected()) {
    FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager","Session bus not available. All features requiring dbus are disabled.");
    delete dbus;
    dbus=NULL;
    }
  else {
    FXint result = dbus_bus_request_name(dbus->connection(),GOGGLESMM_DBUS_NAME,DBUS_NAME_FLAG_DO_NOT_QUEUE,NULL);
    switch(result) {
      case DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER:
        {
          DBusError error;
          dbus_error_init(&error);
          dbus_bus_add_match(dbus->connection(),"type='signal',path='"GOGGLESMM_DBUS_PATH"'",&error);
          if (dbus_error_is_set(&error)) {
            dbus_error_free(&error);
            FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager","A DBus error occurred. All features requiring dbus are disabled.");
            delete dbus;
            dbus=NULL;
            }
          else {
            if (!dbus_connection_add_filter(dbus->connection(),dbus_playermanager_filter,this,NULL)){
              FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager","A DBus error occurred. All features requiring dbus are disabled.");
              delete dbus;
              dbus=NULL;
              }
           }
        }
        break;
      case DBUS_REQUEST_NAME_REPLY_EXISTS:
        dbus_send_commands(dbus->connection(),argc,argv);
        return 0;
        //FXMessageBox::error(application,MBOX_OK,"Fatal Error","Another Goggles Music Manager is already running.");
        //return 0;
        break;
      default:
        FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager","Session Bus not available. All features requiring dbus are disabled.");
        delete dbus;
        dbus=NULL;
        break;
      }
    }

  /// Fallback to fifo method to check for existing instance
  if (dbus==NULL) {
    result = init_fifo(argc,argv);
    if (result==FIFO_STATUS_ERROR)
      return 1;
    else if (result==FIFO_STATUS_EXISTS)
      return 0;
    }
#else
  result = init_fifo(argc,argv);
  if (result==FIFO_STATUS_ERROR)
    return 1;
  else if (result==FIFO_STATUS_EXISTS)
    return 0;
#endif

  /// Open Database and initialize all sources.
  if (!init_sources())
    return false;

  /// Everything opened succesfully... now create the GUI
  player = new GMPlayer;

  /// Load Application Preferences
  preferences.load(application->reg());

  /// Open Audio Device if needed.
  if (preferences.play_open_device_on_startup) {
    if (!player->initialize()) {
      FXString errormsg;
      player->getErrorMessage(errormsg);
      FXMessageBox::error(mainwindow,MBOX_OK,"Audio Device Error",errormsg.text());
      }
    }

  /// Receive events from fifo
  if (fifo.isOpen()) {
#if FOXVERSION < FXVERSION(1,7,0)
    application->addInput(fifo.handle(),INPUT_READ,this,GMPlayerManager::ID_DDE_MESSAGE);
#else
    application->addInput(this,GMPlayerManager::ID_DDE_MESSAGE,fifo.handle(),INPUT_READ);
#endif
    }

  GMTag::init();

  /// Show user interface
  init_window();

#ifdef HAVE_DBUS

  if (dbus) {

    daemon = new GMNotifyDaemon(dbus);

    /// Integrate Dbus into FOX Event Loop
    FXDBusConnection::initEventLoop();
    }
#endif
  lastfm = new GMAudioScrobbler(application);

  /// Run the application
  return application->run();
  }





void GMPlayerManager::removeSource(GMSource *src) {
  sources.remove(src);
  }

void GMPlayerManager::exit() {

  application->removeTimeout(this,GMPlayerManager::ID_HANDLE_EVENTS);

  application->removeTimeout(this,GMPlayerManager::ID_UPDATE_TRACK_DISPLAY);

  application->removeTimeout(this,GMPlayerManager::ID_SLEEP_TIMER);

  application->removeTimeout(this,GMPlayerManager::ID_PLAY_NOTIFY);

  application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);

  preferences.save(application->reg());

  if (lastfm) lastfm->shutdown();

#ifdef HAVE_DBUS
  if (dbus) {
    DBusError error;
    dbus_error_init(&error);

    dbus_connection_remove_filter(dbus->connection(),dbus_playermanager_filter,this);

    dbus_bus_remove_match(dbus->connection(),"type='signal',path='"GOGGLESMM_DBUS_PATH"'",&error);
    if (dbus_error_is_set(&error)) {
      dbus_error_free(&error);
      }

    }
#endif


  application->exit(0);
  }


GMTrackView * GMPlayerManager::getTrackView() const {
  return mainwindow->trackview;
  }

GMSourceView * GMPlayerManager::getSourceView() const {
  return mainwindow->sourceview;
  }

FXbool GMPlayerManager::init_database(GMTrackDatabase *& db, const FXString & filename){
  FXString dir = FXSystem::getHomeDirectory() + PATHSEPSTRING + ".goggles";
  FXString databasefilename = dir + PATHSEPSTRING + filename;

  if (FXStat::exists(dir)) {

    if (!FXStat::isDirectory(dir))
      return FALSE;

    }
  else {

    if (!FXDir::create(dir))
      return FALSE;
    }

  if (!FXStat::isWritable(dir) || !FXStat::isReadable(dir))
    return FALSE;

  if (FXStat::exists(databasefilename) && (!FXStat::isWritable(databasefilename) || !FXStat::isReadable(databasefilename)))
    return FALSE;

  if (!db) {
    db = new GMTrackDatabase();
    }

  /// Init Database
  if (!db->init(databasefilename)) {
    FXMessageBox::error(application,MBOX_OK,"Fatal Error","Goggles Music Manager was unable to open the database.\nThe database may have been corrupted. Please remove ~/.goggles/goggles.db to try again.\nif the error keeps occuring, please send a bug report to <sander@knology.net> ");
    return false;
    }

  return TRUE;
  }

FXbool GMPlayerManager::hasSourceWithKey(const char * key) const{
  for (FXint i=0;i<sources.no();i++){
    if (sources[i]->settingKey()==key)
      return true;
    }
  return false;
  }


void GMPlayerManager::cleanSourceSettings() {
  FXint s;
  FXStringList keys;

  for (s=application->reg().first();s<application->reg().size();s=application->reg().next(s)){
    if (comparecase(application->reg().key(s),"database",8)==0){
      if (!hasSourceWithKey(application->reg().key(s))) {
        keys.append(application->reg().key(s));
        }
      }
    }

  for (s=0;s<keys.no();s++){
    application->reg().deleteSection(keys[s].text());
    }
  }




void GMPlayerManager::removePlayListSources(){
  GMSource * src;
  for (FXint i=sources.no()-1;i>=0;i--){
    src=sources[i];
    if (src->getType()==SOURCE_DATABASE_PLAYLIST) {
      FXApp::instance()->reg().deleteSection(sources[i]->settingKey().text());
      sources.erase(i);
      delete src;
      }
    }
  }

/*
FXbool GMPlayerManager::open() {
  FXString mrl;

  if (!initSources())
    return false;

  if (FXPNGIcon::supported==false) {
    FXMessageBox::warning(application,MBOX_OK,"Goggles Music Manager",
      "For some reason the FOX library was compiled without PNG support.\n"
      "In order to show all icons, Goggles Music Manager requires PNG\n"
      "support in the FOX library. If you've compiled FOX yourself, most\n"
      "likely the libpng header files were not installed on your system.");
    }

  /// Any Filename on the command line ?
  for (FXint i=1;i<application->getArgc();i++){
    if (FXStat::exists(application->getArgv()[i])) {
      mrl = application->getArgv()[i];
      break;
      }
    }

  /// Play File and such from command line
  if (!mrl.empty()) {
    play(mrl);
    mainwindow->init();
    mainwindow->show();
    }
  else {
    if (sources[0]->getNumTracks()==0) {

      cleanSourceSettings();

      FXWizardDialog dialog(mainwindow,GMIconTheme::instance()->icon_about);
      if (dialog.execute(PLACEMENT_SCREEN)) {
        FXString searchdir = dialog.getDirectory();
        FXStringList list;
        list.append(searchdir);
        GMSearchDialog dialog(mainwindow,list,-1,getPreferences(),mainwindow->font_thick);
        dialog.execute();
        application->reg().writeStringEntry("SETTINGS","LastScanDir",searchdir.text());
        }
      mainwindow->init();
      mainwindow->show();
      }
    else {
      if (preferences.gui_startup==STARTUP_MAINWINDOW || (preferences.gui_startup==STARTUP_LAST && preferences.gui_last_startup==STARTUP_MAINWINDOW) ) {
        mainwindow->show();
        if (preferences.gui_always_show_remote) {
          remote->show();
          }
        }
      else {
        remote->show();
        }
      mainwindow->init();
      }
    }
  application->addTimeout(this,GMPlayerManager::ID_HANDLE_EVENTS,TIME_MSEC(500));
  return true;
  }

*/

void GMPlayerManager::download(const FXString & filename){
  if (fetch==NULL) {
    fetch = new GMFetch(this);
    }
  else {
    fetch->dispose_and_join();
    delete fetch;
    fetch = new GMFetch(this);
    }

  /// Get the filename
  FXString status = "Downloading " + filename + " ...";

  //fxmessage("%s\n",status.text());

  GMPlayerManager::instance()->setStatus(status);
  fetch->get(filename);
 // sleep(1);
  }

FXbool GMPlayerManager::play(const FXString & filename) {
  //fxmessage("play(const FXString & filename)\n");
  FXString errormsg;

  /// Open Filename
  if (!player->open(filename)){

    /// Reset Source
    if (source) {
      application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
      source->resetCurrent();
      source=NULL;
      }

    /// Reset Track Display
    reset_track_display();

    /// Get Error Message
    player->getErrorMessage(errormsg);

    FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

    /// Close
    if (preferences.play_close_stream)
      player->close();

    return false;
    }

  /// Start Playback
  if (!player->play()) {

    /// Reset Source
    if (source) {
      application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
      source->resetCurrent();
      source=NULL;
      }

    /// Reset Track Display
    reset_track_display();

    /// Get Error Message
    player->getErrorMessage(errormsg);

    FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

    /// Close
    if (preferences.play_close_stream)
      player->close();

    return false;
    }
  return true;
  }




void GMPlayerManager::play() {
  FXString filename;
  FXint track;

  /// Stop Current Playback
  player->stop();

  /// Remove Current Timeout
  if (source) {
    application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
    source->resetCurrent();
    source=NULL;
    }

  track = getTrackView()->getCurrent();
  if (track==-1) return;

  /// Get the filename
  filename = getTrackView()->getTrackFilename(track);

  /// Mark Track
  source = getTrackView()->getSource();
  getTrackView()->mark(track);

  /// Check Extension if we need to fetch a file ourselves...
  FXString ext = FXPath::extension(filename);
  if ( (comparecase(ext,"m3u")==0) || (comparecase(ext,"pls")==0) ) {
    download(filename);
    return;
    }

  if (play(filename) && isLocalFile(filename))
    update_track_display();
  }


void GMPlayerManager::stop(FXbool force_close) {

  if (fetch) {
    fetch->dispose_and_join();
    delete fetch;
    fetch=NULL;
    }

  /// Reset Source
  if (source) {
    application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
    source->resetCurrent();
    source=NULL;
    }

  if (preferences.play_close_stream || force_close){
    player->stop();
    player->close();
    }
  else {
    player->stop();
    }

  /// Reset Track Display
  reset_track_display();
  }

namespace FX {
extern FXlong fxgetticks();
}

static FXint next_track;

void GMPlayerManager::next() {

  FXString filename;
  FXint track;

  player->stop();

    /// Remove Current Timeout
  if (source) {
    application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
    source->resetCurrent();
    source=NULL;
    }

  track = getTrackView()->getNext(true);
  if (track==-1) return;

  /// Get the filename
  filename = getTrackView()->getTrackFilename(track);

  /// Mark Track
  source = getTrackView()->getSource();
  getTrackView()->mark(track);

  /// Check Extension if we need to fetch a file ourselves...
  FXString ext = FXPath::extension(filename);
  if ( (comparecase(ext,"m3u")==0) || (comparecase(ext,"pls")==0) ) {
    download(filename);
    return;
    }

  if (play(filename) && isLocalFile(filename))
    update_track_display();
  }


long GMPlayerManager::onCmdPlayNext(FXObject*,FXSelector,void*){
  //fxmessage("timeout fired\n");

  FXString filename;

  /// Stop Current Playback
  player->stop();

  /// Get the filename
  filename = getTrackView()->getTrackFilename(next_track);




  /// Check Extension if we need to fetch a file ourselves...
  FXString ext = FXPath::extension(filename);
  if ( (comparecase(ext,"m3u")==0) || (comparecase(ext,"pls")==0) ) {
    download(filename);
    return 1;
    }

  if (play(filename) && isLocalFile(filename))
    update_track_display();

  return 1;
  }

void GMPlayerManager::prev() {
  FXString filename;
  FXint track;

  /// Stop Current Playback
  player->stop();

  /// Remove Current Timeout
  if (source) {
    application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
    source->resetCurrent();
    source=NULL;
    }

  track = getTrackView()->getPrevious();
  if (track==-1) return;

  /// Get the filename
  filename = getTrackView()->getTrackFilename(track);

  /// Mark Track
  source = getTrackView()->getSource();
  getTrackView()->mark(track);

  /// Check Extension if we need to fetch a file ourselves...
  FXString ext = FXPath::extension(filename);
  if ( (comparecase(ext,"m3u")==0) || (comparecase(ext,"pls")==0) ) {
    download(filename);
    return;
    }

  if (play(filename) && isLocalFile(filename))
    update_track_display();
  }

void GMPlayerManager::seek(FXint pos) {

  /// Remove Current Timeout
  application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);

  player->play(pos);
  /// Prevent jumping of time slider
  application->addTimeout(this,GMPlayerManager::ID_HANDLE_EVENTS,TIME_MSEC(500));
  }

void GMPlayerManager::pause() {
  if (preferences.play_pause_close_device){
    player->pause();
    player->close_device();
    }
  else {
    player->pause();
    }
  count_track_remaining = application->remainingTimeout(source,GMSource::ID_TRACK_PLAYED);
  application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
  }

void GMPlayerManager::unpause() {
  player->unpause();

  if (player->getVolume()==0)
    player->setVolume(50);

  if (count_track_remaining>0 && source){
    application->addTimeout(source,GMSource::ID_TRACK_PLAYED,count_track_remaining);
    }
  count_track_remaining=0;

  mainwindow->update_volume_display(player->getVolume());

  if (remote) remote->update_volume_display(player->getVolume());
  }

FXbool GMPlayerManager::playlist_empty() {
  if (preferences.play_repeat) return false;
  if (getTrackView()->hasTracks()) return false;
  if (getTrackView()->getNext()!=-1) return false;
  return true;
  }

void GMPlayerManager::notify_playback_finished() {
  FXString errormsg;
  FXString filename;
  FXint track;
  FXint remaining = player->remaining();


  /// Can we just start playback without user interaction
  if (!getTrackView()->getSource()->autoPlay()) {

    /// Reset Source
    if (source) {
      source->resetCurrent();
      source=NULL;
      }

    reset_track_display();
    return;
    }


  if (preferences.play_repeat){
    track = getTrackView()->getCurrent();
    if (track==-1) return;
    filename = getTrackView()->getTrackFilename(track);
    }
  else {

    if (source) {
      source->resetCurrent();
      source=NULL;
      }

    track = getTrackView()->getNext();
    if (track==-1) {
       reset_track_display();
       return;
       }

    getTrackView()->mark(track);
    source = getTrackView()->getSource();

    filename=getTrackView()->getTrackFilename(track);
    }

  if (play(filename)) {
    if (remaining>0)
      application->addTimeout(this,GMPlayerManager::ID_UPDATE_TRACK_DISPLAY,TIME_SEC(remaining));
    else
      update_track_display();
    }

  }






#if 0






  FXString filename;
  FXint track;

  /// Stop Current Playback
  player->stop();

  /// Remove Current Timeout
  if (source) {
    application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
    source->resetCurrent();
    source=NULL;
    }

  track = getTrackView()->getCurrent();
  if (track==-1) return;

  /// Get the filename
  filename = getTrackView()->getTrackFilename(track);

  /// Mark Track
  source = getTrackView()->getSource();
  getTrackView()->mark(track);

  /// Check Extension if we need to fetch a file ourselves...
  FXString ext = FXPath::extension(filename);
  if ( (comparecase(ext,"m3u")==0) || (comparecase(ext,"pls")==0) ) {
    download(filename);
    return;
    }

  play(filename);


















    filename = source->getTrackFilename(source->getCurrentTrack());
    if (!player->open(filename)){

      /// Reset Source
      if (source) {
        source->resetCurrent();
        source=NULL;
        }

      /// Reset Track Display
      reset_track_display();

      /// Get Error Message
      player->getErrorMessage(errormsg);

      FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

      /// Close
      if (preferences.play_close_stream)
        player->close();

      return;
      }

    if (!player->play()) {

      /// Reset Source
      if (source) {
        source->resetCurrent();
        source=NULL;
        }

      /// Reset Track Display
      reset_track_display();

      /// Get Error Message
      player->getErrorMessage(errormsg);

      FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

      /// Close
      if (preferences.play_close_stream)
        player->close();

      return;
      }

    }
  else {


    /// Reset Source
    if (source) {
      source->resetCurrent();
      source=NULL;
      }

    /// Can we just start playback without user interaction
    if (!getTrackView()->getSource()->autoPlay()) {
      reset_track_display();
      return;
      }

    /// Next Track
    track = getTrackView()->getNext();
    if (track==-1) {
       reset_track_display();
       return;
       }

    filename=getTrackView()->getTrackFilename(track);
    if (!player->open(filename)){

      /// Reset Track Display
      reset_track_display();

      /// Get Error Message
      player->getErrorMessage(errormsg);

      FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

      /// Close
      if (preferences.play_close_stream)
        player->close();

      return;
      }

    if (!player->play()) {

      /// Reset Track Display
      reset_track_display();

      /// Get Error Message
      player->getErrorMessage(errormsg);

      FXMessageBox::error(mainwindow,MBOX_OK,"Playback Error",errormsg.text());

      /// Close
      if (preferences.play_close_stream)
        player->close();

      return;
      }

    getTrackView()->mark(track);
    source = getTrackView()->getSource();
    }

  if (remaining>0)
    application->addTimeout(this,GMPlayerManager::ID_UPDATE_TRACK_DISPLAY,TIME_SEC(remaining));
//  else
//    update_track_display_nodb();
  }

#endif
void GMPlayerManager::fastforward(){
  player->incSpeed();
  }

void GMPlayerManager::fastbackward(){
  player->decSpeed();
  }

FXbool GMPlayerManager::playing() const {
  return player->playing() || fetch;
  }

FXbool GMPlayerManager::audio_device_opened() const{
  return player->opened();
  }

FXint GMPlayerManager::current_position() const {
  return player->getPosition();
  }

void GMPlayerManager::reset_track_display() {
  FXTRACE((51,"GMPlayerManager::reset_track_display()\n"));

  /// Reset Status Text
  mainwindow->statusbar->getStatusLine()->setNormalText("Ready.");

  /// Clear Cover
  mainwindow->loadCover(FXString::null);

  /// Reset Remote Control
  //remote->setToolTip(FXString::null);

  if (remote) remote->reset();

  /// Reset Active Track
  getTrackView()->mark(-1);

  /// Remove Notify 
  application->removeTimeout(this,ID_PLAY_NOTIFY);
  }


void GMPlayerManager::setStatus(const FXString & text){
  mainwindow->statusbar->getStatusLine()->setNormalText(text);
  }




void GMPlayerManager::update_track_display(FXbool notify) {
  FXTRACE((51,"GMPlayerManager::update_track_display()\n"));

  GMTrack information;

  if (source) {
    if (!source->getTrack(information)) {
      player->getTrackInformation(information);
      source->setTrack(information);
      }

    FXint time = (FXint) (((double)information.time) * 0.80);
    if (time <= 5) {
      application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
      source->handle(this,FXSEL(SEL_TIMEOUT,GMSource::ID_TRACK_PLAYED),NULL);
      }
    else {
      count_track_remaining=0;
      application->addTimeout(source,GMSource::ID_TRACK_PLAYED,TIME_SEC(time));
      }
    }
  else if (player->playing()) {
    player->getTrackInformation(information);
    }

  /// Update the Remote Control
  //remote->setToolTip(GMStringVal(information.no)+". "+information.title+"\n"+information.artist+"\n"+information.album);

  if (remote && player->playing()) {
    remote->display(information);
    }


  /// Update Ticker Text
  FXString statustext = FXString("Now playing: " + information.artist + " - " + information.title);
  mainwindow->statusbar->getStatusLine()->setNormalText(statustext);

  /// Make sure Volume Level is up 2 date
  mainwindow->update_volume_display(player->getVolume());

  if (remote) remote->update_volume_display(player->getVolume());

  if (notify) application->addTimeout(this,ID_PLAY_NOTIFY,TIME_MSEC(500));
  }

#if 0
void GMPlayerManager::update_track_display_nodb() {
  GMTrack information;

  /// Get Track Information from the database
  player->getTrackInformation(information);

  /// Count Track
  if (source) {
    FXint time = (FXint) (((double)information.time) * 0.80);
    if (time <= 5) {
      application->removeTimeout(source,GMSource::ID_TRACK_PLAYED);
      source->handle(this,FXSEL(SEL_TIMEOUT,GMSource::ID_TRACK_PLAYED),NULL);
      }
    else {
      count_track_remaining=0;
      application->addTimeout(source,GMSource::ID_TRACK_PLAYED,TIME_SEC(time));
      }
    }

  /// Update the Remote Control
  /// Update the Remote Control
#if FOXVERSION >= FXVERSION(1,7,12)
  remote->setToolTip(FXString::value(information.no)+". "+information.title+"\n"+information.artist+"\n"+information.album);
#else
  remote->setToolTip(FXStringVal(information.no)+". "+information.title+"\n"+information.artist+"\n"+information.album);
#endif



  /// Update Ticker Text
  FXString statustext = FXString("Now playing: " + information.artist + " - " + information.title);
  mainwindow->statusbar->getStatusLine()->setNormalText(statustext);

#ifdef HAVE_DBUS
  if (getPreferences().mugshot_notification){
    DBusMessage * msg = dbus_message_new_signal("/org/fiftplanet/gogglesmm","org.gnome.Banshee.Mugshot","StateChangedEvent");
    if (msg) {
      dbus_uint32_t serial;
      dbus_int32_t state=1;
      const FXchar * c_artist=information.artist.text();
      const FXchar * c_album=information.album.text();
      const FXchar * c_title=information.title.text();

      dbus_message_append_args(msg,DBUS_TYPE_INT32,&state,
                                   DBUS_TYPE_STRING,&c_artist,
                                   DBUS_TYPE_STRING,&c_title,
                                   DBUS_TYPE_STRING,&c_album,
                                   DBUS_TYPE_INT32,&information.time,
                                   DBUS_TYPE_INVALID);

      dbus_connection_send(dbus->connection(),msg,&serial);
      dbus_message_unref(msg);
      dbus_connection_flush(dbus->connection());
      }
    }
#endif
  }
#endif


void GMPlayerManager::handle_async_events() {

  /// Get Events from player
  player->handle_async_events();

  /// Mark Time
  mainwindow->update_elapsed_time(player->getMinutes(),player->getSeconds(),player->getPosition(),player->playing(),player->seekable());

  if (remote)
    remote->elapsed_time(player->getMinutes(),player->getSeconds(),player->getPosition(),player->playing());
//	if (miniwindow->shown() && player->playing()){
//    miniwindow->update_elapsed_time(player->getMinutes(),player->getSeconds(),player->playing());
//		}

  }

FXint GMPlayerManager::volume() const{
  return player->getVolume();
  }

void GMPlayerManager::volume(FXint l) {
  player->setVolume(l);
  }

FXbool GMPlayerManager::can_stop() const {
  if (player->playing() || fetch) return true;
  return false;
  }

FXbool GMPlayerManager::can_play() const {
  if (!player->playing() && getTrackView()->hasTracks() && fetch==NULL) return true;
  return false;
  }

FXbool GMPlayerManager::can_pause() const {
  if (player->playing() && !player->pausing())
    return true;
  return false;
  }

FXbool GMPlayerManager::can_unpause() const {
  if (player->playing() && player->pausing())
    return true;
  return false;
  }

FXbool GMPlayerManager::can_next() const {
  if (player->playing() && !player->pausing() && getTrackView()->getNumTracks()>1)
    return true;
  return false;
  }

FXbool GMPlayerManager::can_prev() const {
  if (player->playing() && !player->pausing() && getTrackView()->getNumTracks()>1)
    return true;
  return false;
  }

FXbool GMPlayerManager::can_fastforward() const {
  if (player->playing() && player->getSpeed()<XINE_SPEED_FAST_4 && player->getSpeed()>=XINE_SPEED_SLOW_4)
    return true;
  return false;
  }

FXbool GMPlayerManager::can_fastbackward() const {
  if (player->playing() && player->getSpeed()>XINE_SPEED_SLOW_4)
    return true;
  return false;
  }
#if FOXVERSION < 107000
void GMPlayerManager::setSleepTimer(FXuint ms) {
  if (ms==0)
    application->removeTimeout(this,GMPlayerManager::ID_SLEEP_TIMER);
  else
    application->addTimeout(this,GMPlayerManager::ID_SLEEP_TIMER,ms);
  }
#else
void GMPlayerManager::setSleepTimer(FXlong ns) {
  if (ns==0)
    application->removeTimeout(this,GMPlayerManager::ID_SLEEP_TIMER);
  else
    application->addTimeout(this,GMPlayerManager::ID_SLEEP_TIMER,ns);
  }
#endif

FXbool GMPlayerManager::hasSleepTimer() {
  return application->hasTimeout(this,GMPlayerManager::ID_SLEEP_TIMER);
  }

void GMPlayerManager::show_message(const FXString & msg){
  FXMessageBox::error(mainwindow,MBOX_OK,"Error",msg.text());
  }


void GMPlayerManager::showRemote(){
  if (!remote) {
    remote = new GMRemote(application,this,ID_REMOTE);
    remote->create();
    update_track_display();
    }
  }

void GMPlayerManager::hideRemote(){
  if (remote) {
    remote->handle(this,FXSEL(SEL_CLOSE,0),NULL);
    }
  }


long GMPlayerManager::onCmdCloseRemote(FXObject*,FXSelector,void*){
  getMainWindow()->show();
  if (getPreferences().gui_always_show_remote){
    return 1;
    }
  remote->writeRegistry();
  remote=NULL;
  return 0;
  }

long GMPlayerManager::onCmdCloseWindow(FXObject*,FXSelector,void*){
  if (getPreferences().gui_hide_player_when_close) {
    getMainWindow()->hide();
    if (!remote) {
      remote = new GMRemote(application,this,ID_REMOTE);
      remote->create();
      update_track_display(false);
      }
    }
  else {
    getMainWindow()->handle(this,FXSEL(SEL_COMMAND,GMWindow::ID_QUIT),NULL);
    }
  return 1;
  }




long GMPlayerManager::onCmdDownloadComplete(FXObject*,FXSelector,void*){
  //fxmessage("=> Download complete\n");
  if ( fetch ) {
    if (fetch->mrl.no()) {
      play(fetch->mrl[0]);
      }
    else {
      reset_track_display();
      }
    delete fetch;
    fetch=NULL;
    }
  return 1;
  }

void GMPlayerManager::list_tracks(GMTrackList *) {
/*
  GMTrackArray tracks;
  player->list_cda_tracks(tracks);
  for (int i=0;i<tracks.no();i++){
    list->appendItem(-1,tracks[i].title.text(),tracks[i].artist.text(),tracks[i].album.text(),tracks[i].genre.text(),tracks[i].time,tracks[i].no);
    }
*/
  }

/*
  FXint p;
  FXint c;
  FXint t,total_hours,total_minutes,total_seconds;

  FXint num=0;
  char ** tracks;
FXint tracktime;

  tracks = xine_get_autoplay_mrls(xine,"CD",&num);
  if (num) {
    open(tracks[0]);
    artist = new GMArtist(xine_get_meta_info(so,XINE_META_INFO_ARTIST));
    GMAlbumList * albumlist = artist->getAlbums();
    GMAlbum * album = new GMAlbum(xine_get_meta_info(so,XINE_META_INFO_ALBUM));
    albumlist->append(album);
    GMSongList * songlist=album->getSongs();
    for (FXint i=0;i<num;i++){
        open(tracks[i]);
        tracktime=0;
        if (xine_get_pos_length(so,&p,&c,&tracktime)){
          tracktime/=1000;
          }
        new GMSong(tracks[i],xine_get_meta_info(so,XINE_META_INFO_TITLE),artist,album,i+1,tracktime,"",xine_get_meta_info(so,XINE_META_INFO_YEAR));
        }
    }
*/

