/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <fx.h>
#include "common.h"
#include "version.h"
#include "GMThread.h"
#include "GMTrackDatabase.h"
#include "GMSearch.h"
#include "GMTag.h"
#include "GMAnimImage.h"
#include "icons.h"

#if APPLICATION_BETA > 0
#define DATABASE_FILENAME "goggles_beta.db"
#else
#define DATABASE_FILENAME "goggles.db"
#endif


GMSearchProgress::GMSearchProgress() : count(0) {
  }

void GMSearchProgress::load(FXStream & store){
  store >> count;
  store >> filename;
  store >> directory;
  }

void GMSearchProgress::save(FXStream & store) const {
  store << count;
  store << filename;
  store << directory;
  }

GMSearchThread::GMSearchThread(const FXStringList & list,FXint pl,const GMPreferences & pref,FXObject*tgt) : GMThread(tgt), options(pref), files(list), playlist(pl),lastid(0) {
  }

GMSearchThread::~GMSearchThread(){
  }

FXint GMSearchThread::run(){
  FXString dir = FXSystem::getHomeDirectory() + PATHSEPSTRING + ".goggles";
  FXString databasefilename = dir + PATHSEPSTRING + DATABASE_FILENAME;

  if (!database.init(databasefilename)){
    const char * msg = "Unable to open the database";
    feedback.message(target,FXSEL(SEL_COMMAND,GMSearchDialog::ID_SEARCH_ERROR),msg,sizeof(msg));
    return 1;
    }

  for (FXint i=0;i<files.no();i++){
    if (FXStat::isDirectory(files[i])){
      path=files[i];
      listDirectory(files[i]);
      }
    else {
      FXString ext = FXPath::extension(files[i]);
      if (comparecase(ext,"mp3")==0 || comparecase(ext,"ogg")==0 || comparecase(ext,"flac")==0 || comparecase(ext,"mpc")==0
#ifndef DISABLE_ASF_SUPPORT
      || comparecase(ext,"wma")==0 || comparecase(ext,"asf")==0
#endif
#ifndef DISABLE_MP4_SUPPORT
      || comparecase(ext,"m4a")==0 || comparecase(ext,"mp4")==0 || comparecase(ext,"aac")==0 || comparecase(ext,"m4p")==0) {
#else
        ) {
#endif
        path=FXPath::directory(files[i]);
        insertFile(FXPath::directory(files[i]),FXPath::name(files[i]),i);
        }
      }
    }

  if (running) {
    feedback.message(target,FXSEL(SEL_COMMAND,GMSearchDialog::ID_SEARCH_DONE),NULL,0);
    }
  return 1;
  }

void GMSearchThread::sendProgress() {
  FXuchar * data;
  FXuval size;
  FXMemoryStream mem;
  if (mem.open(FXStreamSave,NULL)){
    progress.save(mem);
    mem.takeBuffer(data,size);
    feedback.message(target,FXSEL(SEL_COMMAND,GMSearchDialog::ID_SEARCH_PROGRESS),data,size);
    mem.giveBuffer(data,size);
    mem.close();
    }
  }


void GMSearchThread::insertFile(const FXString & searchpath,const FXString & file,FXint num) {
  FXbool is_audio_file=false;
  FXint id;
  GMTrack info;
  if (database.hasTrack(searchpath,file,id)) {
    if (playlist!=-1)
      database.insertTrackInPlaylist(playlist,id);
    if (lastid==0)
      lastid=id;

    progress.directory=FXPath::relative(path,searchpath);
    progress.filename=file;
    sendProgress();
    return;
    }

  info.mrl=searchpath + PATHSEPSTRING + file;

  GMTag::getTrackInformation(info,is_audio_file);

  if (!is_audio_file) return;

  progress.directory=FXPath::relative(path,searchpath);
  progress.filename=FXPath::name(info.mrl);
  progress.count++;
  sendProgress();

  FXbool dir_as_album=FALSE;

  /// Check for Empty Title Tag
  if (info.title.empty()){
    if (options.import_use_filename_for_title){
      info.title = FXPath::title(info.mrl);
      if (options.import_replace_underscores) info.title = info.title.substitute("_"," ");
      }
    else {
      info.title =options.import_default_user_title;
      }
    }

  /// Check for Empty Album Tag
  if (info.album.empty()){
    if (options.import_use_directory_as_album){
      dir_as_album=TRUE;
      info.album = FXPath::name(searchpath);
      if (options.import_replace_underscores) info.album = info.album.substitute("_"," ");
      }
    else {
      info.album = options.import_default_user_album;
      }
    }

  /// Check for Empty Artist Tag
  if (info.artist.empty()){
    if (options.import_use_directory_as_artist){

      if (dir_as_album && (FXPath::isTopDirectory(searchpath) || FXPath::isTopDirectory(FXPath::upLevel(searchpath)) ))
        info.artist = options.import_default_user_artist;
      else if (dir_as_album)
        info.artist = FXPath::name(FXPath::upLevel(searchpath));
      else
        info.artist = FXPath::name(searchpath);

      if (options.import_replace_underscores) info.artist = info.artist.substitute("_"," ");
      }
    else {
      info.artist = options.import_default_user_artist;
      }
    }

  /// Check for Empty Category Tag
  if (info.genre.empty()){
    info.genre = options.import_default_user_genre;
    }

  /// Check for Empty Track Tag
  if (info.no==0 && options.import_track_from_filelist){
    info.no=(num+1);
    }

  if (!database.insertTrack(info,id)) {
    const char * msg = "Unable to insert track into the database";
    feedback.message(target,FXSEL(SEL_COMMAND,GMSearchDialog::ID_SEARCH_DONE),msg,sizeof(msg));
    running=FALSE;
    }
  else {
    lastid=id;
    if (playlist>=0)
      database.insertTrackInPlaylist(playlist,id);
    }
  }

int GMSearchThread::getLastInserted() const{
  return lastid;
  }

void GMSearchThread::listDirectory(const FXString & searchpath){
  register FXint i=0;
  FXint no=0;

  FXString * files;
  FXString track;
  GMTrack info;

  /// For all Subdirectories, call listDirectory...
  no = FXDir::listFiles(files,searchpath,"*",FXDir::AllDirs|FXDir::NoParent|FXDir::NoFiles);
  for (i=0;(i<no)&&running;i++){
    if (!FXStat::isLink(searchpath+PATHSEPSTRING+files[i]))
      listDirectory(searchpath+PATHSEPSTRING+files[i]);
    }
  delete [] files;

  if (!running) return;

  /// Ok.. we're done, now list the rest of the files in this directory....
  no=FXDir::listFiles(files,searchpath,"*.(ogg,mp3,mpc,flac"
#ifndef DISABLE_MP4_SUPPORT
",mp4,m4a,aac,m4p"
#endif
#ifndef DISABLE_ASF_SUPPORT
",asf,wma"
#endif
")",FXDir::NoDirs|FXDir::NoParent|FXDir::CaseFold);

  for (i=0;(i<no)&&running;i++){
    insertFile(searchpath,files[i],i);
    }
  delete [] files;
  }



// Define Message Map
FXDEFMAP(GMSearchDialog) GMSearchDialogMap[]={
  //____Message_Type______________ID_______________Message_Handler___
  FXMAPFUNC(SEL_COMMAND,GMSearchDialog::ID_SEARCH_DONE,GMSearchDialog::onSearchDone),
  FXMAPFUNC(SEL_COMMAND,GMSearchDialog::ID_SEARCH_ERROR,GMSearchDialog::onSearchError),
  FXMAPFUNC(SEL_COMMAND,GMSearchDialog::ID_SEARCH_PROGRESS,GMSearchDialog::onSearchProgress),
  FXMAPFUNC(SEL_CLOSE,0,GMSearchDialog::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND,FXDialogBox::ID_CANCEL,GMSearchDialog::onCmdCancel),
  };

// Object implementation
FXIMPLEMENT(GMSearchDialog,FXDialogBox,GMSearchDialogMap,ARRAYNUMBER(GMSearchDialogMap))

GMSearchDialog::GMSearchDialog(FXApp * app,const FXStringList & path,FXint playlist,const GMPreferences & prefs,FXFont * font) :
  FXDialogBox(app,"Goggles Music Manager - Scanning Files...",DECOR_ALL,0,0,0,0,0,0,0,0,0,0){

  animation=NULL;
  searchthread = new GMSearchThread(path,playlist,prefs,this);
  setup(font);
  }

GMSearchDialog::GMSearchDialog(FXWindow* owner,const FXStringList & path,FXint playlist,const GMPreferences & prefs,FXFont * font) :
  FXDialogBox(owner,"Goggles Music Manager - Scanning Files...",DECOR_TITLE|DECOR_BORDER,0,0,0,0,0,0,0,0,0,0){
  animation=NULL;
  searchthread = new GMSearchThread(path,playlist,prefs,this);
  setup(font);
  }

GMSearchDialog::~GMSearchDialog(){
  delete searchthread;
  delete animation_image;
  }

void GMSearchDialog::setup(FXFont * titlefont){
  FXIconSource source(getApp());
  animation_image = source.loadImageData(x32_animations_processworking_png,"png");
  animation_image->blend(FXRGB(255,255,255));
  animation_image->create();

  FXLabel*label;
  FXHorizontalFrame * header = new FXHorizontalFrame(this,LAYOUT_FILL_X,0,0,0,0,0,20,0,0,0,0);
  header->setBackColor(FXRGB(255,255,255));
  FXVerticalFrame * frame = new FXVerticalFrame(header,LAYOUT_FILL_X,0,0,0,0,0,0,0,0,0,0);
  label = new FXLabel(frame,"Scanning Files...",NULL,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,10,0,10,0);
  label->setBackColor(FXRGB(255,255,255));
  label->setFont(titlefont);
  label = new FXLabel(frame,"Please wait. This may take a while.",NULL,LAYOUT_FILL_X|JUSTIFY_LEFT|TEXT_AFTER_ICON,0,0,0,0,10+30,0,0,10);
  label->setBackColor(FXRGB(255,255,255));

  animation = new GMAnimImage(header,animation_image,FRAME_NONE|LAYOUT_CENTER_Y);
  animation->setBackColor(FXRGB(255,255,255));
  new FXSeparator(this,LAYOUT_FILL_X|SEPARATOR_GROOVE|LAYOUT_SIDE_TOP);

  FXVerticalFrame * main = new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0,0,0);
  FXMatrix * matrix = new FXMatrix(main,2,MATRIX_BY_COLUMNS|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,15,15,15,15);
  new FXLabel(matrix,"Found:",NULL,LABEL_NORMAL|LAYOUT_FILL_X|JUSTIFY_RIGHT|LAYOUT_CENTER_Y);
  text_count = new FXTextField(matrix,10,NULL,0,TEXTFIELD_READONLY|TEXTFIELD_INTEGER|LAYOUT_FILL_X|JUSTIFY_LEFT);
  text_count->disable();
  text_count->setTextColor(FXRGB(0,0,255));
  text_count->setText("0");

  new FXLabel(matrix,"File:",NULL,LABEL_NORMAL|LAYOUT_FILL_X|JUSTIFY_RIGHT|LAYOUT_CENTER_Y);
  text_file = new FXTextField(matrix,40,NULL,0,TEXTFIELD_READONLY|LAYOUT_FILL_X);
  text_file->disable();

  new FXLabel(matrix,"Directory:",NULL,LABEL_NORMAL|LAYOUT_FILL_X|JUSTIFY_RIGHT|LAYOUT_CENTER_Y);
  text_dir = new FXTextField(matrix,40,NULL,0,TEXTFIELD_READONLY|LAYOUT_FILL_X);
  text_dir->disable();

  new FXSeparator(main,SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXHorizontalFrame *closebox=new FXHorizontalFrame(main,LAYOUT_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox,tr(" &Stop Scanning "),NULL,this,FXDialogBox::ID_CANCEL,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_CENTER_X|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  }

int GMSearchDialog::getLastInserted() const{
  return searchthread->getLastInserted();
  }

/// Run modal invocation of the dialog
FXuint GMSearchDialog::execute(FXuint placement){
  create();
  show(placement);
  getApp()->refresh();
  searchthread->start();
  return getApp()->runModalFor(this);
  }

// Close dialog with a cancel
long GMSearchDialog::onCmdCancel(FXObject*,FXSelector,void*){
  searchthread->dispose_and_join();
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
  }


long GMSearchDialog::onSearchProgress(FXObject*,FXSelector,void*ptr){
  FXMemoryStream store;
  if (store.open(FXStreamLoad,*(((FXint*)ptr)-1),(FXuchar*)ptr)){
    GMSearchProgress progress;
    progress.load(store);
    text_count->setText(GMStringVal(progress.count));
    text_file->setText(progress.filename);
    text_dir->setText(progress.directory);
    store.close();
    }
  return 1;
  }

long GMSearchDialog::onSearchDone(FXObject*,FXSelector,void*){
  searchthread->join();
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
  }

long GMSearchDialog::onSearchError(FXObject*,FXSelector,void*ptr){
  const FXchar * msg = (FXchar*)ptr;
  searchthread->join();
  FXMessageBox::error(this,MBOX_OK,"Fatal Error","%s\nPlease contact support if this error keeps occuring.",msg);
  getApp()->stopModal(this,false);
  hide();
  return 1;
  }

