/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <fileref.h>
#include <tag.h>
#include <tstring.h>
#include <id3v2tag.h>
#include <id3v2framefactory.h>

#include <mpegfile.h>
#include <vorbisfile.h>
#include <flacfile.h>
#include <textidentificationframe.h>
#include <attachedpictureframe.h>

/*

      //disc = mp4file->tag()->itemListMap()["disk"].toIntPair().first;
      //fxmessage("%d/%d\n",mp4file->tag()->itemListMap()["disk"].toIntPair().first,mp4file->tag()->itemListMap()["disk"].toIntPair().second);
      fxmessage("%s\n",mp4file->tag()->itemListMap()["disk"].toStringList().front().toCString(true));//.tofirst,mp4file->tag()->itemListMap()["disk"].toIntPair().second);


//    for(TagLib::MP4::ItemListMap::Iterator i = mp4file->tag()->itemListMap().begin(); i != mp4file->tag()->itemListMap().end(); i++) {
 //     fxmessage(": %s\n",i->first.toCString(true));
  //    }

*/

#ifndef DISABLE_ASF_SUPPORT
#include "asfresolve.h"
#endif

#ifndef DISABLE_MP4_SUPPORT
#include "mp4file.h"
#include "mp4tag.h"
#include "mp4resolve.h"
#endif

#include "common.h"
#include "GMTag.h"

#include "FXPNGIcon.h"
#include "FXJPGIcon.h"

namespace GMTag {

#ifndef DISABLE_ASF_SUPPORT
static ASFFileTypeResolver asfresolver;
#endif
#ifndef DISABLE_MP4_SUPPORT
static MP4FileTypeResolver mp4resolver;
#endif

void init(){
#ifndef DISABLE_ASF_SUPPORT
  TagLib::FileRef::addFileTypeResolver(&asfresolver);
#endif
#ifndef DISABLE_MP4_SUPPORT
  TagLib::FileRef::addFileTypeResolver(&mp4resolver);
#endif
  TagLib::ID3v2::FrameFactory::instance()->setDefaultTextEncoding(TagLib::String::UTF16);
  }


FXbool sync(GMTrack & info){
  if (!FXStat::isWritable(info.mrl)) return false;


  TagLib::FileRef file(info.mrl.text(),false);
  if (file.isNull() || !file.tag())
    return false;

  TagLib::Tag * tag = file.tag();

  tag->setTitle(TagLib::String(info.title.text(),TagLib::String::UTF8));
  tag->setArtist(TagLib::String(info.artist.text(),TagLib::String::UTF8));
  tag->setAlbum(TagLib::String(info.album.text(),TagLib::String::UTF8));
  tag->setGenre(TagLib::String(info.genre.text(),TagLib::String::UTF8));
  tag->setYear(info.year);
  tag->setTrack(GMTRACKNO(info.no));


  TagLib::FLAC::File * flacfile = NULL;
  TagLib::Ogg::Vorbis::File * oggfile = NULL;
  TagLib::MPEG::File * mpgfile = NULL;
  TagLib::MP4::File * mp4file = NULL;

  FXString disc = GMStringVal(GMDISCNO(info.no));

  if ((oggfile = dynamic_cast<TagLib::Ogg::Vorbis::File*>(file.file()))) {
    if (oggfile->tag()) {
      if (GMDISCNO(info.no)==0)
        oggfile->tag()->removeField("DISCNUMBER");
      else
        oggfile->tag()->addField("DISCNUMBER",TagLib::String(disc.text(),TagLib::String::UTF8),true);
      }
    }
  else if ((flacfile = dynamic_cast<TagLib::FLAC::File*>(file.file()))){
    if (flacfile->xiphComment()) {
      if (GMDISCNO(info.no)==0)
        flacfile->xiphComment()->removeField("DISCNUMBER");
      else
        flacfile->xiphComment()->addField("DISCNUMBER",TagLib::String(disc.text(),TagLib::String::UTF8),true);
      }
    else if (flacfile->ID3v2Tag()){
      if (GMDISCNO(info.no)==0) {
        flacfile->ID3v2Tag()->removeFrames("TPOS");
        }
      else if (!flacfile->ID3v2Tag()->frameListMap()["TPOS"].isEmpty()){
        flacfile->ID3v2Tag()->frameListMap()["TPOS"].front()->setText(TagLib::String(disc.text(),TagLib::String::UTF8));
        }
      else {
        TagLib::ID3v2::TextIdentificationFrame *frame = new TagLib::ID3v2::TextIdentificationFrame("TPOS",TagLib::ID3v2::FrameFactory::instance()->defaultTextEncoding());
        frame->setText(TagLib::String(disc.text(),TagLib::String::UTF8) );
        flacfile->ID3v2Tag()->addFrame(frame);
        }
      }
    }
  else if ((mpgfile = dynamic_cast<TagLib::MPEG::File*>(file.file()))){
    if (mpgfile->ID3v2Tag()){
      if (GMDISCNO(info.no)==0) {
        mpgfile->ID3v2Tag()->removeFrames("TPOS");
        }
      else if (!mpgfile->ID3v2Tag()->frameListMap()["TPOS"].isEmpty()){
        mpgfile->ID3v2Tag()->frameListMap()["TPOS"].front()->setText(TagLib::String(disc.text(),TagLib::String::UTF8));
        }
      else {
        TagLib::ID3v2::TextIdentificationFrame *frame = new TagLib::ID3v2::TextIdentificationFrame("TPOS",TagLib::ID3v2::FrameFactory::instance()->defaultTextEncoding());
        frame->setText(TagLib::String(disc.text(),TagLib::String::UTF8) );
        mpgfile->ID3v2Tag()->addFrame(frame);
        }
      }
    }
#ifndef DISABLE_MP4_SUPPORT
  else if ((mp4file = dynamic_cast<TagLib::MP4::File*>(file.file()))){
    }
#endif

  return file.save();
  }






#define FLAC_LAST_BLOCK(h) (h&0x80)
#define FLAC_BLOCK_TYPE(h) (h&0x7f)
#define FLAC_BLOCK_SIZE(h) ( (((h&0xFF00)>>8)<<16) | (((h&0xFF0000)>>16)<<8) | ((h&0xFF000000)>>24) )

enum {
  FLAC_BLOCK_STREAMINFO = 0,
  FLAC_BLOCK_PICTURE = 6
  };  

enum {
	FLAC_STREAMINFO_BLOCK = 0,
	FLAC_VORBIS_COMMENT_BLOCK = 4,
	FLAC_PICTURE_BLOCK = 6
	};

FXIcon * load_flac_picture(FXStream & store,FXint size) {
	FXbool swap=store.swapBytes();
  store.setBigEndian(true);
	FXuint picture_type;
	FXuint picture_width;
	FXuint picture_height;
	FXuint picture_bps;
	FXuint picture_ncolors;
	FXString mimetype;
	FXString description;
	FXuint nlength;

	store >> picture_type;

	store >> nlength;
	mimetype.length(nlength);
	store.load(&mimetype[0],nlength);

	store >> nlength;
	description.length(nlength);
	store.load(&description[0],nlength);

	store >> picture_width;
	store >> picture_height;
	store >> picture_bps;
	store >> picture_bps;
	store >> picture_ncolors;

	store.swapBytes(swap);

  FXIconSource src(FXApp::instance());

  if ((comparecase(mimetype,"image/jpg")==0) || (comparecase(mimetype,"image/jpeg")==0)) {
    return src.loadScaledIconStream(store,size,1,FXJPGIcon::fileExt);
    }
  else if (comparecase(mimetype,FXPNGIcon::mimeType)==0) {
    return src.loadScaledIconStream(store,size,1,FXPNGIcon::fileExt);
    }
  return NULL;
  }

FXIcon * load_flac_albumart(const FXString & mrl,FXint size) {
  FXchar  marker[4];
  FXuint  header;
  FXint   nread;
  FXFile io;
 
  if (io.open(mrl,FXIO::Reading)) {

    nread = io.readBlock(marker,4);
    if (nread!=4 || compare(marker,"fLaC",4)) return NULL;

    nread = io.readBlock(&header,4);
    if (nread!=4 || FLAC_BLOCK_TYPE(header)!=FLAC_BLOCK_STREAMINFO || FLAC_BLOCK_SIZE(header)!=34  || FLAC_LAST_BLOCK(header)) return NULL;

    io.position(34,FXIO::Current);  

    while(!FLAC_LAST_BLOCK(header)) {

      nread = io.readBlock(&header,4);
      if (nread!=4) return false;

      if (FLAC_BLOCK_TYPE(header)==FLAC_BLOCK_PICTURE) {
        FXuchar * pictureblock = NULL;
        allocElms(pictureblock,FLAC_BLOCK_SIZE(header));
        nread = io.readBlock(pictureblock,FLAC_BLOCK_SIZE(header));
        if (nread!=(FXint)FLAC_BLOCK_SIZE(header)) {
          freeElms(pictureblock);
          return false;
          }
        FXMemoryStream stream;
        stream.open(FXStreamLoad,FLAC_BLOCK_SIZE(header),pictureblock);      
        FXIcon * icon = load_flac_picture(stream,size);
        stream.close();
        freeElms(pictureblock);
        if (icon) return icon;
        }
      else if (!FLAC_LAST_BLOCK(header)){ 
        io.position(FLAC_BLOCK_SIZE(header),FXIO::Current); 
        }
      }
    }
  return NULL;
  }





FXIcon * albumart(const FXString & mrl,FXint size) {
  FXString mime;
  FXString extension = FXPath::extension(mrl);

  TagLib::File * file = NULL;
  TagLib::ID3v2::Tag * tagv2 = NULL;

  if (comparecase(extension,"mp3")==0) {
    TagLib::MPEG::File * f = new TagLib::MPEG::File(mrl.text(),false);
    tagv2=f->ID3v2Tag();
    file=f;
    }
  else if (comparecase(extension,"flac")==0){
    FXIcon * icon = load_flac_albumart(mrl,size);
    if (icon) return icon;
    TagLib::FLAC::File * f = new TagLib::FLAC::File(mrl.text(),false);
    tagv2=f->ID3v2Tag();
    file=f;
    }
  else {
    return NULL;
    }

  FXIconSource src(FXApp::instance());

  if (file->isValid() && tagv2) {
    TagLib::ID3v2::FrameListMap tags    = tagv2->frameListMap();   
    TagLib::ID3v2::FrameList pictures   = tags["APIC"];
    if(!pictures.isEmpty()){
      if (pictures.size()==1) {
        TagLib::ID3v2::AttachedPictureFrame *picture = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame*>(pictures.front());
        mime = picture->mimeType().toCString(true);
        if ((comparecase(mime,"image/jpg")==0) || (comparecase(mime,"image/jpeg")==0)) {
          return src.loadScaledIconData(picture->picture().data(),size,1,FXJPGIcon::fileExt);
          }
        else if (comparecase(mime,FXPNGIcon::mimeType)==0) {
          return src.loadScaledIconData(picture->picture().data(),size,1,FXPNGIcon::fileExt);
          }
        else {

          }
        }
      else {
        // todo
        TagLib::ID3v2::AttachedPictureFrame *picture = dynamic_cast<TagLib::ID3v2::AttachedPictureFrame*>(pictures.front());
        mime = picture->mimeType().toCString(true);
        if ((comparecase(mime,"image/jpg")==0) || (comparecase(mime,"image/jpeg")==0)) {
          return src.loadScaledIconData(picture->picture().data(),size,1,FXJPGIcon::fileExt);
          }
        else if (comparecase(mime,FXPNGIcon::mimeType)==0) {
          return src.loadScaledIconData(picture->picture().data(),size,1,FXPNGIcon::fileExt);
          }
        }
      }


    }

  delete file;
  return NULL;
  }


/// Return Track Information
FXbool getTrackInformation(GMTrack & info,FXbool & is_audio_file) {

  TagLib::FileRef file(info.mrl.text());
  if (file.isNull() || !file.tag())
    return false;

  TagLib::Tag * tag = file.tag();
  TagLib::AudioProperties *properties = file.audioProperties();

  info.artist = tag->artist().toCString(true);
  info.album  = tag->album().toCString(true);
  info.title  = tag->title().toCString(true);
  info.genre  = tag->genre().toCString(true);
  info.year   = tag->year();
  info.no		  =	GMALBUMNO(0,FXMIN(tag->track(),0xFFFF));

  info.artist	= info.artist.trim();
  info.album 	= info.album.trim();
  info.title 	= info.title.trim();
  info.genre	= info.genre.trim();

  if (properties)
    info.time = properties->length();
  else
    info.time = 0;

  is_audio_file=true;


  FXString disc;
  TagLib::MPEG::File * mpgfile = NULL;
  TagLib::Ogg::Vorbis::File * oggfile = NULL;
  TagLib::FLAC::File  * flacfile = NULL;
  TagLib::MP4::File  * mp4file = NULL;


  if ((oggfile = dynamic_cast<TagLib::Ogg::Vorbis::File*>(file.file()))) {
    if (oggfile->tag()) {
      if (!oggfile->tag()->fieldListMap()["DISCNUMBER"].isEmpty())
        disc = oggfile->tag()->fieldListMap()["DISCNUMBER"].front().toCString(true);

      if (!oggfile->tag()->fieldListMap()["ALBUMARTIST"].isEmpty())
        info.artist = oggfile->tag()->fieldListMap()["ALBUMARTIST"].front().toCString(true);
      }
    }
  else if ((flacfile = dynamic_cast<TagLib::FLAC::File*>(file.file()))){
    if (flacfile->xiphComment()) {

      if (!flacfile->xiphComment()->fieldListMap()["DISCNUMBER"].isEmpty())
        disc = flacfile->xiphComment()->fieldListMap()["DISCNUMBER"].front().toCString(true);

      if (!flacfile->xiphComment()->fieldListMap()["ALBUMARTIST"].isEmpty())
        info.artist = flacfile->xiphComment()->fieldListMap()["ALBUMARTIST"].front().toCString(true);
      
      }
    else if (flacfile->ID3v2Tag()) {
      if ( !flacfile->ID3v2Tag()->frameListMap()["TPOS"].isEmpty() )
        disc = flacfile->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().toCString(true);

      if ( !flacfile->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() )
        info.artist = flacfile->ID3v2Tag()->frameListMap()["TPE2"].front()->toString().toCString(true);

      }
    }
  else if ((mpgfile = dynamic_cast<TagLib::MPEG::File*>(file.file()))){
    if (mpgfile->ID3v2Tag()) {

      if (!mpgfile->ID3v2Tag()->frameListMap()["TPOS"].isEmpty() )
        disc = mpgfile->ID3v2Tag()->frameListMap()["TPOS"].front()->toString().toCString(true);

      if ( !mpgfile->ID3v2Tag()->frameListMap()["TPE2"].isEmpty() )
        info.artist = mpgfile->ID3v2Tag()->frameListMap()["TPE2"].front()->toString().toCString(true);

      }
    }
#ifndef DISABLE_MP4_SUPPORT
  else if ((mp4file = dynamic_cast<TagLib::MP4::File*>(file.file()))){
    if (mp4file->tag() &&  mp4file->tag()->itemListMap().contains("disk") && mp4file->tag()->itemListMap()["disk"].isValid()){
      FXint num = mp4file->tag()->itemListMap()["disk"].toIntPair().first;  
#if FOXVERSION >= FXVERSION(1,7,12)
      info.no = GMALBUMNO(FXMIN(num,0xFFFF),GMTRACKNO(info.no));
#else
      info.no = GMALBUMNO(FXMIN(num,0xFFFF),GMTRACKNO(info.no));
#endif
      }
    }
#endif

  if (!disc.empty()) {
    disc.trim();
    disc = disc.before('/');
    if (!disc.empty()) {
#if FOXVERSION >= FXVERSION(1,7,12)
      info.no = GMALBUMNO(FXMIN(disc.toUInt(),0xFFFF),GMTRACKNO(info.no));
#else
      info.no = GMALBUMNO(FXMIN(FXUIntVal(disc),0xFFFF),GMTRACKNO(info.no));
#endif
      }
    }

  return true;
  }


FXbool properties(const FXString & mrl,Properties & p) {
  p.bitrate=-1;
  p.samplerate=-1;
  p.channels=-1;

  TagLib::FileRef file(mrl.text());
  if (file.isNull())
    return false;

  TagLib::AudioProperties *prop = file.audioProperties();

  if (!prop)
    return false;

  p.bitrate    = prop->bitrate();
  p.samplerate = prop->sampleRate();
  p.channels   = prop->channels();
  return true;
  }
}
