/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <limits.h>
#include "common.h"
#include <FXPNGIcon.h>
#include "GMTrackList.h"
#include "GMTrackItem.h"
#include "GMList.h"
#include "GMSource.h"
#include "GMHeaderButton.h"
#include "GMTrackView.h"
#include "GMPlayerManager.h"
#include "icons.h"

#define SIDE_SPACING             4    // Left or right spacing between items
#define DETAIL_TEXT_SPACING      2    // Spacing between text and icon in detail icon mode
#define MINI_TEXT_SPACING        2    // Spacing between text and icon in mini icon mode
#define BIG_LINE_SPACING         6    // Line spacing in big icon mode
#define BIG_TEXT_SPACING         2    // Spacing between text and icon in big icon mode
#define ITEM_SPACE             128    // Default space for item


#define ICON_WIDTH 10
#define ICON_HEIGHT 15

FXint GMDBTrackItem::max_trackno=0;
FXint GMDBTrackItem::max_queue=0;


FXint GMDBTrackItem::max_digits(FXint num){
  if (num>9) {
    FXint n=0;
    while(num>0) { ++n; num/=10; }
    return n;
    }
  return 1;
  }



FXIMPLEMENT(GMDBTrackItem,GMTrackItem,NULL,0);

GMDBTrackItem::GMDBTrackItem(FXint track_id,const FXchar * track_title,const FXchar * track_artist,const FXchar * track_album,const FXchar * track_genre,FXint track_time,FXuint track_no,FXint track_queue,FXushort track_year) :
  GMTrackItem(), artist(track_artist),album(track_album),genre(track_genre),queue(track_queue),year(track_year){
  data=(void*)(FXival)track_id;
  // dirty but saves 8 bytes per item
  bigIcon=(FXIcon*)(void*)(FXival)track_time;
  miniIcon=(FXIcon*)(void*)(FXuval)track_no;
  label=track_title;
  state|=FXIconItem::DRAGGABLE;
  }


GMDBTrackItem::~GMDBTrackItem(){
  bigIcon=NULL;
  miniIcon=NULL;
  }


const FXString * GMDBTrackItem::getColumnData(FXint type,FXString &text,FXuint & justify,FXint & max) const{
  const FXString * textptr;
  justify=COLUMN_JUSTIFY_NORMAL;
  switch(type){
    case HEADER_QUEUE   : text.format("%d",queue);
                          textptr=&text;
                          justify=COLUMN_JUSTIFY_LEFT_RIGHT_ALIGNED;
                          max=GMDBTrackItem::max_queue;
                          break;
    case HEADER_TRACK   : text.format("%d",GMTRACKNO((FXuint)(FXuval)miniIcon));
                          textptr=&text;
                          justify=COLUMN_JUSTIFY_LEFT_RIGHT_ALIGNED;
                          max=GMDBTrackItem::max_trackno;
                          break;
    case HEADER_TITLE   : textptr = &label;  			break;
    case HEADER_ALBUM   : textptr = &album;  			break;
    case HEADER_ARTIST  : textptr = &artist;  		break;
    case HEADER_GENRE   : textptr = &genre;    	  break;
    case HEADER_YEAR    : //justify=COLUMN_JUSTIFY_CENTER_RIGHT_ALIGNED;
													//max=9999; 
                          if (year>0) {text.format("%d",year); textptr=&text; } else textptr=NULL; break;
    case HEADER_TIME    : /*textptr = &timestring;*/
                          text.format("%d:%.2d",((FXint)(FXival)bigIcon)/60,((FXint)(FXival)bigIcon)%60);
                          textptr=&text;
                          justify=COLUMN_JUSTIFY_CENTER_RIGHT_ALIGNED;
                          max=GMTrackItem::max_time;
                          break;
    default							: textptr=NULL;			 			break;
    }
  return textptr;
  }


static inline FXbool begins_with_keyword(const FXString & t){
  for (FXint i=0;i<GMPlayerManager::instance()->getPreferences().gui_sort_keywords.no();i++){
    if (comparecase(t,GMPlayerManager::instance()->getPreferences().gui_sort_keywords[i],GMPlayerManager::instance()->getPreferences().gui_sort_keywords[i].length())==0) return TRUE;
    }
  return FALSE;
  }


FXint GMDBTrackItem::browseSort(const FXIconItem * pa,const FXIconItem * pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0)
    return (GMTrackView::reverse_album) ? -x : x;

  a=b=0;
  if (begins_with_keyword(ta->artist)) a=FXMIN(ta->artist.length()-1,ta->artist.find(' ')+1);
  if (begins_with_keyword(tb->artist)) b=FXMIN(tb->artist.length()-1,tb->artist.find(' ')+1);
  x = compare(&ta->artist[a],&tb->artist[b]);

  if (x!=0)
    return (GMTrackView::reverse_artist) ? -x : x;

  return ascendingTrack(pa,pb);
  }


FXint GMDBTrackItem::ascendingTitle(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;
  register FXint a=0,b=0;
  if (begins_with_keyword(ta->label)) a=FXMIN(ta->label.length()-1,ta->label.find(' ')+1);
  if (begins_with_keyword(tb->label)) b=FXMIN(tb->label.length()-1,tb->label.find(' ')+1);
  return compare(&ta->label[a],&tb->label[b]);
  }

FXint GMDBTrackItem::descendingTitle(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;
  register FXint a=0,b=0;
  if (begins_with_keyword(ta->label)) a=FXMIN(ta->label.length()-1,ta->label.find(' ')+1);
  if (begins_with_keyword(tb->label)) b=FXMIN(tb->label.length()-1,tb->label.find(' ')+1);
  return -compare(&ta->label[a],&tb->label[b]);
  }

FXint GMDBTrackItem::ascendingTrack(const FXIconItem* pa,const FXIconItem* pb){
  register const FXuint a=GMTRACKNO((FXuint)(FXuval)((GMDBTrackItem*)pa)->miniIcon);
  register const FXuint b=GMTRACKNO((FXuint)(FXuval)((GMDBTrackItem*)pb)->miniIcon);
  if (a>b) return 1;
  else if (a<b) return -1;
  return 0;
  }

FXint GMDBTrackItem::descendingTrack(const FXIconItem* pa,const FXIconItem* pb){
  register const FXint a=GMTRACKNO((FXuint)(FXuval)((GMDBTrackItem*)pa)->miniIcon);
  register const FXint b=GMTRACKNO((FXuint)(FXuval)((GMDBTrackItem*)pb)->miniIcon);
  if (a>b) return -1;
  else if (a<b) return 1;
  return 0;
  }

FXint GMDBTrackItem::ascendingQueue(const FXIconItem* pa,const FXIconItem* pb){
  register const FXint a=(FXint)(FXival)((GMDBTrackItem*)pa)->queue;
  register const FXint b=(FXint)(FXival)((GMDBTrackItem*)pb)->queue;
  if (a>b) return 1;
  else if (a<b) return -1;
  return 0;
  }

FXint GMDBTrackItem::descendingQueue(const FXIconItem* pa,const FXIconItem* pb){
  register const FXint a=(FXint)(FXival)((GMDBTrackItem*)pa)->queue;
  register const FXint b=(FXint)(FXival)((GMDBTrackItem*)pb)->queue;
  if (a>b) return -1;
  else if (a<b) return 1;
  return 0;
  }

FXint GMDBTrackItem::ascendingYear(const FXIconItem* pa,const FXIconItem* pb){
  register const FXushort a=(FXushort)(FXival)((GMDBTrackItem*)pa)->year;
  register const FXushort b=(FXushort)(FXival)((GMDBTrackItem*)pb)->year;
  if (a>b) return 1;
  if (a<b) return -1;
  return 0;
  }

FXint GMDBTrackItem::descendingYear(const FXIconItem* pa,const FXIconItem* pb){
  register const FXushort a=(FXushort)(FXival)((GMDBTrackItem*)pa)->year;
  register const FXushort b=(FXushort)(FXival)((GMDBTrackItem*)pb)->year;
  if (a>b) return -1;
  if (a<b) return 1;
  return 0;
  }

FXint GMDBTrackItem::ascendingTime(const FXIconItem* pa,const FXIconItem* pb){
  register const FXint a=(FXint)(FXival)((GMDBTrackItem*)pa)->bigIcon;
  register const FXint b=(FXint)(FXival)((GMDBTrackItem*)pb)->bigIcon;
  if (a>b) return 1;
  if (a<b) return -1;
  return 0;
  }

FXint GMDBTrackItem::descendingTime(const FXIconItem* pa,const FXIconItem* pb){
  register const FXint a=(FXint)(FXival)((GMDBTrackItem*)pa)->bigIcon;
  register const FXint b=(FXint)(FXival)((GMDBTrackItem*)pb)->bigIcon;
  if (a>b) return -1;
  if (a<b) return 1;
  return 0;
  }

FXint GMDBTrackItem::ascendingAlbum(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }

FXint GMDBTrackItem::descendingAlbum(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&tb->album[b],&ta->album[a]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }


FXint GMDBTrackItem::ascendingArtist(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;

  if (begins_with_keyword(ta->artist)) a=FXMIN(ta->artist.length()-1,ta->artist.find(' ')+1);
  if (begins_with_keyword(tb->artist)) b=FXMIN(tb->artist.length()-1,tb->artist.find(' ')+1);
  x = compare(&ta->artist[a],&tb->artist[b]);
  if (x!=0) return x;

  a=b=0;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }


FXint GMDBTrackItem::descendingArtist(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;

  if (begins_with_keyword(ta->artist)) a=FXMIN(ta->artist.length()-1,ta->artist.find(' ')+1);
  if (begins_with_keyword(tb->artist)) b=FXMIN(tb->artist.length()-1,tb->artist.find(' ')+1);
  x = compare(&tb->artist[b],&ta->artist[a]);
  if (x!=0) return x;

  a=b=0;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }

FXint GMDBTrackItem::ascendingGenre(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;

  x = comparecase(ta->genre,tb->genre);
  if (x!=0) return x;

  if (begins_with_keyword(ta->artist)) a=FXMIN(ta->artist.length()-1,ta->artist.find(' ')+1);
  if (begins_with_keyword(tb->artist)) b=FXMIN(tb->artist.length()-1,tb->artist.find(' ')+1);
  x = compare(&ta->artist[a],&tb->artist[b]);
  if (x!=0) return x;

  a=b=0;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }

FXint GMDBTrackItem::descendingGenre(const FXIconItem* pa,const FXIconItem* pb){
  const GMDBTrackItem * const ta = (GMDBTrackItem*)pa;
  const GMDBTrackItem * const tb = (GMDBTrackItem*)pb;

  register FXint a=0,b=0,x;

  x = comparecase(tb->genre,ta->genre);
  if (x!=0) return x;

  if (begins_with_keyword(ta->artist)) a=FXMIN(ta->artist.length()-1,ta->artist.find(' ')+1);
  if (begins_with_keyword(tb->artist)) b=FXMIN(tb->artist.length()-1,tb->artist.find(' ')+1);
  x = compare(&ta->artist[a],&tb->artist[b]);
  if (x!=0) return x;

  a=b=0;
  if (begins_with_keyword(ta->album)) a=FXMIN(ta->album.length()-1,ta->album.find(' ')+1);
  if (begins_with_keyword(tb->album)) b=FXMIN(tb->album.length()-1,tb->album.find(' ')+1);
  x = compareversion(&ta->album[a],&tb->album[b]);

  if (x!=0) return x;
  return ascendingTrack(pa,pb);
  }






FXint GMStreamTrackItem::max_trackno=0;

FXint GMStreamTrackItem::max_digits(FXint num){
  if (num>9) {
    FXint n=0;
    while(num>0) { ++n; num/=10; }
    return n;
    }
  return 1;
  }

FXIMPLEMENT(GMStreamTrackItem,GMTrackItem,NULL,0);

GMStreamTrackItem::GMStreamTrackItem(FXint track_id,const FXchar * track_title,const FXchar * track_genre,FXint track_no,FXint track_bitrate) :
  GMTrackItem(), genre(track_genre) {
  data=(void*)(FXival)track_id;
  // dirty but saves 8 bytes per item
  bigIcon=(FXIcon*)(void*)(FXival)track_bitrate;
  miniIcon=(FXIcon*)(void*)(FXival)track_no;
  label=track_title;
//  state|=FXIconItem::DRAGGABLE;
  }

GMStreamTrackItem::~GMStreamTrackItem(){
  bigIcon=NULL;
  miniIcon=NULL;
  }


const FXString * GMStreamTrackItem::getColumnData(FXint type,FXString &text,FXuint & justify,FXint & max) const{
  const FXString * textptr;
  justify=COLUMN_JUSTIFY_NORMAL;
  switch(type){
    case HEADER_TRACK   : text.format("%d",GMTRACKNO((FXuint)(FXuval)miniIcon));
                          textptr=&text;
                          justify=COLUMN_JUSTIFY_LEFT_RIGHT_ALIGNED;
                          max=GMDBTrackItem::max_trackno;
                          break;
    case HEADER_TITLE   : textptr = &label;  			break;
    case HEADER_GENRE   : textptr = &genre;    	  break;
    case HEADER_BITRATE : text.format("%d",(FXint)(FXival)bigIcon);
                          textptr=&text;
                          break;    
    default							: textptr=NULL;			 			break;
    }
  return textptr;
  }
