/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <limits.h>
#include "common.h"
#include <FXPNGIcon.h>
#include "GMTrackList.h"
#include "GMTrackItem.h"
#include "GMList.h"
#include "GMSource.h"
#include "GMHeaderButton.h"
#include "GMTrackView.h"
#include "GMPlayerManager.h"
#include "icons.h"

#define SIDE_SPACING             4    // Left or right spacing between items
#define DETAIL_TEXT_SPACING      2    // Spacing between text and icon in detail icon mode
#define MINI_TEXT_SPACING        2    // Spacing between text and icon in mini icon mode
#define BIG_LINE_SPACING         6    // Line spacing in big icon mode
#define BIG_TEXT_SPACING         2    // Spacing between text and icon in big icon mode
#define ITEM_SPACE             128    // Default space for item


#define ICON_WIDTH 10
#define ICON_HEIGHT 15






class GMHeader : public FXHeader {
  FXDECLARE(GMHeader)
protected:
  GMHeader();
private:
  GMHeader(const GMHeader&);
  GMHeader &operator=(const GMHeader&);
public:
  /// Construct new header control
  GMHeader(FXComposite* p,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=HEADER_NORMAL,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

  /// Destructor
  virtual ~GMHeader();
  };

FXDEFMAP(GMHeader) GMHeaderMap[]={{0,0,0}
//	FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,0,GMHeader::onRightBtnRelease)
	};

FXIMPLEMENT(GMHeader,FXHeader,GMHeaderMap,ARRAYNUMBER(GMHeaderMap))

GMHeader::GMHeader(){
	}

GMHeader::GMHeader(FXComposite* p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb) :
	FXHeader(p,tgt,sel,opts,x,y,w,h,pl,pr,pt,pb) {
	}

GMHeader::~GMHeader(){
	}




FXDEFMAP(GMTrackList) GMTrackListMap[]={
  FXMAPFUNC(SEL_PAINT,0,GMTrackList::onPaint),
#if FOXVERSION < FXVERSION(1,7,11)
  FXMAPFUNC(SEL_UPDATE,GMTrackList::ID_HEADER_CHANGE,GMTrackList::onUpdHeader),
  FXMAPFUNC(SEL_COMMAND,GMTrackList::ID_HEADER_CHANGE,GMTrackList::onCmdHeader),
  FXMAPFUNC(SEL_CLICKED,GMTrackList::ID_HEADER_CHANGE,GMTrackList::onCmdHeaderClicked),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,GMTrackList::ID_HEADER_CHANGE,GMTrackList::onCmdHeaderRightBtnRelease),
#else
  FXMAPFUNC(SEL_UPDATE,GMTrackList::ID_HEADER,GMTrackList::onUpdHeader),
  FXMAPFUNC(SEL_COMMAND,GMTrackList::ID_HEADER,GMTrackList::onCmdHeader),
  FXMAPFUNC(SEL_CLICKED,GMTrackList::ID_HEADER,GMTrackList::onCmdHeaderClicked),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE,GMTrackList::ID_HEADER,GMTrackList::onCmdHeaderRightBtnRelease),
#endif
  };

FXIMPLEMENT(GMTrackList,FXIconList,GMTrackListMap,ARRAYNUMBER(GMTrackListMap))


GMTrackList::GMTrackList(){
  }

// File List
GMTrackList::GMTrackList(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h) : FXIconList(p,tgt,sel,opts,x,y,w,h) {
  rowColor=backColor;
  activeColor=backColor;
  activeTextColor=textColor;
  activeIcon=NULL;
  selectedIcon=NULL;
  activeFont=font;
  activeItem=-1;
  sortMethod=HEADER_DEFAULT;
  }

GMTrackList::~GMTrackList(){
  delete activeIcon;
  delete selectedIcon;
  }


FXIconItem * GMTrackList::createItem(const FXString&,FXIcon *,FXIcon*,void*) {
  return new GMTrackItem();
  }

void GMTrackList::setUnsorted() {
  sortMethod=HEADER_DEFAULT;
  sortfunc=NULL;
  }


void GMTrackList::create() {
  FXIconList::create();

  if (activeIcon==NULL){
    activeIcon = new FXPNGIcon(getApp(),x16_status_playing_png);
    activeIcon->blend(activeColor);
    activeIcon->create();
    }

  if (selectedIcon==NULL){
    selectedIcon = new FXPNGIcon(getApp(),x16_status_playing_white_png);
    selectedIcon->blend(getSelBackColor());
    selectedIcon->create();
    }

  }


// Replace recompute with cheaper one...
void GMTrackList::measure(){
  register FXint ih=15,th=4+font->getFontHeight();

  itemWidth=header->getTotalSize();
  itemHeight=FXMAX(ih,th);
  itemSpace=itemWidth;

  nrows=items.no();
  ncols=1;

  flags&=~FLAG_RECALC;
  }

FXint GMTrackList::getContentWidth() {
  if(flags&FLAG_RECALC) measure();
  return header->getTotalSize();
  }

/// Return content height
FXint GMTrackList::getContentHeight() {
  if(flags&FLAG_RECALC) measure();
  return nrows*itemHeight;
  }

void GMTrackList::appendHeader(const FXString & label,FXint size,GMColumn * data){
	GMColumn * c;
	for (FXint i=0;i<header->getNumItems();i++){
		c = getHeaderData(i);
		if (data->index < c->index){
			header->insertItem(i,label,NULL,size,data);
			return;
			}
		}
	header->appendItem(label,NULL,size,data);
  }


void GMTrackList::clearHeaders() {
  header->clearItems();
  }


void GMTrackList::saveHeaders() {
  GMColumn * column = NULL;
  for (FXint i=0;i<header->getNumItems();i++){
    column = getHeaderData(i);
    FXASSERT(column);
    column->size  = header->getItemSize(i);
		//column->index = i;
    }
  }


/// Set the current playing color
void GMTrackList::setActiveColor(FXColor c) {
  activeColor=c;
  if (xid) {
    delete activeIcon;
    activeIcon = new FXPNGIcon(getApp(),x16_status_playing_png);
    activeIcon->blend(activeColor);
    activeIcon->create();
    }
  update();
  }


long GMTrackList::onCmdHeaderClicked(FXObject*,FXSelector,void* ptr){
  register FXint hi=(FXint)(FXival)ptr;
  register FXint i,tw,w,nw=0,type;
  FXuint justify;
  FXint max;
  const FXString * textptr;
  FXString text;

  type=getHeaderData(hi)->type;

  for(i=0;i<items.no();i++){
    w=0;
    textptr=((GMTrackItem*)items[i])->getColumnData(type,text,justify,max);
    if (justify) {
      w=max+SIDE_SPACING+2;
      }
    else if (textptr && !textptr->empty()){
      tw=font->getTextWidth(*textptr);
      w=tw+SIDE_SPACING+2;
      }
    if(w>nw) nw=w;
    }

  if (hi==0) {
    nw+=ICON_WIDTH+DETAIL_TEXT_SPACING+SIDE_SPACING/2;
    }

  // Set new header width
  if(nw>0 && nw!=header->getItemSize(hi)){
    header->setItemSize(hi,nw);
    flags&=~FLAG_RECALC;
    }
  return 1;
  }


long GMTrackList::onCmdHeader(FXObject*,FXSelector,void*ptr){
  GMColumn * data = getHeaderData((FXuint)(FXival)ptr);
  if (data) {
    if (data->type==sortMethod) {
      if (sortfunc==data->ascending)
        sortfunc=data->descending;
      else
        sortfunc=data->ascending;
      }
    else {
      sortMethod=data->type;
      sortfunc=data->ascending;
      }
    if (sortfunc) 
      GMPlayerManager::instance()->getTrackView()->sortTracks();

      //sortItems();
    }
  return 1;
  }

long GMTrackList::onUpdHeader(FXObject*,FXSelector,void*){
  GMColumn * data;
  for (FXint i=0;i<header->getNumItems();i++){
    data = getHeaderData(i);
    if (data && sortMethod==data->type) {
#if FOXVERSION < FXVERSION(1,7,0)
      if (sortfunc==data->ascending)
        header->setArrowDir(i,FALSE);
      else
        header->setArrowDir(i,TRUE);
      }
    else {
      header->setArrowDir(i,MAYBE);
      }
#else
      if (sortfunc==data->ascending)
        header->setArrowDir(i,FXHeaderItem::ARROW_DOWN);
      else
        header->setArrowDir(i,FXHeaderItem::ARROW_UP);
      }
    else {
      header->setArrowDir(i,FXHeaderItem::ARROW_NONE);
      }
#endif
    }
  return 1;
  }

long GMTrackList::onCmdHeaderRightBtnRelease(FXObject*,FXSelector,void*ptr){
	if (target) target->handle(this,FXSEL(SEL_RIGHTBUTTONRELEASE,message+1),ptr);
	return 1;
	};

long GMTrackList::onPaint(FXObject*,FXSelector,void*ptr){
  register FXint rlo,rhi,dw,index,vw,y;
  FXEvent* event=(FXEvent*)ptr;
  FXDCWindow dc(this,event);

  // Draw nothing
  if (header->getNumItems()==0) {
    dc.setForeground(backColor);
    dc.fillRectangle(0,0,width,height);
    return 1;
    }

  GMTrackItem::max_time = font->getTextWidth("88:88",5);

  // Set font
  dc.setFont(font);

  // Calculate stipple width
  dw=font->getTextWidth(UTF8_ELLIPSIS,3);

  // Exposed rows
  rlo=(event->rect.y-pos_y-header->getDefaultHeight())/itemHeight;
  rhi=(event->rect.y+event->rect.h-pos_y-header->getDefaultHeight())/itemHeight;
  if(rlo<0) rlo=0;
  if(rhi>=items.no()) rhi=items.no()-1;

#if FOXVERSION < FXVERSION(1,7,0)
  vw = getViewportWidth();
#else
  vw = getVisibleWidth();
#endif

  // Repaint the items
  y=pos_y+rlo*itemHeight+header->getDefaultHeight();
  for(index=rlo; index<=rhi; index++,y+=itemHeight){
    if (activeItem==index) {
      dc.setForeground(activeColor);
      dc.setFont(activeFont);
      }
    else if (index%2)
      dc.setForeground(rowColor);
    else
      dc.setForeground(backColor);

    ((GMTrackItem*)items[index])->draw(this,dc,index,pos_x,y,vw,itemHeight,dw);

    if (activeItem==index)
      dc.setFont(font);

    }

  // Background below
  y=pos_y+(rhi+1)*itemHeight+header->getDefaultHeight();
  if(y<event->rect.y+event->rect.h){
    dc.setForeground(backColor);
    dc.fillRectangle(event->rect.x,y,event->rect.w,event->rect.y+event->rect.h-y);
    }
  return 1;
  }



FXint GMTrackItem::max_time = 0;

FXIMPLEMENT(GMTrackItem,FXIconItem,NULL,0);

GMTrackItem::GMTrackItem() : FXIconItem(FXString::null) {
  }

GMTrackItem::~GMTrackItem(){
  bigIcon=NULL;
  miniIcon=NULL;
  }

FXint GMTrackItem::hitItem(const FXIconList*,FXint,FXint,FXint,FXint) const{
  return 2;
  }

void GMTrackItem::draw(const GMTrackList* list,FXDC& dc,FXint index,FXint x,FXint y,FXint w,FXint h,FXint dw) const {
  register FXFont *font=(index==list->activeItem) ? list->activeFont : list->getFont();
  register FXHeader *header=list->getHeader();
  register FXint iw=10,ih=15,tw=0,th=0,yt,hi,drw,space,used,xx,type;
  FXString text;
  const FXString * textptr;
  FXint max=50;
  FXuint justify;
  FXIcon * icon=NULL;

  if(header->getNumItems()==0) return;

  /// Draw background
  if(isSelected()){
    if (list->activeItem==index) icon = list->selectedIcon;
    dc.setForeground(list->getSelBackColor());
    dc.fillRectangle(0,y,w,h);
    }
  else{
    if (list->activeItem==index) icon = list->activeIcon;
    dc.fillRectangle(0,y,w,h);
    }

  /// Draw Focus
  if(hasFocus()){
    dc.drawFocusRectangle(x+1,y+1,list->getHeader()->getTotalSize()-2,h-2);
    }

  /// Draw Icon
  xx=x+SIDE_SPACING/2;
  if(icon){
    FXASSERT(iw==icon->getWidth());
    FXASSERT(ih==icon->getHeight());
    dc.setClipRectangle(x,y,header->getItemSize(0),h);
    dc.drawIcon(icon,xx,y+(h-ih)/2);
    dc.clearClipRectangle();
    }
  xx+=iw+DETAIL_TEXT_SPACING;

  /// Draw Text
  th=font->getFontHeight();
  yt=y+(h-th-4)/2;
  if(!isEnabled())
    dc.setForeground(makeShadowColor(list->getBackColor()));
  else if(isSelected())
    dc.setForeground(list->getSelTextColor());
  else
    dc.setForeground(list->getTextColor());


  used=iw+DETAIL_TEXT_SPACING+SIDE_SPACING/2;
#if FOXVERSION < FXVERSION(1,7,0)
  for(hi=0;hi<header->getNumItems()&& xx<=w; hi++){
#else
  for(hi=0;hi<header->getNumItems()&& xx<=w+list->getVisibleX(); hi++){
#endif
    space=header->getItemSize(hi)-used;
#if FOXVERSION < FXVERSION(1,7,0)
    if (xx+space>=0){
#else
    if (xx+space>=list->getVisibleX()){
#endif
      type=list->getHeaderData(hi)->type;
      textptr=getColumnData(type,text,justify,max);
      if (textptr) {
        drw=textptr->length();
        tw=font->getTextWidth(*textptr);
        if(tw>space-4){
          while((tw=font->getTextWidth(textptr->text(),drw))+dw>space-4 && drw>1) drw=textptr->dec(drw);
          dc.setClipRectangle(xx,y,space,h);
          dc.drawText(xx+2,yt+font->getFontAscent()+2,textptr->text(),drw);
          dc.drawText(xx+tw+2,yt+font->getFontAscent()+2,UTF8_ELLIPSIS,3);
          dc.clearClipRectangle();
          }
        else if (justify==0 || (justify && max>space)) {
          dc.drawText(xx+2,yt+font->getFontAscent()+2,textptr->text(),drw);
          }
        else if (justify==COLUMN_JUSTIFY_LEFT_RIGHT_ALIGNED) {
          dc.drawText(xx+2+max-tw,yt+font->getFontAscent()+2,text.text(),drw);
          }
        else if (justify==COLUMN_JUSTIFY_CENTER_RIGHT_ALIGNED){
          dc.drawText(xx+((space/2)-(max/2))+(max-tw),yt+font->getFontAscent()+2,textptr->text(),drw);
          }
        else {
          dc.drawText(xx+2,yt+font->getFontAscent()+2,textptr->text(),drw);
          }
        }
      }
    xx+=space;
    used=0;
    }
  }


