/*******************************************************************************
*                         Goggles Music Manager                                *
********************************************************************************
*           Copyright (C) 2006-2008 by Sander Jansen. All Rights Reserved      *
*                               ---                                            *
* This program is free software: you can redistribute it and/or modify         *
* it under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License, or            *
* (at your option) any later version.                                          *
*                                                                              *
* This program is distributed in the hope that it will be useful,              *
* but WITHOUT ANY WARRANTY; without even the implied warranty of               *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
* GNU General Public License for more details.                                 *
*                                                                              *
* You should have received a copy of the GNU General Public License            *
* along with this program.  If not, see http://www.gnu.org/licenses.           *
********************************************************************************/
#include <fx.h>
#include "GMWizard.h"

const FXchar welcometext0[]="Welcome to Goggles Music Manager!";

const FXchar welcometext1[]=
"In order to enjoy your music collection, you need\n"
"to import your music into the music library. Please\n"
"specify below which directory Goggles Music Manager\n"
"should scan. For more advanced import options, you may\n"
"wish to skip this step and setup the import options\n"
"in the preferences panel first.";

const FXchar welcometext2[]=
"To skip this step and lauch Goggles Music Manager\n"
"please press cancel.";

// Define Message Map
FXDEFMAP(FXWizardDialog) FXWizardDialogMap[]={
  //____Message_Type______________ID_______________Message_Handler___
  FXMAPFUNC(SEL_COMMAND,FXWizardDialog::ID_SELECT_DIR,FXWizardDialog::onCmdSelectDir)
  };

// Object implementation
FXIMPLEMENT(FXWizardDialog,FXDialogBox,FXWizardDialogMap,ARRAYNUMBER(FXWizardDialogMap))

FXWizardDialog::FXWizardDialog(FXApp * app,FXIcon * icon) : FXDialogBox(app,"Goggles Music Manager",DECOR_ALL,0,0,0,0,0,0,0,0,0,0), dt_directory(directory) {
  setup(icon);
  }

FXWizardDialog::FXWizardDialog(FXWindow* owner,FXIcon * icon) : FXDialogBox(owner,"Goggles Music Manager",DECOR_TITLE|DECOR_BORDER,0,0,0,0,0,0,0,0,0,0), dt_directory(directory){
  setup(icon);
  }

FXWizardDialog::~FXWizardDialog(){
  //delete titlefont;
  }

void FXWizardDialog::setDirectory(const FXString & d){
  directory = d;
  }

FXString FXWizardDialog::getDirectory() const {
  return directory;
  }


void FXWizardDialog::setup(FXIcon * icon){
  const FXchar * musicdir[]={
    "Music",
    "music",
    "mp3",
    "MP3",
    "Mp3",
    "ogg",
    "OGG",
    "Ogg",
#ifndef DISABLE_ASF_SUPPORT
    "mp4",
    "MP4",
    "Mp4",
#endif
    NULL
    };

  FXString test;
  FXString home=FXSystem::getHomeDirectory();
  for (int i=0;musicdir[i]!=NULL;i++){
    test = home + PATHSEPSTRING + musicdir[i];
    if (FXStat::exists(test) && FXStat::isDirectory(test)){
      directory=test;
      break;
      }
    }
  if (directory.empty()) directory=home;


/*  FXFontDesc d;
  getApp()->getNormalFont()->getFontDesc(d);
  d.slant  = FXFont::Italic;
  d.weight = FXFont::Bold;
  titlefont = new FXFont(getApp(),d);
  titlefont->create();
*/
  FXLabel * label;

  FXHorizontalFrame *closebox=new FXHorizontalFrame(this,LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,0,0,0,0);
  new FXButton(closebox,tr("&Scan Music"),NULL,this,FXDialogBox::ID_ACCEPT,BUTTON_INITIAL|BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  new FXButton(closebox,tr("&Cancel"),NULL,this,FXDialogBox::ID_CANCEL,BUTTON_DEFAULT|LAYOUT_RIGHT|FRAME_RAISED|FRAME_THICK,0,0,0,0, 20,20);
  new FXSeparator(this,LAYOUT_SIDE_BOTTOM|SEPARATOR_GROOVE|LAYOUT_FILL_X);
  FXVerticalFrame * logoframe = new FXVerticalFrame(this,LAYOUT_FILL_Y|LAYOUT_SIDE_LEFT,0,0,0,0,0,0,0,0,0,0);
  label = new FXLabel(logoframe,FXString::null,icon,LAYOUT_FILL_Y|JUSTIFY_LEFT|TEXT_AFTER_ICON|JUSTIFY_TOP,0,0,0,0,0,0,0,0);
  label->setBackColor(FXRGB(255,255,255));
  new FXSeparator(this,SEPARATOR_GROOVE|LAYOUT_SIDE_LEFT|LAYOUT_FILL_Y);
  FXVerticalFrame * frame = new FXVerticalFrame(this,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,5,5,5,5,10,10);
//  label = new FXLabel(frame,welcometext0,NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
//  label->setFont(titlefont);
  new FXLabel(frame,welcometext1,NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
  FXVerticalFrame * vframe = new FXVerticalFrame(frame,LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0,0,0,0,0);
  new FXLabel(vframe,"Scan for music in directory:",NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
  FXHorizontalFrame * hframe = new FXHorizontalFrame(vframe,LAYOUT_FILL_X,0,0,0,0,0,0,0,0);
  new FXTextField(hframe,20,&dt_directory,FXDataTarget::ID_VALUE,LAYOUT_FILL_X|TEXTFIELD_NORMAL);
  new FXButton(hframe,"...",NULL,this,ID_SELECT_DIR,BUTTON_NORMAL);
  new FXLabel(frame,welcometext2,NULL,LAYOUT_FILL_X|JUSTIFY_LEFT);
  }

long FXWizardDialog::onCmdSelectDir(FXObject*,FXSelector,void*){
  FXDirDialog dialog(this,"Select Directory");
  dialog.setDirectory(directory);
  if (dialog.execute()){
    directory =  dialog.getDirectory();
    }
  return 1;
  }
